/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __HW_BOARD_PARAMETERS_DEFINE_H__
#define __HW_BOARD_PARAMETERS_DEFINE_H__

/*!
* @file hwboard_parameters_define.h
*
* @brief This file provides Hardware board parameter define.
*
*/

/* ===========================================  Includes  =========================================== */

/* ============================================  Define  ============================================ */
#define ACTIVE_LOW      0
#define ACTIVE_HIGH     1
#define TURN_ON         1
#define TURN_OFF        0
#define ACTIVE          1
#define INACTIVE        0

/*!
* CAUTION!
* !!!!!! AC7801x MotorAPP demo board ONLY SUPPORTS GATE_DRIVER_HIGH_HIGH MODE !!!!!!
*
* Gate Driver (Predrive) Mode Selection
*
* HIGH_TRUE / ACTIVE_HIGH:
* in UP_DOWN_COUNT mode, if the PWM count value is larger than the comparison value calculated by SVPWM,
* output of upper arm PWM channel is high level, and the output of lower arm PWM channel is complementary with upper channel.
                      /\      /\
                Cnt__/  \____/  \__
                 \  /    \  /    \
                  \/      \/      \/
PWM upper channel    |--|    |--|
                 ____|  |____|  |___

PWM lower channel ---|  |----|  |---
                     |__|    |__|

* LOW_TRUE / ACTIVE_LOW:
* in UP_DOWN_COUNT mode, if the PWM count value is larger than the comparison value calculated by SVPWM,
* output of upper arm PWM channel is low level, and the output of lower arm PWM channel is complementary with upper channel.
                      /\      /\
                Cnt__/  \____/  \__
                 \  /    \  /    \
                  \/      \/      \/
PWM upper channel ---|  |----|  |---
                     |__|    |__|

PWM lower channel    |--|    |--|
                 ____|  |____|  |___
*/

/*!
* Gate driver output the same level as MCU PWM module to drive MOSFET.
* PWM level_mode can be setted as ACTIVE_HIGH or ACTIVE_LOW,
* either of these, with AC7801x demo board.
*/
#define GATE_DRIVER_HIGH_HIGH

/*!
* Gate driver keeps the level of upper MCU PWM channel, and reverses the level of the lower MCU PWM channel.
* PWM level_mode can be setted as ACTIVE_HIGH.
*/
//#define GATE_DRIVER_HIGH_LOW

/*!
* Gate driver reverses the level of the upper MCU PWM channel, and keeps the level of lower MCU PWM channel.
* PWM level_mode can be setted as ACTIVE_LOW.
*/
//#define GATE_DRIVER_LOW_HIGH

/*!
* Gate driver reverses both level of the upper and lower MCU PWM channels.
* PWM level_mode can be setted as ACTIVE_LOW.
*/
//#define GATE_DRIVER_LOW_LOW

/*!
* PWM level mode and polarity.
* Level mode of PWM, ACTIVE_HIGH: Upper arm is Active_High; ACITVE_LOW: Upper arm is Active_Low.
*/
#if (defined GATE_DRIVER_HIGH_HIGH)
#define PHASE_UVW_POLARITY              ACTIVE_HIGH
//#define PHASE_UVW_POLARITY            ACTIVE_LOW
#elif (defined GATE_DRIVER_HIGH_LOW)
#define PHASE_UVW_POLARITY              ACTIVE_HIGH
#elif ((defined GATE_DRIVER_LOW_HIGH) || (defined GATE_DRIVER_LOW_LOW))
#define PHASE_UVW_POLARITY              ACTIVE_LOW
#endif

#define HW_DEAD_TIME_NS              100            /*!< Dead-time by Pre-driver Module,Unit:ns. 100 means 100ns = 0.1us */
#define SW_DEAD_TIME_NS              1000           /*!< Dead-time by MCU-PWM Module,Unit:ns. 1000 means 1000ns = 1us */

#define SAMPLE_TRIG_EDGE             0              /*!< SINGLE_SHUNT_SAMPLE method sample trig edge choose, 0 means Up edge sample, 1 means Down edge sample */
#define SAMPLE_HOLD_TIME             2000           /*!< Sampling hold time,Unit:ns. 500 means 500ns = 0.5us*/
#define SAMPLE_RISE_TIME             1000           /*!< Sampling rise time,Unit:ns. 1000 means 1000ns = 1us*/

#define VBUS_ATTENUATE_FACTOR        (float)(24.0)  /*!< Voltage attenuation ratio by Vbus voltage sensing */

#define MAX_BUS_VOLTAGE              30             /*!< Maximum operating voltage of the controller.Unit:V */
#define NORMAL_BUS_VOLTAGE           24             /*!< Normal operating voltage of the controller.Unit:V */
#define MIN_BUS_VOLTAGE              9              /*!< Minimum operating voltage of the controller.Unit:V */

/*!
* Current sensing section define.
* There are four sampling methods for user to choose: current sensor sample(ICS); single shunt sample; two shunts sample; three shunts sample.
* According to the hardware circuir, the user set CUR_SAMPLE_MODE to select the corresponding samplle mode.
*/
#define ICS_SENSORS_SAMPLE           0                      /*!< Current sampling with current sensor */
#define SINGLE_SHUNT_SAMPLE          1                      /*!< Current sampling with single shunt */
#define TWO_SHUNT_SAMPLE             2                      /*!< Current sampling with two shunts */
#define THREE_SHUNT_SAMPLE           3                      /*!< Current sampling with three shunts */

#define CUR_SAMPLE_MODE              TWO_SHUNT_SAMPLE     /*!< samplle mode select */
#define RSHUNT                       (float)(0.005)         /*!< Sampling resistance, unit: Ohm */
#define ICS_GAIN                     (float)(0.50)          /*!< current sensor sample gain, unit: V/A */


/* Amplification gain and polarity for shunts based sensing */
#define OP_AMPLIFICATION_GAIN        (float)(7.5)           /*!< amplification gain, default positive, reverse amplification need turn negative*/

/* Divide coefficient of phase voltage */
#define PHASE_VOLTAGE_DIVIDE_COEFF   (float)(23.7)          /*!< Divide coefficient of phase voltage, for bldc sensorless software detect bemf way*/

#define HW_MAX_CURRENT               8                      /*!< Maximum operating current of the hardware board, used to indicate the maximum current of hardwarecircuit, unit: A */
#define ADC_RANGE                    4096                   /*!< ADC sampling accuracy of MCU ,4096 means the ADC accuracy is 12bit*/
#define VSVREF                       (float)(5.0)           /*!< ADC sampling voltage range of MCU  */

#define OVERCUR_FEEDBACK_POLARITY    PWM_INPUT_POLARITY_ACTIVE_LOW     /*!< Polarity setting of over current protection */

#define LED_TURN_ON                 0U
#define LED_TURN_OFF                1U

/* ===========================================  Typedef  ============================================ */

#endif /* __HW_BOARD_PARAMETERS_DEFINE_H__ */

/* =============================================  EOF  ============================================== */
