/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __PROTECTOR_PARAMETERS_DEFINE_H__
#define __PROTECTOR_PARAMETERS_DEFINE_H__
/*!
* @file protector_parameters_define.h
*
* @brief This file provides motor protection functions configuration parameters definition.
*
*/

/* ===========================================  Includes  =========================================== */

/* ============================================  Define  ============================================ */
/*!
* @brief Definition of bus overvoltage and undervoltage fault protection parameters.
*/
#define VBUS_OVERVOLTAGE                        DIAGNOSTIC_FAIL             /*!< Report status fault */
#define VBUS_NORMAL                             DIAGNOSTIC_OK               /*!< Report status ok */
#define VBUS_UNDERVOLTAGE                       DIAGNOSTIC_FAIL             /*!< Report status fault */
#define VBUS_RECOVER                            DIAGNOSTIC_UNKNOWN          /*!< Report status fault */

#define VBUS_UNDERVOLTAGE_PROTECTION_ENABLE                                 /*!< Bus voltage undervoltage fault protection enable */
#define VBUS_OVERVOLTAGE_PROTECTION_ENABLE                                  /*!< Bus voltage overvoltage fault protection enable */

#define UNDERVOLTAGE_RECOVERY_VOLTAGE           (MIN_BUS_VOLTAGE + 2)       /*!< Bus voltage undervoltage fault recovery voltage, 2 means to recover
                                                                            *    from the fault when the bus voltage is 2V higher than the undervoltage point.
                                                                            */
#define OVERVOLTAGE_RECOVERY_VOLTAGE            (MAX_BUS_VOLTAGE - 2)       /*!< Bus voltage overvoltage fault recovery voltage, 2 means to recover
                                                                            *    from the fault when the bus voltage is 2V lower than the overvoltage point.
                                                                            */

#define VBUS_UNDERVOLTAGE_THRESHOLD             (MIN_BUS_VOLTAGE * 10)                 /*!< 18V * 10, real vbus value enlarges 10 times, action threshold of undervoltage protection */
#define VBUS_UNDERVOLTAGE_RECOVERY_THRESHOLD    (UNDERVOLTAGE_RECOVERY_VOLTAGE * 10)   /*!< 20V * 10, real value enlarges 10 times, action threshold of undervoltage recovery */
#define VBUS_OVERVOLTAGE_THRESHOLD              (MAX_BUS_VOLTAGE * 10)                 /*!< 30V * 10, real vbus value enlarges 10 times, action threshold of overvoltage protection */
#define VBUS_OVERVOLTAGE_RECOVERY_THRESHOLD     (OVERVOLTAGE_RECOVERY_VOLTAGE * 10)    /*!< 28V * 10, real value enlarges 10 times, action threshold of overvoltage recovery */

/*!
* @brief Definition of phase current protection parameters.
*/
//#define PHASE_CURRENT_PROTECTION_ENABLE                                   /*!< Phase current overcurrent protection enable */
#define MAX_IPHASE_THRESHOLD                    Math_IQ(0.95)               /*!< Phase current overcurrent protection threshold, refer to current base value */
#define IPHASE_DBC                              10                          /*!< Phase current overcurrent protection operation period value in ms unit, 10 = 10ms */

/*!
* @brief Definition of bus current protection parameters.
*/
//#define BUS_OVERCURRENT_PROTECTION_ENABLE                                 /*!< Bus current overcurrent protection enable */
#define BUS_OVERCURRENT_THRESHOLD               200                         /*!< Bus current overcurrent protection threshold, 100 timers larger than true value, 200 means 2A*/
#define BUS_OVERCURRENT_DBC                     10                          /*!< Bus current overcurrent protection operation period value in ms unit, 10 = 10ms */

/*!
* @brief Definition of sensorless control stall protection parameters.
*/
//#define STALL_PROTECTION_SENSORLESS_ENABLE                                /*!< Sensorless control stall protection enable */
#define BEMF_CONSISTENT_THRESHOLD               2                          /*!< Maxiumum accepted back-EMF on speed estimates (percentage) */
#define BEMF_DBC                                20                         /*!< Sensorless stall protection operation period value on back-EMF method in ms unit, 500 = 500ms*/

/*!
* @brief Definition of Hall sensor control stall protection parameters.
*/
//#define STALL_PROTECTION_SENSOR_ENABLE                                    /*!< Hall sensor control stall protection enable */
#define STALL_SPD_THRESHOLD                     20                          /*!< Stall judge speed, true */
#define STALL_CUR_THRESHOLD                     Math_IQ(0.3)                /*!< Stall judge current, pu, refer to current base value*/
#define STALL_DBC                               500                         /*!< Hall sensor stall protection operation period value in ms unit, 500 = 500ms */

/*!
* @brief Definition of lose phase protection parameters.
*/
//#define LOSE_PHASE_PROTECTION_ENABLE                                        /*!< Lose phase protection enable */
#define LOSE_PHASE_THRESHOLD                    Math_IQ(0.025)              /*!< Lose phase protection operation current threshold value, refer to current base value */
#define LOSE_PHASE_DBC                          3                           /*!< Lose phase protection operation period value in ms unit, 3 = 3ms */
#define LOSE_PHASE_ADD_CNT                      6                           /*!< Lose phase protection operation add count, 6 means 2^6=64 */
#define LOSE_PHASE_JUDGE_THRESHOLD              Math_IQ(3.0)                /*!< Lose phase protection operation Imax/Imin ratio judge threshold value */
/*!
* @brief Definition of phase current short-circuit protection parameters.
*/
//#define PHASE_SHORT_PROTECTION_ENABLE                                       /*!< Phase current short-circuit protection enable */
#define SPEED_THRESHOLD_SHORT                   Math_IQ(1.0 / BASE_FREQ)    /*!< Phase current short-circuit protection speed judge threshold value, refer to base frequency */
#define JUDGE_THRESHOLD_SHORT                   5                           /*!< Phase current short-circuit protection current judge threshold value */
#define IPHASE_SHORT_DBC                        10                          /*!< Phase current short-circuit protection operation period value in ms unit, 10 = 10ms */

/*!
* @brief Definition of MOSFET bridge protection parameters.
*/
#define SHORT_THRESHOLD_DUTY                    2                           /*!< Short circuit protection duty threshold value */
#define SHORT_THRESHOLD_CUR                     Math_IQ(0.1)                /*!< Short circuit protection current threshold value */
#define OPEN_THRESHOLD_DUTY                     500                         /*!< Open circuit protection duty threshold value */
#define OPEN_THRESHOLD_CUR                      Math_IQ(0.02)               /*!< Open circuit protection current threshold value */
#define DUTY_STEP                               1                           /*!< MOSFET protection duty calculation step value */

/*!
* @brief Definition of DC Bus current limiting protection parameters.
*/
//#define DCCUR_LIMIT_ENABLE                                                /*!< DC Bus current limiting protection enable */
#define DCCUR_H                                 575                         /*!< DC Bus current upper limit, true value increase 100 times */
#define DCCUR_L                                 520                         /*!< DC Bus current lower limit, true value increase 100 times */
#define DCCUR_CHECK_TIME                        200                         /*!< DC Bus current limiting protection operation period value, 200ms */

/*!
* @brief Definition of Input power limiting protection parameters.
*/
//#define POWER_LIMIT_ENABLE                                                /*!< Input power limiting protection enable */
#define POWER_H                                 2100                        /*!< Input power upper limit, true value increase 10 times */
#define POWER_L                                 1800                        /*!< Input power lower limit, true value increase 10 times */
#define POWER_CHECK_TIME                        200                         /*!< Input power limiting protection operation period value, 200ms */

/*!
* @brief Definition of lose speed protection parameters.
*/
//#define LOSE_SPEED_PROTECTION_ENABLE                                      /*!< Lose speed protection enable */
#define LOSE_SPEED_THRESHOLD                    Math_IQ(0.99)               /*!< Frequency threshold of lose speed protection */
#define LOSE_SPEED_ERR_THRESHOLD                Math_IQ(0.1)                /*!< Frequency error threshold of lose speed protection */
#define LOSE_SPEED_DBC                          10                          /*!< Lose speed protection operation period value in ms unit, 2 = 2ms */

/*!
* @brief Definition of zero speed check parameters.
*/
//#define ZERO_SPEED_CHECK_ENABLE                                           /*!< Zero speed protection enable */
#define ZERO_SPEED_THRESHOLD                   Math_IQ(0.07)                /*!< Zero speed protection frequency threshold */
#define ZERO_SPEED_DBC                         2                            /*!< Zero speed protection operation period value in ms unit, 2 = 2ms */

/*!
* @brief Definition of startup check parameters.
*/
//#define STARTUP_CHECK_RESTART_ENABLE                                      /*!< Startup error restart protection enable */
#define RESTART_DELAY           2000                                        /*!< Delay cycles to judge restart */
#define RESTART_NUMBER          3                                           /*!< Restart number upper limit to report fault */
#define RESTART_JUDGE_TIME      30000                                       /*!< Restart time limit to report fault, unit: ms */

/*!
* @brief Over load protect parameters.
*/
//#define OVER_LOAD_PROTECT_ENABLE                                          /*!< Over load protection enable */
#define OL_RECOVER_CUR             Math_IQ(0.8)                             /*!< Over load protection recover current */
#define OL_JUDGE_CUR1              Math_IQ(0.85)                            /*!< Over load protection judge current1 */
#define OL_JUDGE_CUR2              Math_IQ(0.9)                             /*!< Over load protection judge current2 */
#define OL_JUDGE_CUR3              Math_IQ(0.98)                            /*!< Over load protection judge current3 */
#define HOLD_TO_RUN_TIME           30000                                    /*!< Over load protection HOLD state to RUN state time, unit: ms */
#define HOLD_TO_FALL_TIME          30000                                    /*!< Over load protection HOLD state to FALL state time, unit: ms */
#define OL_JUDGE_TIME1             30000                                    /*!< Over load protection judge time1, unit: ms */
#define OL_JUDGE_TIME2             20000                                    /*!< Over load protection judge time2, unit: ms */
#define OL_FALL_FREQ_COF           Math_IQ(0.8)                             /*!< Over load protection fall frequency coefficient, base on current feedback speed */

/*!
* @brief Definition of power reduced protection parameters.
*/
//#define POWER_DECREASE_PROTECT_ENABLE                                     /*!< Power reduced protection enable */
#define SPD_START                               Math_IQ(0.95)               /*!< Starting point of reduced power operation electric frequency, pu */
#define SPD_STOP                                Math_IQ(0.98)               /*!< Ending point of reduced power operation electric frequency, pu */

#define MOS_TEMP_START                          Math_IQ(0.9)                /*!< Starting point of reduced power operation MOSFET temperature, pu */
#define MOS_TEMP_STOP                           Math_IQ(0.95)               /*!< Ending point of reduced power operation MOSFET temperature, pu */
#define MOS_TEMP_BASE                           100                         /*!< MOSFET temperature base value, unit: Celsius degree */

#define IBUS_START                              Math_IQ(0.9)                /*!< Starting point of reduced power operation bus current, pu */
#define IBUS_STOP                               Math_IQ(0.95)               /*!< Ending point of reduced power operation bus current, pu */

#define VBUS_HIGH_START                         Math_IQ(1.16)               /*!< Starting point of reduced power operation over bus voltage pu value */
#define VBUS_HIGH_STOP                          Math_IQ(1.33)               /*!< Ending point of reduced power operation over bus voltage pu value */

#define VBUS_LOW_START                          Math_IQ(0.83)               /*!< Starting point of reduced power operation low bus voltage pu value */
#define VBUS_LOW_STOP                           Math_IQ(0.67)               /*!< Ending point of reduced power operation low bus voltage pu value */

#define POWER_DECREASE_FILTER_COEFF             Math_IQ(0.99)               /*!< Power reduced protection filter coefficient, only use in power_decrease cofficient increase */

#define HALL_SENSOR_CHECK_ENALBE
#define HALL_DISCONNECT_JUDGE                   20U
#define HALL_DISCONNECT_FREQ_THRESHOLD          Math_IQ(0.1)

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

/* ======================================  Functions define  ======================================== */

#endif /* __PROTECTOR_PARAMETERS_DEFINE_H__ */

/* =============================================  EOF  ============================================== */
