/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __BLDC_COMMON_H__
#define __BLDC_COMMON_H__
/*!
* @file bldc_common.h
*
* @brief This file provides the declaration of common variables.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_parameters_init.h"
#include "bldc_mcu_config.h"
#include "bldc_speed.h"
#include "bldc_protector.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Bldc startup status enumeration.
*/
typedef enum
{
    STARTUP_GOING = 0,  /*!< Starting */
    STARTUP_FAIL,       /*!< Start failure */
    STARTUP_READY,      /*!< Start success */
} BLDC_StartupStatus;

/* ==========================================  Variables  =========================================== */

/*!
* Motor parameters
*/
extern BLDC_MOTOR                  *pBldcMotorParam;           /*!< Motor parameters measurement*/

/*!
* Hardware parameters
*/
extern BLDC_HW                     *pBldcHWParam;         /*!< Hardware parameters of motor controller */

/*!
* Speed control
*/
extern SPEED_RAMP_TYPE              g_bldc_speedCmd;        /*!< BLDC Speed command ramp structure*/

extern BLDC_VARS_CTRL               g_bldcVarsCtrl;         /*!< BLDC control parameters structure */
extern BLDC_VARS                    g_bldcVars;             /*!< BLDC parameters structure */

/*!
* PID
*/
/* Acr */
extern PID_IN                       g_bldc_acrPIIn;         /*!< BLDC Current loop PID input structure */
extern PID_COF                      g_bldc_acrCof;          /*!< BLDC Current loop PID parameter structure */
extern PID_OUT                      g_bldc_acrPIOut;        /*!< BLDC Current loop PID output structure */

/* Asr */
extern PID_IN                       g_bldc_asrPIIn;         /*!< BLDC Speed loop PID input structure */
extern PID_COF                      g_bldc_asrCof;          /*!< BLDC Speed loop PID parameter structure */
extern PID_OUT                      g_bldc_asrPIOut;        /*!< BLDC Speed loop PID output structure */

/* PID structure */
extern PID_TYPE                     g_bldc_acrPid;          /*!< BLDC current loop PID regulator structure */
extern PID_TYPE                     g_bldc_asrPid;          /*!< BLDC Speed loop PID regulator structure */

extern BLDC_ADC_TYPE                g_bldc_adSample;        /*!< BLDC adc sample struct */
extern BLDC_HALL_TYPE               g_hallControl;          /*!< BLDC hall sensor control struct */
extern BLDC_SENSORLESS_TYPE         g_bemfControl;          /*!< BLDC sensorless bemf control struct */

/*!
* Configuration
*/
extern BLDC_VARS_CFG                *pBldcVarsCfg;          /*!< BLDC configuration parameters */
extern BLDC_PID_CFG                 *p_bldc_PidCfg;         /*!< BLDC PID regulator configuration parameters */

/*
* Sensorless Bemf Configuration
*/
extern BLDC_BEMF_CFG                *pBldcBemfCfg;          /*!< BLDC sensorless configuration parameters */

/*!
* Protector
*/
extern PROTECTOR_TYPE               g_bldc_protector;       /*!< BLDC protector parameters */

extern DATA_ACCESS_FLASH_CFG        *pEflashCfg;            /*!< Eflash access configuration parameters */

/*!
* Flash operation
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
extern DATA_ACCESS_FLASH            g_dataAccessFlash;      /*!< Variables access flash operation parameters structure */
#endif

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_COMMON_H__ */

/* =============================================  EOF  ============================================== */
