/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_PARAMETERS_INIT_H__
#define __BLDC_PARAMETERS_INIT_H__
/*!
* @file bldc_parameters_init.h
*
* @brief This file provides the common structure declaration.
*
*/

/* ===========================================  Includes  =========================================== */
#include "stdint.h"
#include "bldc_data_access_eflash.h"

/* ============================================  Define  ============================================ */
#define BUFFER_SIZE 8

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Common data storage buffer structure of BLDC.
*/
typedef struct
{
    uint16_t array[BUFFER_SIZE];                    /*!< Buffer array to storage noisy signal to be filtered */
    uint16_t bufferIndex;                           /*!< Buffer array storage index */
    uint16_t maxValue;                              /*!< Buffer array storage max value */
    uint16_t minValue;                              /*!< Buffer array storage min value */
} BUFFER_TYPE;

/*!
* @brief Adc sample parameter structure of BLDC.
*/
typedef struct
{
    uint16_t busCurOffset;                          /*!< Bus current zero offset in bldc control */
    int16_t  busCurAD;                              /*!< Bus current AD value in bldc control */
    int16_t  ibcPu;                                 /*!< Phase current B or C value in Pu unit */
    int16_t  iacPu;                                 /*!< Phase current A or C value in Pu unit */
    int16_t  iabPu;                                 /*!< Phase current A or B value in Pu unit */
    uint16_t busVoltageAD;                          /*!< Bus voltage AD value in bldc control */
    uint16_t phaseVoltageOffsetA;                   /*!< Phase voltage A value offset in bldc control */
    uint16_t phaseVoltageOffsetB;                   /*!< Phase voltage B value offset in bldc control */
    uint16_t phaseVoltageOffsetC;                   /*!< Phase voltage C value offset in bldc control */
    uint16_t phaseVoltageA;                         /*!< Phase voltage A value in bldc control */
    uint16_t phaseVoltageB;                         /*!< Phase voltage B value in bldc control */
    uint16_t phaseVoltageC;                         /*!< Phase voltage C value in bldc control */
    int16_t  motorFeedbackCur;                      /*!< Bus current AD feedback in bldc control */
    int16_t  motorFeedbackCurOld;                   /*!< Bus current AD last feedback in bldc control */
    uint16_t busCurTrue;                            /*!< Bus current true value in bldc control in 100 times, 100 menas 1A */
    uint16_t busVoltageTrue;                        /*!< Bus voltage true value in bldc control in 10 times, 10 menas 1V */
    uint16_t busPowerTrue;                          /*!< Bus power true value in bldc control in 10 times, 10 menas 1W */
    uint32_t busOverCurCnt;                         /*!< Bus current AD over peek current threshold times count */

    BUFFER_TYPE busCur;                             /*!< Buffer to storage bus current in bldc control */
} BLDC_ADC_TYPE;

/*!
* @brief Common parameter structure of BLDC hall control.
*/
typedef struct
{
    uint8_t  currentDriverVector;                   /*!< Current six step vector drive on MOS */
    uint16_t pwdtValid;                             /*!< Pwdt measure data valid check */
    uint16_t currentPhase;                          /*!< Bldc hall sector in bldc hall control */
    uint16_t averagePulseWidth;                     /*!< Bldc PDWT pulse average width in bldc hall control */
    BUFFER_TYPE totalPulseWidth;                    /*!< Bldc PDWT pulse total width in bldc hall control */
} BLDC_HALL_TYPE;

/*!
* @brief Common parameter structure of BLDC sensorless control.
*/
typedef struct
{
    uint8_t  commutateState;                        /*!< Bldc commutate status */
    uint8_t  currentDriverVector;                   /*!< Current six step vector drive on MOS */
    uint8_t  nextDriverVector;                      /*!< Next six step vector drive on MOS */
    uint8_t  prepositonIndex;                       /*!< Preposition times */
    uint16_t kickTimeThreshold;                     /*!< Time threshold during open kick */
    uint16_t undrivePhaseVoltage;                   /*!< Voltage of undrive phase */
    uint16_t forceChangePhaseNum;                   /*!< Force change phase number of times */
    uint16_t normalChangePhaseNum;                  /*!< Normal change phase number of times */
    uint32_t changePhaseTime;                       /*!< Current vector affect time count, units: TIMER_XUS us */
    uint32_t saveChangePhaseTime;                   /*!< Current vector affect time count saved for commutate delay, units: TIMER_XUS us */
    uint32_t lastChangePhaseTime;                   /*!< Last vector affect time count to determine after flow time, units: TIMER_XUS us */
    uint32_t afterFlowTime;                         /*!< After flow time while don't detect over zero point, units: TIMER_XUS us */
    uint32_t aveChangePhaseTime;                    /*!< Current vector affect average time count, units: TIMER_XUS us */
    uint32_t maxChangePhaseTime;                    /*!< Max current vector affect average time count, units: TIMER_XUS us */
    uint32_t minChangePhaseTime;                    /*!< Min current vector affect average time count, units: TIMER_XUS us */
    uint32_t delayTime;                             /*!< Bldc delay 30 degree elec angle time count, units: TIMER_XUS us */
    BUFFER_TYPE commutationTime;                    /*!< Buffer to storage commutate time */

    _Bool overZero_Flag;                            /*!< Bldc over zero point detect flag */
    _Bool afterFlow_Flag;                           /*!< Bldc after flow detect flag */
    _Bool forceChangePhaseFlag;                     /*!< Bldc force change phase detect flag */
    _Bool updateSpeedFlag;                          /*!< Bldc speed update flag */
} BLDC_SENSORLESS_TYPE;

/*!
* @brief PID input parameters struct.
*/
typedef struct
{
    int32_t refPu;          /*!< PID input reference value */
    int32_t fbkPu;          /*!< PID input feedback value */
    int32_t maxPu;          /*!< Parameter: upper saturation limit */
    int32_t minPu;          /*!< Parameter: lower saturation limit */
    int32_t deadZonePu;     /*!< Error deadband setting */
} PID_IN;

/*!
* @brief PID output parameters struct.
*/
typedef struct
{
    int32_t outPu;          /*!< PID output value */
    int32_t errPu;          /*!< Data: Err, errPu = refPu - fbkPu  */
    int32_t err1Pu;         /*!< Data: Err of last calculation period */
    int32_t err2Pu;         /*!< Data: Err of penultimate calculation period */
    int32_t upPu;           /*!< Data: proportional term */
    int32_t uiPu;           /*!< Data: integral term */
    int32_t v1Pu;           /*!< Data: pre-saturated controller output */
    int32_t i1Pu;           /*!< Data: integrator storage */
    int32_t w1Pu;           /*!< Data: saturation record */
    int32_t outErr;         /*!< Err of PID output */
    int32_t outPu1;         /*!< PID output before limiting amplitude */
} PID_OUT;

/*!
* @brief PID calculation coefficient struct.
*/
typedef struct
{
    int32_t kpPu;          /*!< Proportional gain of PID regulator */
    int32_t kiPu;          /*!< Integral gain of PID regulator */
    int32_t kcPu;          /*!< Reserve */
    int32_t kdPu;          /*!< Reserve */
} PID_COF;

/*!
* @brief PID struct.
*/
typedef struct
{
    PID_IN *pPidInput;          /*!< Input parameters struct of PID */
    PID_COF *pPidCoef;          /*!< Coefficient struct of PID */
    PID_OUT *pPidOutput;        /*!< Output parameters struct of PID */
} PID_TYPE;

/*!
* @brief Total bldc control structure, include hall bldc struct, sensorless bldc struct, and common parameter.
*/
typedef struct
{
    BLDC_HALL_TYPE           *pHall;                /*!< Bldc hall control struct pointer */
    BLDC_SENSORLESS_TYPE     *pBemf;                /*!< Bldc bemf control struct pointer */
    uint16_t                 bldc_startPwm;         /*!< Bldc start Mos PWM flag, Reserve for GATE_DRIVER_LOW_LOW PWM init channels enable */
    uint16_t                 bldcStartTime;         /*!< Count time during bldc startup */
    int16_t                  startupStatus;         /*!< Bldc startup state */
    uint16_t                 matchTriggerValue;     /*!< PWM match trigger ADC value */
    int16_t                  ibusRefPu;             /*!< Current loop reference */
    int16_t                  ibusFdkPu;             /*!< Current loop feedback */
    int16_t                  setDuty;               /*!< PWM manual setting duty */
    int16_t                  deDutyCoeff;           /*!< Decrease output duty coefficient */
    uint16_t                 pwmOutDuty;            /*!< PWM output duty */
    uint16_t                 pwmOutPu;              /*!< PWM output in Q15 format */
    uint16_t                 pwmLastOutPu;          /*!< Last PWM output in Q15 format */
} BLDC_VARS_CTRL;

/*!
* @brief Configuration parameter structure. Used to set control direction, pwm frequency, etc.
*/
typedef struct
{
    int16_t  motorDir;                              /*!< Motor direction */
    uint16_t controlMode;                           /*!< Motor Control Mode */
    int16_t  initDuty;                              /*!< Motor init starup duty */
    int16_t maxDuty;                                /*!< Motor max duty */
    int16_t minDuty;                                /*!< Motor min duty */

    int16_t speedMaxLimit;                          /*!< Max limit of speed command */
    int16_t speedMinLimit;                          /*!< Min limit of speed command */

    uint16_t pwmFreq;                               /*!< Motor pwm frequency */
    uint16_t curFreq;                               /*!< Motor current loop frequency */
    uint16_t speedFreq;                             /*!< Motor speed loop frequency */
    uint16_t msUnitInt;                             /*!< Ms count value */
    uint16_t baseFreq;                              /*!< Base frequency */
    uint16_t halfPwmPeriod;                         /*!< Half of pwm period */
    uint16_t pwmPeriod;                             /*!< PWM period */
    uint16_t pwmDeadTime;                           /*!< PWM dead time */
    uint16_t pwdtPrescal;                           /*!< PWDT prescaler val */
    uint16_t bemfNeutralCoef;                       /*!< Bemf neutral point threshold coefficient in BLDC_SENSORLESS BEMF_DETECT_SOFTWARE way */
    uint32_t sysBusClk;                             /*!< System bus clock */
    int32_t  motorAcceleration;                     /*!< Motor speed loop acceleration */
    int32_t  motorDeceleration;                     /*!< Motor speed loop deacceleration */
} BLDC_VARS_CFG;

/*!
* @brief Configuration parameter structure. Used to set PI parameters in different PID regulators.
*/
typedef struct
{
    int32_t currentKp;                              /*!< Current loop PID kp */
    int32_t currentKi;                              /*!< Current loop PID ki */
    int32_t currentKd;                              /*!< Current loop PID kd */
    int32_t maxCurrent;                             /*!< Current loop PID max */
    int32_t minCurrent;                             /*!< Current loop PID min */

    int32_t speedKp;                                /*!< Speed loop PID kp */
    int32_t speedKi;                                /*!< Speed loop PID ki */
    int32_t speedKd;                                /*!< Speed loop PID kd */
    int32_t maxSpeed;                               /*!< Speed loop PID max */
    int32_t minSpeed;                               /*!< Speed loop PID min */
} BLDC_PID_CFG;

typedef struct
{
    uint32_t statorRs;              /*!< Stator resistance (unit:1mohm) */
    uint32_t statorLs;              /*!< Stator inductance (unit:1uH) */
    uint32_t ld;                    /*!< Motor D-axis inductance (unit:1uH) */
    uint32_t lq;                    /*!< Motor q-axis inductance (unit:1uH) */
    uint32_t rotorRs;               /*!< Induction motor Rotor resistance (unit:1mohm) */
    uint32_t lm;                    /*!< Induction motor inductance (unit:1uH) */
    uint32_t lleak;                 /*!< Induction motor leakage inductance (unit:1uH) */

    uint32_t jm;                    /*!< Rotational intertia (unit:ukg*m2) */
    uint32_t polePairs;             /*!< Number of pole pairs (constant value) */
    uint32_t encoderLines;          /*!< Encoder pulse number per Motor (constant value) */

    uint32_t maxCur;                /*!< Maximum motor current of the motor (unit:1A,pk) */
    uint32_t ratedVoltage;          /*!< rated voltage of the HW board (unit:1V) */
    uint32_t maxSpeed;              /*!< Maximum speed of the motor (unit:1rpm) */
    uint32_t maxFreq;               /*!< Maximum speed of the motor (unit:HZ) */

    uint32_t maxTorque;             /*!< Maximum torque of the motor (unit:1mNm) */
    uint32_t maxPower;              /*!< Maximum power of the motor (unit:W) */

    float bEmf;                     /*!< Back emf coefficient (unit:Vrms/Krpm) */
} BLDC_MOTOR;

/*!
* @brief Hardware board parameter structure.
*/
typedef struct
{
    uint16_t pwmPolarity;           /*!< Pwm active polarity */

    uint16_t hwDeadTime;            /*!< Hardware dead time size,usually refers to the deadband size of the gate driver */
    uint16_t swDeadTime;            /*!< Software dead time size,usually refers to the deadband size set by mcu */

    uint16_t maxCurrent;            /*!< Maximum operating current of the controller */

    uint16_t maxVoltage;            /*!< Maximum operating voltage of the controller */
    uint16_t normalVoltage;         /*!< Normal operating voltage of the controller */
    uint16_t minVoltage;            /*!< Minimum operating voltage of the controller */

    uint16_t adcRange;              /*!< ADC sampling accuracy of MCU */
    uint16_t overCurrentPolarity;   /*!< Polarity setting of over current protection */

    float vbusAttenuateVal;         /*!< Voltage attenuation ratio */

    float rShunt;                   /*!< Resistance value for current samping */
    float opGainVal;                /*!< Amplification factor of operational amplifier when sampling current */
    float icsGainVal;               /*!< Amplification factor of ICS when sampling current */
//    uint16_t opGainPolarity;      /*!< Polarity of operational amplifier when sampling current */

    float adcRefV;                  /*!< ADC sampling voltage range of MCU  */
} BLDC_HW;

/*!
* @brief Sensorless Bemf parameter structure.
*/
typedef struct
{
    int16_t prepositionDutyMin;    /*!< Min duty while preposition in BLDC sensorless startup */
    int16_t prepositionDutyMax;    /*!< Max duty while preposition in BLDC sensorless startup */
    int16_t prepositionDutyStep;   /*!< Step duty once while preposition in BLDC sensorless startup */
    int16_t prepositionTimeMin;    /*!< Min time while preposition in BLDC sensorless startup(unit:ms) */
    int16_t prepositionTimeMax;    /*!< Max time while preposition in BLDC sensorless startup(unit:ms) */
    int16_t prepositionTimeStep;   /*!< Step time once while preposition in BLDC sensorless startup(unit:ms) */
    int16_t kickDutyMax;           /*!< Max duty while open kick in BLDC sensorless startup */
    int16_t kickDutyStep;          /*!< Step duty once while open kick in BLDC sensorless startup */
    int16_t kickTimeMin;           /*!< Min time while open kick in BLDC sensorless startup(unit:ms) */
    int16_t kickTimeMax;           /*!< Max time while open kick in BLDC sensorless startup(unit:ms) */
    int16_t kickTimeStep;          /*!< Step time once while open kick in BLDC sensorless startup(unit:ms) */
} BLDC_BEMF_CFG;

/*!
* @brief Bldc parameter structure.
*/
typedef struct
{
    BLDC_VARS_CFG *pCfg;                            /*!< Common configuration parameter structure of BLDC */
    BLDC_VARS_CTRL *pCtrl;                          /*!< Current loop structure */
} BLDC_VARS;

/* ==========================================  Variables  =========================================== */
extern BLDC_MOTOR              defBldcMotorParam;      /*!< Motor parameters */
extern BLDC_HW                 defBldcHwBoard;         /*!< System hardware parameters */

extern BLDC_VARS_CFG           defBldcVars;
extern BLDC_PID_CFG            defBldcPid;             /*!< PID regulator configuration parameters */

extern BLDC_BEMF_CFG           defBemfVars;            /*!< BLDC sensorless configuration parameters */

extern DATA_ACCESS_FLASH_CFG   defEflashAccessCfg;     /*!< Eflash access configuration parameters */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_PARAMETERS_INIT_H__ */

/* =============================================  EOF  ============================================== */
