/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_PROTECTOR_H__
#define __BLDC_PROTECTOR_H__
/*!
* @file bldc_protector.h
*
* @brief This file provides motor protection functions interfaces in bldc control.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>

/* ============================================  Define  ============================================ */

#define OVER_CURRENT_IBUS_FAULT           1                       /*!< DC Bus over current fault */
#define VBUS_OVER_VOLTAGE_FAULT           3                       /*!< DC Bus over voltage fault */
#define VBUS_UNDER_VOLTAGE_FAULT          4                       /*!< DC Bus under voltage fault */
#define STARTUP_FAILURE_FAULT             7                       /*!< Startup fault */
#define MOTOR_STALL_FAULT                 8                       /*!< Motor stall fault */
#define MOTOR_LOSE_PHASEA_FAULT           12                      /*!< Motor lose phase A fault */
#define MOTOR_LOSE_PHASEB_FAULT           13                      /*!< Motor lose phase B fault */
#define MOTOR_LOSE_PHASEC_FAULT           14                      /*!< Motor lose phase C fault */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief System Error Flag bits data structure.
*/
typedef struct
{
    uint32_t reserved1 :                    1;                      /*!< Bits length reserve */
    uint32_t overCurrent :                  1;                      /*!< Bits length of DC Bus over current fault flag */
    uint32_t reserved2 :                    1;                      /*!< Bits length reserve */
    uint32_t overVoltage :                  1;                      /*!< Bits length of DC Bus overvoltage fault flag */
    uint32_t lowVoltage :                   1;                      /*!< Bits length of DC Bus undervoltage fault flag */
    uint32_t reserved3 :                    1;                      /*!< Bits length reserve */
    uint32_t reserved4 :                    1;                      /*!< Bits length reserve */
    uint32_t startupFail :                  1;                      /*!< Bits length of startup fail fault flag */
    uint32_t motorStall :                   1;                      /*!< Bits length of motor stall fault flag */
    uint32_t reserved6 :                    1;                      /*!< Bits length reserve */
    uint32_t reserved7 :                    1;                      /*!< Bits length reserve */
    uint32_t reserved8 :                    1;                      /*!< Bits length reserve */
    uint32_t losePhaseA :                   1;                      /*!< Bits length of Phase A lose fault flag */
    uint32_t losePhaseB :                   1;                      /*!< Bits length of Phase B lose fault flag */
    uint32_t losePhaseC :                   1;                      /*!< Bits length of Phase C lose fault flag */
    uint32_t hallStatus :                   1;                      /*!< Bits length of Hall sensor status fault flag */
    uint32_t reserved9 :                    1;                      /*!< Bits length reserve */
    uint32_t reserved10 :                   1;                      /*!< Bits length reserve */
    uint32_t mosfetLowerBridgeAShort :      1;                      /*!< Bits length of Phase A lower bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeBShort :      1;                      /*!< Bits length of Phase B lower bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeCShort :      1;                      /*!< Bits length of Phase C lower bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeAShort :      1;                      /*!< Bits length of Phase A upper bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeBShort :      1;                      /*!< Bits length of Phase B upper bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeCShort :      1;                      /*!< Bits length of Phase C upper bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeAOpen :       1;                      /*!< Bits length of Phase A lower bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeBOpen :       1;                      /*!< Bits length of Phase B lower bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeCOpen :       1;                      /*!< Bits length of Phase C lower bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeAOpen :       1;                      /*!< Bits length of Phase A upper bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeBOpen :       1;                      /*!< Bits length of Phase B upper bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeCOpen :       1;                      /*!< Bits length of Phase C upper bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t reserved11 :                   1;                      /*!< Bits length reserve */
    uint32_t hwFault :                      1;                      /*!< Bits length of PWM hardware fault flag */
} MOTOR_FAULT_STATE_BIT;

/*!
* @brief System Error Flag data structure.
*/
typedef union
{
    MOTOR_FAULT_STATE_BIT bits;                                     /*!< Fault bits */
    uint32_t all;                                                   /*!< Fault code */
} MOTOR_FAULT_FLAG;

/*!
* @brief System protection data structure.
*/
typedef struct
{
    MOTOR_FAULT_FLAG faultFlag;
    uint16_t HVoltCnt;                                              /*!< Overvoltage protection count */
    uint16_t LVoltCnt;                                              /*!< Undervoltage protection count */
    uint16_t recoverCnt;                                            /*!< Protection recovery count */

    uint16_t ErrRestartCnt;                                         /*!< Error restart count */
    uint16_t ErrCntResetTimer;                                      /*!< Error count reset time */
    uint16_t stallFlag;                                             /*!< Motor stall flag, has opportunity to recover from stall */
    uint16_t stallTime;                                             /*!< Motor stall time count */
    uint16_t stallRecoverTime;                                      /*!< Motor stall recover time count */

    uint16_t protectPC;                                             /*!< Protection count */
    uint16_t protectTime;                                           /*!< Protection time */

    uint16_t losePhase;                                             /*!< The ID of lose phase, (1)U or (2)V or (3)W */
    uint16_t IuAvg;                                                 /*!< Phase U current judge average value */
    uint16_t IvAvg;                                                 /*!< Phase V current judge average value */
    uint16_t IwAvg;                                                 /*!< Phase W current judge average value */
    uint16_t Imax;                                                  /*!< Phase current MAX value */
    uint16_t Imid;                                                  /*!< Phase current MID value */
    uint16_t Imin;                                                  /*!< Phase current MIN value */
    uint16_t curLimitCoeff;                                         /*!< DC bus current limit coefficient */
    uint32_t losePhaseCoeff;                                        /*!< Lose phase coefficient */

} PROTECTOR_TYPE;
/* ==========================================  Variables  =========================================== */


/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Derease output duty while sensorless stall happened.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void BLDC_Decrease_Duty_Protect(BLDC_VARS_CTRL *bldcCtrl, PROTECTOR_TYPE *protect);

/*!
* @brief Motor protection main function. The corresponding protection functions
*        are implemented according to different time bases.
*
* @param[in] none
* @return none
*/
void BLDC_Motor_Protection(void);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_PROTECTOR_H__ */

/* =============================================  EOF  ============================================== */
