/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_PWM_H__
#define __BLDC_PWM_H__
/*!
* @file bldc_pwm.h
*
* @brief This file provides PWM module functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_output.h"
#include "pwm_input.h"
#include "bldc_common.h"

/* ============================================  Define  ============================================ */
/* define six step square wave PWM register status */
#if (defined GATE_DRIVER_HIGH_HIGH)
    #define ALL_CH_OFF                  0x003F              /*!< PWM all channels off */

    #define CH0_MODULATION_CH3_ON       0x083C              /*!< PWM CH0 modulate, CH3 on */
    #define CH0_MODULATION_CH5_ON       0x203C              /*!< PWM CH0 modulate, CH5 on */
    #define CH2_MODULATION_CH5_ON       0x2033              /*!< PWM CH2 modulate, CH5 on */
    #define CH2_MODULATION_CH1_ON       0x0233              /*!< PWM CH2 modulate, CH1 on */
    #define CH4_MODULATION_CH1_ON       0x020F              /*!< PWM CH4 modulate, CH1 on */
    #define CH4_MODULATION_CH3_ON       0x080F              /*!< PWM CH4 modulate, CH3 on */

    #define PWM_CH0_ON                  0x013F              /*!< only PWM CH0 on */
    #define PWM_CH5_ON                  0x203F              /*!< only PWM CH5 on */
    #define PWM_CH2_ON                  0x043F              /*!< only PWM CH2 on */
    #define PWM_CH1_ON                  0x023F              /*!< only PWM CH1 on */
    #define PWM_CH4_ON                  0x103F              /*!< only PWM CH4 on */
    #define PWM_CH3_ON                  0x083F              /*!< only PWM CH3 on */
#elif (defined GATE_DRIVER_LOW_LOW)
    #define ALL_CH_OFF                  0x3F3F              /*!< PWM all channels off */

    #define CH0_MODULATION_CH3_ON       0x373C              /*!< PWM CH0 modulate, CH3 on */
    #define CH0_MODULATION_CH5_ON       0x1F3C              /*!< PWM CH0 modulate, CH5 on */
    #define CH2_MODULATION_CH5_ON       0x1F33              /*!< PWM CH2 modulate, CH5 on */
    #define CH2_MODULATION_CH1_ON       0x3D33              /*!< PWM CH2 modulate, CH1 on */
    #define CH4_MODULATION_CH1_ON       0x3D0F              /*!< PWM CH4 modulate, CH1 on */
    #define CH4_MODULATION_CH3_ON       0x370F              /*!< PWM CH4 modulate, CH3 on */

    #define PWM_CH0_ON                  0x3E3F              /*!< only PWM CH0 on */
    #define PWM_CH5_ON                  0x1F3F              /*!< only PWM CH5 on */
    #define PWM_CH2_ON                  0x3B3F              /*!< only PWM CH2 on */
    #define PWM_CH1_ON                  0x3D3F              /*!< only PWM CH1 on */
    #define PWM_CH4_ON                  0x2F3F              /*!< only PWM CH4 on */
    #define PWM_CH3_ON                  0x373F              /*!< only PWM CH3 on */
#else

#endif

/* ===========================================  Typedef  ============================================ */
typedef enum {
    INVALID_VECTOR_MIN = 0,                                 /*!< min invalid ID of six step square wave */
    AH_PWM_BL_ON,                                           /*!< A+B- of six step square wave */
    AH_PWM_CL_ON,                                           /*!< A+C- of six step square wave */
    BH_PWM_CL_ON,                                           /*!< B+C- of six step square wave */
    BH_PWM_AL_ON,                                           /*!< B+A- of six step square wave */
    CH_PWM_AL_ON,                                           /*!< C+A- of six step square wave */
    CH_PWM_BL_ON,                                           /*!< C+B- of six step square wave */
    INVALID_VECTOR_MAX                                      /*!< max invalid ID of six step square wave */
} BLDC_COMMUTATION_VECTOR;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief update six step square wave magnetic vector by current hall status.
*
* @param[in] motorDir: motor run direction
* @param[in] hallStatus: current rotor position hall status
* @return none
*/
void BLDC_Commutate_BaseSector(int16_t motorDir, uint8_t hallStatus);

/*!
* @brief Clear pwm duty and stop bldc motor.
*
* @param[in] none
* @return none
*/
void BLDC_Stop(void);

/*!
* @brief Get PWM module effective duty in 10000 times
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] PWMx: PWM Module
* @return pwm effective duty
*/
uint16_t Get_EffectiveDuty(BLDC_VARS_CTRL *bldcCtrl, PWM_Type *PWMx);

/*!
* @brief update PWM channel output by six step square wave magnetic vector to driver MOS.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] ChangePhaseVector: current magnetic vector in bldc control
* @return none
*/
void MosDriver(PWM_Type *PWMx, BLDC_VARS_CFG *bldcCfg, uint8_t ChangePhaseVector);

/*!
* @brief Update PWM output duty by manual settig or current PID calculate.
*
* @param[in] PWMx: PWM Module
* @param[in] pwmDuty: range: 0 ~ PWMx->MCVR, corresponding to 0 ~ 100% duty cycle
* @return none
*/
void UpDatePwmDuty(PWM_Type *PWMx, uint16_t pwmDuty);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_PWM_H__ */

/* =============================================  EOF  ============================================== */
