/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __APP_FOC_H__
#define __APP_FOC_H__
/*!
* @file foc_app.h
*
* @brief This file provides FOC control parameters initialization, state machine and millisecond time base functions interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwmdrv.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
extern uint16_t g_flashTaskFinish;              /*!< Finish tag of flash task, used to uart communication */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Motor vector control global variable parameters initialization.
*
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void STC_ParamInit(STC_PARAM *STC);

/*!
* @brief Foc control variables initialization, clear all the variables of FOC_VARS_CTRL to zero.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @return none
*/
void FOC_CtrlVarsInit(FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Foc control parameters initialization, executed in the Stop state and the first Power on.
*
* @param[in] none
* @return none
*/
void FOC_Parameters_Initialize(void);

/*!
* @brief ACR parameters initialization of stator resistance identification.
*
* @param[in] pwmModuler: PWM moduler to get Hall sector
* @return hallInitStatus
*/
uint8_t GetHallInitStatus(const uint8_t pwmModuler);

/*!
* @brief Foc control initialization, include control parameters and peripheral initialization.
*
* @param[in] none
* @return none
*/
void MC_Initialize(void);

/*!
* @brief LED1 light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED1Display(uint8_t ledStatus);

/*!
* @brief LED2 light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED2Display(uint8_t ledStatus);

/*!
* @brief Foc task in 1 ms time base, do speed loop, speed ramp, etc. Sensorless, hall, encoder all in this function, callback in task_scheduler.
*
* @param[in] none
* @return none
*/
void Motor_1ms_Task(void);

/*!
* @brief Foc task in 2 ms time base. If TOOL_UART_FUNCTION macro is defined, the UART function with host PC tool is executed in 2ms time base task.
*
* @param[in] none
* @return none
*/
void Motor_2ms_Task(void);

/*!
* @brief Foc task in 4 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_4ms_Task(void);

/*!
* @brief Foc task in 6 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_6ms_Task(void);

/*!
* @brief Foc task in 10 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_10ms_Task(void);

/*!
* @brief Foc task in 2 s time base.
*
* @param[in] none
* @return none
*/
void FOC_2000ms_Task(void);

/* ======================================  Functions define  ======================================== */

#endif /* __APP_FOC_H__ */

/* =============================================  EOF  ============================================== */
