/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PROTECTOR_H__
#define __PROTECTOR_H__
/*!
* @file protector.h
*
* @brief This file provides motor protection functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adcdrv.h"
#include "protector_parameters_define.h"
#include "speed_angle.h"

/* ============================================  Define  ============================================ */
#define P_MIN_UINT                          60                      /*!< Protection minute count unit */
#define P_SEC_UINT                          1                       /*!< Protection second count unit */

#define SEPARATION_CHIP                     0                       /*!< Separation gatedriver */
#define PREDRIVE_SEPARATION                 1                       /*!< Gatedriver chip */

#define SEPARATION_TYPE                     PREDRIVE_SEPARATION     /*!< Definition gatedriver type*/

#define DIAGNOSTIC_OK                       0                       /*!< The operation status is normal */
#define DIAGNOSTIC_FAIL                     1                       /*!< THe operation status is fault */
#define DIAGNOSTIC_UNKNOWN                  2                       /*!< THe operation status is unknow */

#define MOSFET_DIAGNOSTIC_DONE              0                       /*!< MOSFET fault diagnosis completed */
#define MOSFET_DIAGNOSTIC_NODONE            1                       /*!< MOSFET fault diagnosis not completed */

#define OVER_CURRENT_PHASE_FAILURE          0                       /*!< Phase over current fault */
#define OVER_CURRENT_IBUS_FAILURE           1                       /*!< DC Bus over current fault */
#define OVER_CURRENT_IS_FAILURE             2                       /*!< Output over current fault */
#define VBUS_OVER_VOLTAGE_FAILURE           3                       /*!< DC Bus over voltage fault */
#define VBUS_UNDER_VOLTAGE_FAILURE          4                       /*!< DC BUs under voltage fault */
#define LOSE_SPEED_FAILURE                  5                       /*!< Lose speed fault */
#define ZERO_SPEED_FAILURE                  6                       /*!< Zero speed fault */
#define STARTUP_FAILURE                     7                       /*!< Startup fault */
#define MOTOR_STALL_FAILURE                 8                       /*!< Motor stall fault */
#define PHASEA_CURRENT_MIDPOINT_FAILURE     9                       /*!< Phase A current sampling zero drift fault */
#define PHASEB_CURRENT_MIDPOINT_FAILURE     10                      /*!< Phase B current sampling zero drift fault */
#define PHASEC_CURRENT_MIDPOINT_FAILURE     11                      /*!< Phase C current sampling zero drift fault */
#define LOSE_PHASEA_FAILURE                 12                      /*!< Phase A lose phase fault */
#define LOSE_PHASEB_FAILURE                 13                      /*!< Phase B lose phase fault */
#define LOSE_PHASEC_FAILURE                 14                      /*!< Phase C lose phase fault */
#define HALL_STATUS_FAILURE                 15                      /*!< Hall sensor status fault */
#define HALL_STATUS_CHANGE_FAILURE          16                      /*!< Hall sensor change fault */
#define PHASE_CURRENT_SHORT_FAILURE         17                      /*!< Phase current short circuit fault */
#define MOSFET_LOWERBRIDGEA_SHORT_FAILURE   18                      /*!< Phase A lower bridge arm short circuit of MOSFET inverter */
#define MOSFET_LOWERBRIDGEB_SHORT_FAILURE   19                      /*!< Phase B lower bridge arm short circuit of MOSFET inverter */
#define MOSFET_LOWERBRIDGEC_SHORT_FAILURE   20                      /*!< Phase C lower bridge arm short circuit of MOSFET inverter */
#define MOSFET_UPPERBRIDGEA_SHORT_FAILURE   21                      /*!< Phase A upper bridge arm short circuit of MOSFET inverter */
#define MOSFET_UPPERBRIDGEB_SHORT_FAILURE   22                      /*!< Phase B upper bridge arm short circuit of MOSFET inverter */
#define MOSFET_UPPERBRIDGEC_SHORT_FAILURE   23                      /*!< Phase C upper bridge arm short circuit of MOSFET inverter */
#define MOSFET_LOWERBRIDGEA_OPEN_FAILURE    24                      /*!< Phase A lower bridge arm open circuit of MOSFET inverter */
#define MOSFET_LOWERBRIDGEB_OPEN_FAILURE    25                      /*!< Phase B lower bridge arm open circuit of MOSFET inverter */
#define MOSFET_LOWERBRIDGEC_OPEN_FAILURE    26                      /*!< Phase C lower bridge arm open circuit of MOSFET inverter */
#define MOSFET_UPPERBRIDGEA_OPEN_FAILURE    27                      /*!< Phase A upper bridge arm open circuit of MOSFET inverter */
#define MOSFET_UPPERBRIDGEB_OPEN_FAILURE    28                      /*!< Phase B upper bridge arm open circuit of MOSFET inverter */
#define MOSFET_UPPERBRIDGEC_OPEN_FAILURE    29                      /*!< Phase C upper bridge arm open circuit of MOSFET inverter */
#define HALL_SELF_LEARN_FAILURE             30                      /*!< Hall sensor sector self-learning fault */

#define HARDWARE_OC_FAILURE                 31                      /*!< Hardware over current fault */

#define OL_STATE_RUN                        0                       /*!< Over load protect run state */
#define OL_STATE_HOLD                       1                       /*!< Over load protect hold state */
#define OL_STATE_FALL                       2                       /*!< Over load protect fall frequency state */
#define OL_STATE_FAULT                      3                       /*!< Over load protect fault state */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief System Error Flag bits data structure.
*/
typedef struct MOTOR_ERR_FLAG_BIT {
    uint32_t overPhaseCurrent           :   1;                      /*!< Bits length of phase over current fault flag */
    uint32_t overIbusCurrent            :   1;                      /*!< Bits length of DC Bus over current fault flag */
    uint32_t overIsCurrent              :   1;                      /*!< Bits length of output over current fault flag */
    uint32_t overVoltage                :   1;                      /*!< Bits length of DC Bus overvoltage fault flag */
    uint32_t lowVoltage                 :   1;                      /*!< Bits length of DC Bus undervoltage fault flag */
    uint32_t loseSpeed                  :   1;                      /*!< Bits length of lose speed fault flag */
    uint32_t zeroSpeed                  :   1;                      /*!< Bits length of zero speed fault flag */
    uint32_t startUpErr                 :   1;                      /*!< Bits length of startup fault flag */
    uint32_t motorStall                 :   1;                      /*!< Bits length of lose speed fault flag */
    uint32_t curSensorAErr              :   1;                      /*!< Bits length of Phase A sampling zero drift fault flag */
    uint32_t curSensorBErr              :   1;                      /*!< Bits length of Phase B sampling zero drift fault flag */
    uint32_t curSensorCErr              :   1;                      /*!< Bits length of Phase C sampling zero drift fault flag */
    uint32_t losePhaseA                 :   1;                      /*!< Bits length of Phase A lose fault flag */
    uint32_t losePhaseB                 :   1;                      /*!< Bits length of Phase B lose fault flag */
    uint32_t losePhaseC                 :   1;                      /*!< Bits length of Phase C lose fault flag */
    uint32_t hallStatus                 :   1;                      /*!< Bits length of Hall sensor status fault flag */
    uint32_t hallStatusChange           :   1;                      /*!< Bits length of Hall sensor status change fault flag */
    uint32_t phaseCurrentShort          :   1;                      /*!< Bits length of phase current short circuit fault flag */
    uint32_t mosfetLowerBridgeAShort    :   1;                      /*!< Bits length of Phase A lower bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeBShort    :   1;                      /*!< Bits length of Phase B lower bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeCShort    :   1;                      /*!< Bits length of Phase C lower bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeAShort    :   1;                      /*!< Bits length of Phase A upper bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeBShort    :   1;                      /*!< Bits length of Phase B upper bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeCShort    :   1;                      /*!< Bits length of Phase C upper bridge arm short circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeAOpen     :   1;                      /*!< Bits length of Phase A lower bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeBOpen     :   1;                      /*!< Bits length of Phase B lower bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetLowerBridgeCOpen     :   1;                      /*!< Bits length of Phase C lower bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeAOpen     :   1;                      /*!< Bits length of Phase A upper bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeBOpen     :   1;                      /*!< Bits length of Phase B upper bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t mosfetUpperBridgeCOpen     :   1;                      /*!< Bits length of Phase C upper bridge arm open circuit of MOSFET inverter fault flag */
    uint32_t hallSelflearnFail          :   1;                      /*!< Bits length of Hall sensor sector self-learning fault flag */
    uint32_t hwFault                    :   1;                      /*!< Bits length of hardware over current fault flag */
//    uint32_t reserved                   :       1;                  /*!< Bits length reserve */
} MOTOR_ERR_FLAG_BIT;

/*!
* @brief System Error Flag data structure.
*/
typedef union MOTOR_ERR_FLAG {
    MOTOR_ERR_FLAG_BIT bits;                                        /*!< Fault bits */
    uint32_t all;                                                   /*!< Fault code */
} MOTOR_ERR_FLAG;

/*!
* @brief System protection data structure.
*/
typedef struct PROTECTOR {
    MOTOR_ERR_FLAG ErrFlag;                                         /*!< Fault flag bits */
    uint16_t countSpeedProA1;                                       /*!< Speed protection countA1 */
    uint16_t countSpeedProA2;                                       /*!< Speed protection countA2 */
    uint16_t countSpeedProA3;                                       /*!< Speed protection countA3 */
    uint16_t countSpeedProB1;                                       /*!< Speed protection countB1 */
    uint16_t countSpeedProB2;                                       /*!< Speed protection countB2 */
    uint16_t countSpeedProB3;                                       /*!< Speed protection countB3 */
    uint16_t HVoltCnt;                                              /*!< Overvoltage protection count */
    uint16_t LVoltCnt;                                              /*!< Undervoltage protection count */
    uint16_t recoverCnt;                                            /*!< Protection recovery count */
    uint16_t HCurrentCnt;                                           /*!< Overcurrent protection count */
    uint16_t ErrRestartCnt;                                         /*!< Error restart count */
    uint16_t ErrCntResetTimer;                                      /*!< Error count reset time */
    uint16_t motorBlockCounter;                                     /*!< Motor block count */
    uint16_t hallErrCounter;                                        /*!< Hall error count */
    uint16_t delayTimer;                                            /*!< Dely time */
    uint16_t startUpErrDelay;                                       /*!< StartUp error dely */
    uint16_t startUpDelayCounter;                                   /*!< StartUp dely count */
    uint16_t zeroSpeedDelay;                                        /*!< Zero speed dely */
    uint16_t loseSpeedDely;                                         /*!< Lose speed dely */
    uint16_t losAphTime;                                            /*!< Phase A phase loss detection time */
    uint16_t losBphTime;                                            /*!< Phase B phase loss detection time */
    uint16_t losCphTime;                                            /*!< Phase C phase loss detection time */
    uint16_t losePhaseDelay;                                        /*!< phase loss protection delay */
    uint16_t losePhaseTime;                                         /*!< Phase loss protection detection time threshold */
    uint32_t losePhaseLimit;                                        /*!< Phase loss protection current threshold */
    uint16_t losePhase;                                             /*!< Lose phase is which pahase, U or V or W */
    uint16_t IuAvg;                                                 /*!< Phase U current average value */
    uint16_t IvAvg;                                                 /*!< Phase V current average value */
    uint16_t IwAvg;                                                 /*!< Phase W current average value */
    uint16_t Imax;                                                  /*!< Phase current MAX value */
    uint16_t Imin;                                                  /*!< Phase current MIN value */
    uint32_t losePhaseCof;                                          /*!< Lose phase coefficient */
    uint16_t startUpErrFlag;                                        /*!< StartUp error flag */
    uint16_t lowSpeedProFlag;                                       /*!< LowSpeed error flag */
    uint16_t protectPC;                                             /*!< Protection count */
    uint16_t protectTime;                                           /*!< Protection time */
    uint16_t zeroSpeedProFlag;                                      /*!< Zero speed protect flag */
    uint16_t loseSpeedProFlag;                                      /*!< Lose speed protect flag */
    uint16_t stallProFlag;                                          /*!< Stall protect flag */
} PROTECTOR;

/*!
* @brief Stall protection data structure.
*/
typedef struct
{
    uint8_t stallFlag;                                              /*!< Stall protect flag */
    uint16_t stallCoeff;                                            /*!< Stall protect coefficient */
    uint16_t judegTime;                                             /*!< Stall protect judge time */
    uint16_t judegTimepu;                                           /*!< Stall protect judge time pu */
    uint16_t totalTime;                                             /*!< Stall protect total time */
    uint16_t totalTimePu;                                           /*!< Stall protect total time pu */
    uint16_t cntTime;                                               /*!< Stall protect count time */
    uint16_t cntTimepu;                                             /*!< Stall protect count time pu */
    uint8_t judgeSpd;                                               /*!< Stall protect judge speed rpm */
    uint16_t judgeCur;                                              /*!< Stall protect judge current pu */
    uint8_t stallTimes;                                             /*!< Stall protect times */
    uint32_t stallGain;                                             /*!< Stall protect gain */
    uint16_t timeBase;                                              /*!< Stall protect time base */
} STALL_PROTECT;

/*!
* @brief Power reduced protection data structure.
*/
typedef struct
{
    uint8_t spdFlag;                                                /*!< Power reduced protection speed flag */
    uint8_t mosTempFlag;                                            /*!< Power reduced protection MOSFET temperature flag */
    uint8_t ibusFlag;                                               /*!< Power reduced protection bus current flag */
    uint8_t vbusHighFlag;                                           /*!< Power reduced protection bus over voltage flag */
    uint8_t vbusLowFlag;                                            /*!< Power reduced protection bus low voltage flag */
    uint8_t depDoneFlag;                                            /*!< Power reduced protection done flag */
    uint8_t depFlag;                                                /*!< Power reduced protection flag */

    uint16_t spdIn;                                                 /*!< Power reduced protection speed input */
    uint16_t spdStart;                                              /*!< Power reduced protection speed start value */
    uint16_t spdStop;                                               /*!< Power reduced protection speed stop value */
    uint16_t spdCoeff;                                              /*!< Power reduced protection speed coeff */

    uint16_t mosTempIn;                                             /*!< Power reduced protection MOSFET temperature input */
    uint16_t mosTempStart;                                          /*!< Power reduced protection MOSFET temperature start value */
    uint16_t mosTempStop;                                           /*!< Power reduced protection MOSFET temperature stop value */
    uint16_t mosTempCoeff;                                          /*!< Power reduced protection MOSFET temperature coeff */

    uint16_t ibusIn;                                                /*!< Power reduced protection bus current input after filter */
    uint16_t ibus;                                                  /*!< Power reduced protection bus current before filter */
    uint16_t ibusStart;                                             /*!< Power reduced protection bus current start value */
    uint16_t ibusStop;                                              /*!< Power reduced protection bus current stop value */
    uint16_t ibusCoeff;                                             /*!< Power reduced protection bus current coeff */

    uint16_t vbusHighIn;                                            /*!< Power reduced protection bus over voltage input */
    uint16_t vbusHighStart;                                         /*!< Power reduced protection bus over voltage start value */
    uint16_t vbusHighStop;                                          /*!< Power reduced protection bus over voltage stop value */
    uint16_t vbusHighCoeff;                                         /*!< Power reduced protection bus over voltage coeff */

    uint16_t vbusLowIn;                                             /*!< Power reduced protection bus low voltage input */
    uint16_t vbusLowStart;                                          /*!< Power reduced protection bus low voltage start value */
    uint16_t vbusLowStop;                                           /*!< Power reduced protection bus low voltage stop value */
    uint16_t vbusLowCoeff;                                          /*!< Power reduced protection bus low voltage coeff */

    uint16_t depCoeff;                                              /*!< Power reduced protection coeff */
    uint16_t speedBuf;                                              /*!< Speed target command buff value before power reduced protection operate */

    uint32_t spdGain;                                               /*!< Power reduced protection speed gain */
    uint32_t mosTempGain;                                           /*!< Power reduced protection MOSFET temperature gain */
    uint32_t ibusGain;                                              /*!< Power reduced protection bus current gain */
    uint32_t vbusHighGain;                                          /*!< Power reduced protection bus over voltage gain */
    uint32_t vbusLowGain;                                           /*!< Power reduced protection bus low voltage gain */
} POWER_DECREASE;

/*!
* @brief Power reduced type data enumeration, 0:over limit type power decrease, 1:low limit type power decrease.
*/
typedef enum
{
    DEPOWER_H = 0,
    DEPOWER_L = 1
} DEP_TYPE;

/*!
* @brief Power reduced protection data enumeration.
*/
enum
{
    DEPOWER_NUM = 5,                                                /*!< Power reduced protection enumeration parameters */

    SPEED_H = 0,                                                    /*!< Power reduced protection over speed parameters */
    MOS_H = 1,                                                      /*!< Power reduced protection over temperature parameters */
    IBUS_H = 2,                                                     /*!< Power reduced protection over ibus parameters */
    VBUS_H = 3,                                                     /*!< Power reduced protection over vbus parameters */
    VBUS_L = 4                                                      /*!< Power reduced protection low vbus parameters */
};

/*!
* @brief DC Bus current limit protection data structure.
*/
typedef struct
{
    uint16_t dcCur;                                                 /*!< DC bus current */
    uint16_t fallCnt;                                               /*!< Fall time count */
    uint16_t upCnt;                                                 /*!< Up time count */
    uint16_t curLimitCoeff;                                         /*!< DC bus current limit coefficient */
} DCCUR_LIMIT;

/*!
* @brief Input power limit protection data structure.
*/
typedef struct
{
    uint16_t power;                                                 /*!< Input power */
    uint16_t fallCnt;                                               /*!< Fall time count */
    uint16_t upCnt;                                                 /*!< Up time count */
    uint16_t powerLimitCoeff;                                       /*!< Power limit coefficient */
} POWER_LIMIT;

/*!
* @brief System protection configuration structure.
*/
typedef struct
{
    uint16_t shortDutyThreshold;                                    /*!< MOSFET bridge short circuit duty threshold value */
    uint16_t openDutyThreshold;                                     /*!< MOSFET bridge open circuit duty threshold value */
    uint32_t shortCurThreshold;                                     /*!< MOSFET bridge short circuit current threshold value */
    uint32_t openCurThreshold;                                      /*!< MOSFET bridge open circuit current threshold value */
} PROTECTOR_CFG;

/*!
* @brief FOC startup recover protection parameters structure.
*/
typedef struct
{
    uint16_t restartFlag;                                           /*!< flag of restart */
    uint16_t runToRestartFlag;                                      /*!< flag of restart begin */
    uint16_t restartTimes;                                          /*!< restart times */
    int16_t restartSpeedBuf;                                        /*!< speed command value backup buffer */
    uint16_t restartDoneFlag;                                       /*!< flag of restart finished */
    uint16_t overLoadFault;                                         /*!< report overload fault after restart has occurred too many times */
    uint16_t restartClearTime;                                      /*!< time counter to clear the restart number */
    uint16_t restartDlyCnt;                                         /*!< delay cycle counter to judge restart */
    uint16_t startUpCnt;                                            /*!< startup count */
} FOC_RESTARTUP;

/*!
* @brief FOC over load protection parameters structure.
*/
typedef struct
{
    uint8_t state;                                                  /*!< over load state */
    uint8_t fallFreqFlag;                                           /*!< over load fall frequency flag */
    uint16_t fallFreqCof;                                           /*!< fall frequency output coefficient */
    uint16_t freqGrade;                                             /*!< fall frequency output grade, 1~5 */
    int16_t speedBuff;                                              /*!< speed command buff value before fall frequency */
    uint32_t inputIs;                                               /*!< input Is value */
    uint32_t filterIs;                                              /*!< Is value after filter */
    uint32_t recoverTime;                                           /*!< fall frequency recover time */
    uint32_t holdToFallTime;                                        /*!< hold state to fall state time */
    uint32_t fallTime1;                                             /*!< fall frequency time1 */
    uint32_t fallTime2;                                             /*!< fall frequency time2 */
} OVERLOAD_PRO;

/* ==========================================  Variables  =========================================== */
extern STALL_PROTECT      g_stallProtect;                           /*!< pointer to Stall protection structure */
extern POWER_DECREASE     g_powerDecrease;                          /*!< pointer to Power reduced protection structure */
extern DCCUR_LIMIT        g_dcCurLimit;                             /*!< pointer to DC Bus current limit protection structure */
extern POWER_LIMIT        g_powerLimit;                             /*!< pointer to Input power limit protection structure */
extern FOC_RESTARTUP      g_restart;                                /*!< startup error restart structure */
extern OVERLOAD_PRO       g_overLoadPro;                            /*!< over load protection structure */

extern int32_t g_estBemfDebug;           /*!< Back-EMF estimated value */
extern int32_t g_bemfReliableThreshold;  /*!< Back-EMF reliable threshold value */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief System protection initialization.
*
* @param[in] none
* @return none
*/
void Init_Protection(void);

/*!
* @brief Current sencor sampling zero drift check. The current sampling
*        ADC zero drift value exceeds the limit, it is a fault.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void Current_Sensor_Check(ADC_SAMPLE *adcSampleReal);

/*!
* @brief Hall sensor sector status protection.
*
* @param[in] hallstatus: Current hall sector state
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
uint16_t Hall_Sensor_Status_Check(uint16_t hallstatus);

/*!
* @brief Hall Sensor protection.
*
* @param[in] hallnew: Current hall sector state
* @param[in] hallold: Previous hall sector state
* @param[in] protect: pointer to PROTECTOR structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
uint16_t Hall_Sensor_Change_Check(uint16_t hallnew, uint16_t hallold, PROTECTOR *protect);

/*!
* @brief Motor protection main function. The corresponding protection functions
*        are implemented according to different time bases.
*
* @param[in] none
* @return none
*/
void Motor_Protection(void);

/*!
* @brief Hall sensor galloping protection. Prevent motor speed uncontrolled increasing.
*
* @param[in] none
* @return none
*/
void HallFlyingCarProtect(void);

/*!
* @brief Report Fault identification.
*
* @param[in] faultid: Fault identification
* @return none
*/
void Fault_Report(uint16_t faultid);

/*!
* @brief Clear Fault identification.
*
* @param[in] faultid: Fault identification
* @return none
*/
void Fault_Clear(uint16_t faultid);

/*!
* @brief Power reduced protection parameters initialization.
*
* @param[in] powerDecrease: pointer to POWER_DECREASE structure
* @return none
*/
void PowerDecrease_init(POWER_DECREASE *powerDecrease);

/*!
* @brief Power reduced protection calculation function.
*
* @param[in] powerDecrease: pointer to POWER_DECREASE structure
* @return none
*/
void PowerDecrease_Calc(POWER_DECREASE *powerDecrease);

/*!
* @brief DC Bus current limit protection parameters initialization.
*
* @param[in] dcCurLimit: pointer to DCCUR_LIMIT structure
* @return none
*/
void DcCurLimit_Init(DCCUR_LIMIT *dcCurLimit);

/*!
* @brief Input power limit protection parameters initialization.
*
* @param[in] powerLimit: pointer to POWER_LIMIT structure
* @return none
*/
void PowerLimit_Init(POWER_LIMIT *powerLimit);

/*!
* @brief Startup error restart parameters initialization.
*
* @param[in] restart: pointer to FOC_RESTARTUP structure
* @return none
*/
void Foc_RestartInit(FOC_RESTARTUP *restart);

/*!
* @brief Startup error restart judge process.
*
* @param[in] restart: pointer to FOC_RESTARTUP structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void Foc_RestartJudgeProcess(FOC_RESTARTUP *restart, SPEED_RAMP_HANDLE *speed);

/*!
* @brief Startup error restart state process.
*
* @param[in] restart: pointer to FOC_RESTARTUP structure
* @return none
*/
void Foc_RestartProcess(FOC_RESTARTUP *restart);

/*!
* @brief Motor over load decrease frequency protection parameters initilization.
*
* @param[in] overLoad: pointer to OVERLOAD_PRO structure
* @return none
*/
void Is_OverLoad_Init(OVERLOAD_PRO *overLoad);

/*!
* @brief Motor over load decrease frequency protection. When the amplitude of the phase current synthesis vector exceeds
*        the threshold value and for a certain amount of time, the frequency command will fall.
*
* @param[in] overLoad: pointer to OVERLOAD_PRO structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void Is_OverLoad_FallFreq(OVERLOAD_PRO *overLoad, FOC_VARS_CTRL *focVars, SPEED_RAMP_HANDLE *speed);

/*!
* @brief Limit ASR speed target command, accord to power decrease output coefficient or DC current / Power limit coefficient.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedTargetLimitProcess(SPEED_RAMP_HANDLE *command);

/*!
* @brief DC Bus under voltage protection. When the bus voltage is lower than
*        threshold value, Undervoltage fault is reported, and it will return
*        to normal operation after the bus voltage rises to the recovery threshold.
*
* @param[in] volt: DC Bus voltage value, 10 times the real value
* @return none
*/
void Vbus_UnderVoltage_Check(uint16_t volt);

/*!
* @brief DC Bus over voltage protection. When the bus voltage exceeds the
*        threshold value, Overvoltage fault is reported, and it will return
*        to normal operation after the bus voltage drops to the recovery threshold.
*
* @param[in] volt: DC Bus voltage value, 10 times the real value
* @return none
*/
void Vbus_OverVoltage_Check(uint16_t volt);

/*!
* @brief Phase over current protection. When the amplitude of the phase current
*        synthesis vector exceeds the threshold value, phase over current fault is reported.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Phase_Over_Current_Check(FOC_VARS_CTRL *focVars);

/* ======================================  Functions define  ======================================== */

#endif /* __PROTECTOR_H__ */

/* =============================================  EOF  ============================================== */
