/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PULSE_INJECT_API_H__
#define __PULSE_INJECT_API_H__
/*!
* @file pulse_inject_api.h
*
* @brief  This file mainly provides mcu PWM and ADC module operation function aviod read or wirte mcu registers in libray.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pulse_inject.h"
#include "startup.h"

/* ============================================  Define  ============================================ */
/* PWM_HIGH_LEVEL = 1, PWM_LOW_LEVEL = 0 */
#define UP_BRIDGE_ACTIVE_STATE      PWM_HIGH_LEVEL      /*!< 1: upper conductor high level is active state. 0:upper conductor low level is active state */
#define DOWN_BRIDGE_ACTIVE_STATE    PWM_HIGH_LEVEL      /*!< 1: downer conductor high level is active state. 0:downer conductor low level is active state */
#define UP_BRIDGE_INACTIVE_STATE    PWM_LOW_LEVEL       /*!< 1: upper conductor high level is active state. 0:upper conductor low level is active state */
#define DOWN_BRIDGE_INACTIVE_STATE  PWM_LOW_LEVEL       /*!< 1: downer conductor high level is active state. 0:downer conductor low level is active state */

/*!
* mode0 PWM0-> A  PWM2-> B  PWM4-> C
* mode1 PWM0-> A  PWM2-> C  PWM4-> B
* mode2 PWM0-> B  PWM2-> A  PWM4-> C
* mode3 PWM0-> B  PWM2-> C  PWM4-> A
* mode4 PWM0-> C  PWM2-> A  PWM4-> B
* mode5 PWM0-> C  PWM2-> B  PWM4-> A
*/
#define PWM_IO_MODE                 0                   /*!< select the output mode of PWM channels */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Initial position pulse injection function main status machine, which include five status.
*        CHIP_CONFIG: pulse injection pwm and adc configuration.
*        SELF_LEARNING:  get proper pulse injection width, which is different with each motor.
*        PULSE_INJECTING: pulse injection process and result calculate.
*        CHIP_CONFIG_RESET: pulse injection finished, recover pwm and adc configuration.
*        PULSE_INJECTION_ERR: during pulse injection process, maybe occur some errors.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return Initial position pulse inject function finished flag
*/
uint16_t PulseInject(PULSE_INJECT_DATA *pulseInject);

/*!
* @brief Beacuse the motor maybe bldc or pmsm, the control mode is different.
*        In order to output pulse injection, PWM and ADC module will be configurated, and then go to the next status.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_Config(PULSE_INJECT_DATA *pulseInject);

/*!
* @brief Initial position checking need inject at different direction, so this function can change pwm output.
*        This function can fit to different hardboard, through change settings.
*        What's more, PWM configuration makes pwm output mask effective immediately.
*
* @param[in] section: Judge pulse inject direction
* @return none
*/
void PulseInj_SetChlVal(uint8_t section);

/*!
* @brief Initial position pulse injection function finished, then recover configuration.
*        Because enter initial position module, bldc or pmsm control have different configuration, after initial position checked,
*        recover configuration according to control method.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_ConfigReset(PULSE_INJECT_DATA *pulseInject);

/*!
* @brief In order to make libray decouple with mcu registers, this function is called by libray to get bus current value.
*        It is called when pulse injection finished, and sample bus current.
*
* @param[in] none
* @return bus current AD value
*/
uint16_t PulseInject_GetBusCur(void);

/*!
* @brief Pulse injection function data clear.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_Init(PULSE_INJECT_DATA *pulseInject);

/*!
* @brief Multiple pulse injection process in main status machine(ANGLE_JUDGE).
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return Multiple pulse inject function flag
*         0: pulse injection repeat unfinished, 1: pulse injection repeat finished, 2: pulse injection repeat results are inconsistent
*/
uint16_t PulseInject_Process(PULSE_INJECT_DATA *pulseInject, STARTUP_CTRL *startUp);

/* ======================================  Functions define  ======================================== */

#endif /* __PULSE_INJECT_API_H__ */

/* =============================================  EOF  ============================================== */
