/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file adcdrv.c
*
* @brief This file provides ADC sampling functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Clear A-B-C phase sample current result.
*
* @param[in] phaseCurrent: pointer to ADC_TRANSFORM structure
* @return none
*/
void Clear_CurrentSample(ADC_TRANSFORM *phaseCurrent)
{
    phaseCurrent->iaPu = 0;
    phaseCurrent->ibPu = 0;
    phaseCurrent->icPu = 0;
}

/*!
* @brief Get back emf sample AD value.
*
* @param[in] none
* @return bemf: pointer to BEMF_SAMPLE structure
*/
BEMF_SAMPLE Get_BemfSample(void)
{
    BEMF_SAMPLE bemf;

    /* emf u */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_7);
    /* emf v */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_6);
    /* emf w */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_2, ADC_CH_4);

    bemf.emfU = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    bemf.emfV = ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    bemf.emfW = ADC_GetInjectData(ADC0, ADC_ISEQ_2);

    return bemf;
}

/*!
* @brief Get A-B-C phase sample current offset.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void GetCurrentOffset(ADC_SAMPLE *adcSampleReal)
{
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcSampleReal->ibusOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcSampleReal->iaOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ibOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_1);
//    adcSampleReal->ibusOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    /* Three-shunt sample channels have been changed in RUN status, should be recovered in IDLE status */
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);
    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_5);
    /* IW */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_2, ADC_CH_4);
    /* IBUS */
//    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_3, ADC_CH_2);

    adcSampleReal->iaOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ibOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->icOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_2);
//    adcSampleReal->ibusOffset = ADC_GetInjectData(ADC0, ADC_ISEQ_3);
#else

#endif
}

/*!
* @brief Get A-B-C phase sample current result.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @param[in] adcPuCof: pointer to ADC_SAMPLE_COF structure
* @param[in] phaseCurrent: pointer to ADC_TRANSFORM structure
* @return none
*/
void Get_CurrentSample(ADC_SAMPLE *adcSampleReal, ADC_SAMPLE_COF *adcPuCof, ADC_TRANSFORM *phaseCurrent)
{
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    #if (defined TAIL_HEAD_WIND)
    /* IBUS */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_2);
    /* IBUS */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_2);
    #endif
    int16_t iBusMidMin = 0;
    int16_t iBusMaxMid = 0;
    static int16_t iaPuBuf = 0;
    static int16_t ibPuBuf = 0;
    static int16_t icPuBuf = 0;

    /* 0 means Up edge sample */
    #if (SAMPLE_TRIG_EDGE == 0)
    {
        iBusMaxMid = ADC_GetInjectData(ADC0, ADC_ISEQ_0) - adcSampleReal->ibusOffset;
        iBusMidMin = ADC_GetInjectData(ADC0, ADC_ISEQ_1) - adcSampleReal->ibusOffset;
    }
    /* 1 means Down edge sample */
    #else
    {
        iBusMidMin = ADC_GetInjectData(ADC0, ADC_ISEQ_0) - adcSampleReal->ibusOffset;
        iBusMaxMid = ADC_GetInjectData(ADC0, ADC_ISEQ_1) - adcSampleReal->ibusOffset;
    }
    #endif
    CurRestruct_IaIbIc(iBusMidMin, iBusMaxMid, &g_curRestruct, pCurRestructCfg);
    if (g_curRestruct.nonSampArea)
    {
        g_curRestruct.nonSampArea = 0;
        phaseCurrent->iaPu = iaPuBuf;
        phaseCurrent->ibPu = ibPuBuf;
        phaseCurrent->icPu = icPuBuf;
    }
    else
    {
        phaseCurrent->iaPu = adcPuCof->kCurSamplePu * g_curRestruct.AdIaVal;
        phaseCurrent->ibPu = adcPuCof->kCurSamplePu * g_curRestruct.AdIbVal;
        phaseCurrent->icPu = -phaseCurrent->iaPu - phaseCurrent->ibPu;
    }
    iaPuBuf = phaseCurrent->iaPu;
    ibPuBuf = phaseCurrent->ibPu;
    icPuBuf = phaseCurrent->icPu;
#elif (CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE)
    #if (defined TAIL_HEAD_WIND)
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);

    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_5);
    /* IBUS */
//    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_2, ADC_CH_2);
    #endif

    phaseCurrent->iaPu = adcPuCof->kCurSamplePu * (adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0));
    phaseCurrent->ibPu = adcPuCof->kCurSamplePu * (adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1));
    phaseCurrent->icPu = -phaseCurrent->iaPu - phaseCurrent->ibPu;


#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    #if (defined TAIL_HEAD_WIND)
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);

    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_5);
    /* IW */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_2, ADC_CH_4);
    #endif
    Get_ThreeShuntsCur(adcSampleReal);
    phaseCurrent->iaPu = adcPuCof->kCurSamplePu * adcSampleReal->ia;
    phaseCurrent->ibPu = adcPuCof->kCurSamplePu * adcSampleReal->ib;
    phaseCurrent->icPu = adcPuCof->kCurSamplePu * adcSampleReal->ic;
#elif (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE)
    phaseCurrent->iaPu = adcPuCof->kCurSensorPu * (adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0));
    phaseCurrent->ibPu = adcPuCof->kCurSensorPu * (adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1));
    phaseCurrent->icPu = -phaseCurrent->iaPu - phaseCurrent->ibPu;
#endif

}

/*!
* @brief Get DC bus voltage sampling result.
*
* @param[in] adcPuCof: pointer to ADC_SAMPLE_COF structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] vbus: pointer to ADC_TRANSFORM structure
* @return none
*/
void Get_VoltageSample(ADC_SAMPLE_COF *adcPuCof, MOTOR_PARAM_PU *paramPu, ADC_TRANSFORM *vbus)
{
    vbus->vdcPuBeforeLPF = ((uint32_t)adcPuCof->kVdcSamplePu * (g_ADC1RegularBuffer[0] + g_ADC1RegularBuffer[3])) >> 1;
    /* 120 = 12V, 10times larger */
    vbus->vdcVt = (uint16_t)(((uint32_t)(vbus->vdcPuBeforeLPF) * (paramPu->ubVt * 10)) >> GLOBAL_Q);
    vbus->vdcPu = GetAvgCalc(vbus->vdcPuBeforeLPF, 11);

    vbus->ibusPuAvg = (int32_t)adcPuCof->kCurSamplePu * (g_ADC1RegularBuffer[1] - g_adcSampleReal.ibusOffset);
    vbus->ibusRealAvg = (int32_t)vbus->ibusPuAvg * g_motorParamPu.ibAp >> GLOBAL_Q;
}

/*!
* @brief Get three shunts sample mode data.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void Get_ThreeShuntsCur(ADC_SAMPLE *adcSampleReal)
{
    switch (g_pwmValue.sector)
    {
    case 1:
    adcSampleReal->ib = adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ic = adcSampleReal->icOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ia = -adcSampleReal->ib - adcSampleReal->ic;
    break;

    case 2:
    adcSampleReal->ia = adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ic = adcSampleReal->icOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ib = -adcSampleReal->ia - adcSampleReal->ic;
    break;

    case 3:
    adcSampleReal->ic = adcSampleReal->icOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ia = adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ib = -adcSampleReal->ic - adcSampleReal->ia;
    break;

    case 4:
    adcSampleReal->ib = adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ia = adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ic = -adcSampleReal->ib - adcSampleReal->ia;
    break;

    case 5:
    adcSampleReal->ia = adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ib = adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ic = -adcSampleReal->ia - adcSampleReal->ib;
    break;

    case 6:
    adcSampleReal->ic = adcSampleReal->icOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ib = adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ia = -adcSampleReal->ic- adcSampleReal->ib;
    break;

    default:
    adcSampleReal->ia = adcSampleReal->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    adcSampleReal->ib = adcSampleReal->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    adcSampleReal->ic = -adcSampleReal->ia - adcSampleReal->ib;
    break;
    }
}

/*!
* @brief Changing the sampling channel of ADC is used to save ADC channel and
*        improve sampling speed in 3-shunt sampling mode.
*
* @param[in] none
* @return none
*/
void ChangeAdSampChannel(void)
{
    switch (g_pwmValue.sector)
    {
    case 1:
    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_5);
    /* IW */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_4);
    break;

    case 2:
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);
    /* IW */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_4);
    break;

    case 3:
    /* IW */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_4);
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_3);
    break;

    case 4:
    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_5);
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_3);
    break;

    case 5:
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);
    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_5);
    break;

    case 6:
    /* IW */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_4);
    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_5);
    break;

    default:
    /* IU */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);
    /* IV */
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_1, ADC_CH_5);
    break;
    }
}

/*!
* @brief Adc sampling result pu value transmit to foc variable.
*
* @param[in] adcResult: pointer to ADC_TRANSFORM structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void AdcResultTransmit(ADC_TRANSFORM *adcResult, FOC_VARS_CTRL *focVars)
{
    focVars->iaPu = adcResult->iaPu;
    focVars->ibPu = adcResult->ibPu;
    focVars->icPu = adcResult->icPu;
    focVars->vdcPu = adcResult->vdcPu;
}

/* =============================================  EOF  ============================================== */
