/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file motor_debug.c
*
* @brief This file provides motor control debugging output observation functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_output.h"
#include "spi_master_drv.h"
#include "globalvar.h"
#include "motor_debug.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
uint16_t g_dispCh0 = DUBUG_DAC_CH0_PARAMID;//< Debug channel0 mark
uint16_t g_dispCh1 = DUBUG_DAC_CH1_PARAMID;//< Debug channel1 mark

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Write SPI channel value when use SPI as debug tool.
*
* @param[in] channel, channel of SPI
* @param[in] value, channel value
* @return none
*/
void MC_DEBUG_SPI_WriteChannelVaule(uint8_t channel, uint16_t value)
{
    uint16_t command;

    uint16_t g_spiTxBuff0[1] = {0x0};
    uint16_t g_spiRxBuff0[1] = {0x0};

    command = (channel << 12) | ((value << 2) & 0x0ffc);
    g_spiTxBuff0[0] = command;

//    SPI_TransmitPoll(SPI1, (uint8_t *)g_spiTxBuff0, 2);
    SPI_DRV_MasterTransfer(1, (uint8_t *)g_spiTxBuff0, (uint8_t *)g_spiRxBuff0, 2);

}

/*!
* @brief Watch Debug variable in dac channnel use PWM.
*
* @param[in] channel : 0 or 1, select the DAC channel
* @param[in] paramId : 0--255, ID of debug variable
* @return none
*/
//void Debug_Dac_Watch_PWM(uint16_t channel, uint16_t paramId)
void Debug_Dac_Watch_PWM(pwm_channel_type_t channel, uint16_t paramId)
{
    switch (paramId)
    {
        /* paramId is same as uart list id */
        case 9:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_speedCommand.speedTargetRamp + 32768) >> 4));
        break;

        case 10:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_speedCommand.speedFbk + 32768) >> 4));
        break;

        case 11:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_speedCommand.avgSpeedFbkRpm + 32768) >> 4));
        break;

        case 43:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_asrSpeedPIIn.refPu + 32768) >> 4));
        break;

        case 44:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_asrSpeedPIIn.fbkPu + 32768) >> 4));
        break;

        case 45:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_asrSpeedPIOut.outPu + 32768) >> 4));
        break;

        case 56:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, (g_hallSensor.hallStatus << 9));
        break;

        case 138:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_udqDcpOut.dcpUdPu + 32768) >> 4));
        break;

        case 139:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_udqDcpOut.dcpUqPu + 32768) >> 4));
        break;

        case 142:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->iaPu + 32768) >> 4));
        break;

        case 143:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->ibPu + 32768) >> 4));
        break;

        case 144:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->ialpha + 32768) >> 4));
        break;

        case 145:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->ibeta + 32768) >> 4));
        break;

        case 146:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->iqFdb + 32768) >> 4));
        break;

        case 147:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->idFdb + 32768) >> 4));
        break;

        case 148:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, (((int16_t)g_focVars.pCtrl->idq + 32768) >> 4));
        break;

        case 149:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->iqRef + 32768) >> 4));
        break;

        case 150:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->idRef + 32768) >> 4));
        break;

        case 151:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->vq + 32768) >> 4));
        break;

        case 152:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->vd + 32768) >> 4));
        break;

        case 153:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->vdqLimit + 32768) >> 4));
        break;

        case 154:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->valpha + 32768) >> 4));
        break;

        case 155:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_focVars.pCtrl->vbeta + 32768) >> 4));
        break;

        case 157:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, (g_focVars.pCtrl->elecAngle >> 4));
        break;

        case 158:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, (g_startUpCtrl.elecThetaEst >> 4));
        break;

        case 163:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, (g_hfinj.hfiEstTheta >> 4));
        break;

        case 164:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, ((g_hfinj.hfiEstFreq + 32768) >> 4));
        break;

        default:
            PWM_DRV_SetChannelCountValue(PWM_MODULER_DEBUG, channel, (g_startUpCtrl.elecThetaEst >> 4));
        break;
    }
}

/*!
* @brief Watch Debug variable in dac channnel use SPI.
*
* @param[in] channel : 0 or 1, select the DAC channel
* @param[in] paramId : 0--255, ID of debug variable
* @return none
*/
void Debug_Dac_Watch_SPI(uint16_t channel, uint16_t paramId)
{
    switch (paramId)
    {
        /* paramId is same as uart list id */
        case 9:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_speedCommand.speedTargetRamp + 32768) >> 6));
        break;

        case 10:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_speedCommand.speedFbk + 32768) >> 6));
        break;

        case 11:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_speedCommand.avgSpeedFbkRpm + 32768) >> 6));
        break;

        case 43:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_asrSpeedPIIn.refPu + 32768) >> 6));
        break;

        case 44:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_asrSpeedPIIn.fbkPu + 32768) >> 6));
        break;

        case 45:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_asrSpeedPIOut.outPu + 32768) >> 6));
        break;

        case 56:
            MC_DEBUG_SPI_WriteChannelVaule(channel, (g_hallSensor.hallStatus << 7));
        break;

        case 138:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_udqDcpOut.dcpUdPu + 32768) >> 6));
        break;

        case 139:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_udqDcpOut.dcpUqPu + 32768) >> 6));
        break;

        case 142:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->iaPu + 32768) >> 6));
        break;

        case 143:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->ibPu + 32768) >> 6));
        break;

        case 144:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->ialpha + 32768) >> 6));
        break;

        case 145:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->ibeta + 32768) >> 6));
        break;

        case 146:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->iqFdb + 32768) >> 6));
        break;

        case 147:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->idFdb + 32768) >> 6));
        break;

        case 148:
            MC_DEBUG_SPI_WriteChannelVaule(channel, (((int16_t)g_focVars.pCtrl->idq + 32768) >> 6));
        break;

        case 149:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->iqRef + 32768) >> 6));
        break;

        case 150:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->idRef + 32768) >> 6));
        break;

        case 151:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->vq + 32768) >> 6));
        break;

        case 152:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->vd + 32768) >> 6));
        break;

        case 153:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->vdqLimit + 32768) >> 6));
        break;

        case 154:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->valpha + 32768) >> 6));
        break;

        case 155:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_focVars.pCtrl->vbeta + 32768) >> 6));
        break;

        case 157:
            MC_DEBUG_SPI_WriteChannelVaule(channel, (g_focVars.pCtrl->elecAngle >> 6));
        break;

        case 158:
            MC_DEBUG_SPI_WriteChannelVaule(channel, (g_startUpCtrl.elecThetaEst >> 6));
        break;

        case 163:
            MC_DEBUG_SPI_WriteChannelVaule(channel, (g_hfinj.hfiEstTheta >> 6));
        break;

        case 164:
            MC_DEBUG_SPI_WriteChannelVaule(channel, ((g_hfinj.hfiEstFreq + 32768) >> 6));
        break;

        default:
            MC_DEBUG_SPI_WriteChannelVaule(channel, (g_focVarsCtrl.elecAngle >> 6));
        break;
    }
}

/*!
* @brief Watch Debug variable in dac channnel.
*
* @param[in] channel : 0 or 1, select the DAC channel
* @param[in] paramId : 0--255, ID of debug variable
* @return none
*/
void Debug_Dac_Watch(pwm_channel_type_t channel, uint16_t paramId)
{
    if (DAC_PWM == ENABLE)
    {
        Debug_Dac_Watch_PWM(channel, paramId);
    }
    if (DAC_SPI == ENABLE)
    {
        Debug_Dac_Watch_SPI(channel, paramId);
    }
}

/*!
* @brief Display the debug variables on oscilloscope.
*
* @param[in] none
* @return none
*/
void Debug_Display_Variables(void)
{
    if (DEBUG_DAC_CH0 == ENABLE)
    {
        Debug_Dac_Watch(PWM_CH_0, g_dispCh0);
    }

    if (DEBUG_DAC_CH1 == ENABLE)
    {
        Debug_Dac_Watch(PWM_CH_1, g_dispCh1);
    }
}

/* =============================================  EOF  ============================================== */
