/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file param_identify_api.c
*
* @brief This file contain four motor parameters identification interface, include Rs Ld/Lq Flux Inertia. These parameters can be used for sensorless control.
*
*/
/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "foc_isr.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Ld Lq Parameter identify function initialize configuration
*
* @param[in] none
* @return none
*/
void Ldq_IdentifyConfig(void)
{
    /*! three shunts sample mode, will change adc sample sequence. Unify two shunts and three shunts mode */
    ADC_SetInjectLength(ADC0, 3);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_0, ADC_CH_8);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_1, ADC_CH_2);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_2, ADC_CH_3);
}

/*!
* @brief Ld Lq Parameter identify update pwm period value.
*
* @param[in] period: Updata pwm period value
* @return none
*/
void Ldq_IdentifyUpdataPrd(uint16_t period)
{
    PWM_DRV_SetMaxCountValue(PWM_MODULER_MOTOR1, period);
    PWM_DRV_SetSoftwareTriggerSync(PWM_MODULER_MOTOR1, ENABLE);
}

/*!
* @brief Ld Lq Parameter identify update pwm period value in Method High frequency sine wave.
*
* @param[in] pwmFreq: Updata pwm frequency value
* @param[in] period: Updata pwm period value
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void ParamId_UpdatePwmPrd(uint16_t pwmFreq, uint16_t period, STC_PARAM *STC)
{
    STC->pFOCVars->pCfg->pwmPeriod = period;
    STC->pFOCVars->pCfg->halfPwmPeriod = period >> 1;
    STC->pFOCVars->pCfg->pwmFreq = pwmFreq;
    STC->pFOCVars->pCfg->curFreq = pwmFreq;

    PWM_DRV_SetMaxCountValue(PWM_MODULER_MOTOR1, period);

    PWM_DRV_SetSoftwareTriggerSync(PWM_MODULER_MOTOR1, ENABLE);
}

/*!
* @brief Ld Lq Parameter identify finished, and call this function to reset configuration.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Ldq_IdentifyConfigReset(FOC_VARS *focVars)
{
    PWM_DRV_SetMaxCountValue(PWM_MODULER_MOTOR1, (focVars->pCfg->pwmPeriod));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, (focVars->pCfg->pwmPeriod >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, (focVars->pCfg->pwmPeriod >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, (focVars->pCfg->pwmPeriod >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, (focVars->pCfg->pwmPeriod >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, (focVars->pCfg->pwmPeriod >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, (focVars->pCfg->pwmPeriod >> 1));

    PWM_DRV_SetSoftwareTriggerSync(PWM_MODULER_MOTOR1, ENABLE);
#if (CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE)
    ADC_SetInjectLength(ADC0, 3);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_0, ADC_CH_6);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_1, ADC_CH_8);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_2, ADC_CH_2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    ADC_SetInjectLength(ADC0, 2);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_0, ADC_CH_8);
    ADC_SetInjectConversionChannel(ADC0, ADC_RSEQ_1, ADC_CH_2);
#endif
}

/*!
* @brief Ld Lq Parameter identify set different PWM output. Cotain six direction pulse output, and recover PWM configuration.
*        This function is called by Ld Lq identify isr function.
*
* @param[in] section: Select different PWM output mode
* @return none
*/
void ParamIdentify_LdqSetPwm(uint8_t section)
{   /*!< PWM2_SYNCONF->SWOC = 0,CHOSWCR control reload immediately */
    switch (section)
    {
        case 0:     /*!< A+B- */
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_LOW_LEVEL, true);
        break;

        case 1:     /*!< A-B+ */
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_LOW_LEVEL, true);
        break;

        case 2:     /*!< B+C- */
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_LOW_LEVEL, true);
        break;

        case 3:     /*!< B-C+ */
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_LOW_LEVEL, true);
        break;

        case 4:     /*!< C+A- */
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_LOW_LEVEL, true);
        break;

        case 5:     /*!< C-A+ */
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_HIGH_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_LOW_LEVEL, true);
        break;

        case 6:     /*!< recover */
        PWM_DRV_SetChannelSoftControl(PWM_MODULER_MOTOR1, PWM_CH_0, false);
        PWM_DRV_SetChannelSoftControl(PWM_MODULER_MOTOR1, PWM_CH_1, false);
        PWM_DRV_SetChannelSoftControl(PWM_MODULER_MOTOR1, PWM_CH_2, false);
        PWM_DRV_SetChannelSoftControl(PWM_MODULER_MOTOR1, PWM_CH_3, false);
        PWM_DRV_SetChannelSoftControl(PWM_MODULER_MOTOR1, PWM_CH_4, false);
        PWM_DRV_SetChannelSoftControl(PWM_MODULER_MOTOR1, PWM_CH_5, false);
        break;

        case 7:
        default:
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_0, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_1, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_2, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_3, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_4, PWM_LOW_LEVEL, true);
        PWM_DRV_SoftControlChannelLevel(PWM_MODULER_MOTOR1, PWM_CH_5, PWM_LOW_LEVEL, true);
        break;
    }
}

/*!
* @brief Get parameter identification duty.
*
* @param[in] paramIdent: pointer to PARAM_IDENTIFY_STRUCT structure
* @return Schedule duty of parameter identify
*/
uint16_t ParamIdentify_DutyCheck(PARAM_IDENTIFY_STRUCT *paramIdent)
{
    PARAM_IDENTIFY_MAIN_CTRL *parIdCtrl = paramIdent->pParamIdentifyCtrl;
    PARAM_RS_IDENTIFY_DATA *rsId = paramIdent->pParamRsIdentify;
    PMSM_LDQID_PARAM *ldqIdHFS = paramIdent->pmsmLdqIdentify;
    PARAM_LDQ_IDENTIFY_DATA *ldqIdHFP = paramIdent->pParamLdqIdentify;
    PARAM_FLUX_IDENTIFY_DATA *fluxId = paramIdent->pParamFluxIdentify;
    PARAM_IDENTIFY_CFG *idCfg = paramIdent->pParamIdentifyCfg;
    uint16_t duty = 0;    /* Progress of each step */
    uint16_t step = 0;    /* Step of parameter identification */
    uint16_t retVal = 0;  /* Synthetic value of step and progress, used to transfer to the motor-tool */

    switch (idCfg->identifyParSel)
    {
    case 1: /* only Rs */
        switch (parIdCtrl->mainStep)
        {
        case IDENTIFY_RS_DCTEST: /* do Rs */
            step = 1;
            switch (rsId->phaseOrder)
            {
            case 0:
                if (rsId->subStep < 7)
                {
                    duty = ((uint32_t)rsId->subStep * 110 / 6) * 3;
                }
                else
                {
                    duty = 330;
                }
            break;

            case 1:
                if (rsId->subStep < 7)
                {
                    duty = ((uint32_t)rsId->subStep * 110 / 6) * 3 + 330;
                }
                else
                {
                    duty = 660;
                }
            break;

            case 2:
                if (rsId->subStep < 7)
                {
                    duty = ((uint32_t)rsId->subStep * 110 / 6) * 3 + 660;
                }
                else
                {
                    duty = 990;
                }
            break;

            default:
            break;
            }
            duty = duty / 10;
        break;

        default:
            step = 15;  /* finish */
            duty = 100;
        break;
        }
    break;

    case 2: /* only Ldq */
        if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFS)
        {
            switch (parIdCtrl->mainStep)
            {
                case IDENTIFY_LDQ_STALLTEST:
                    step = 2;
                    duty = ((uint32_t)ldqIdHFS->ldqIdParam->idStep * 10 / 9) * 10;
                break;

                default:
                    step = 15;  /* finish */
                    duty = 100;
                break;
            }
        }
        else if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFP)
        {
            /* should to Rs at the beginning */
            switch (parIdCtrl->mainStep)
            {
                case IDENTIFY_RS_DCTEST: /* do Rs */
                    step = 1;
                    switch (rsId->phaseOrder)
                    {
                        case 0:
                            if (rsId->subStep < 7)
                            {
                                duty = (uint32_t)rsId->subStep * 170 / 6;
                            }
                            else
                            {
                                duty = 170;
                            }
                        break;

                        case 1:
                            if (rsId->subStep < 7)
                            {
                                duty = (uint32_t)rsId->subStep * 170 / 6 + 170;
                            }
                            else
                            {
                                duty = 340;
                            }
                        break;

                        case 2:
                            if (rsId->subStep < 7)
                            {
                                duty = (uint32_t)rsId->subStep * 170 / 6 + 340;
                            }
                            else
                            {
                                duty = 500;
                            }
                        break;

                        default:
                            duty = 500;
                        break;
                    }
                    duty = duty / 10;
                break;

                case IDENTIFY_LDQ_STALLTEST: /* do Ldq */
                    step = 2;
                    duty = ((uint32_t)ldqIdHFP->step * 10 / 8) * 5 + 50;
                break;

                default:
                    step = 15;  /* finish */
                    duty = 100;
                break;
            }
        }
        else
        {
        }
    break;

    case 3: /* Rs and Ldq */
        switch (parIdCtrl->mainStep)
        {
            case IDENTIFY_RS_DCTEST: /* do Rs */
                step = 1;
                switch (rsId->phaseOrder)
                {
                    case 0:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 170 / 6;
                        }
                        else
                        {
                            duty = 170;
                        }
                    break;

                    case 1:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 170 / 6 + 170;
                        }
                        else
                        {
                            duty = 340;
                        }
                    break;

                    case 2:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 170 / 6 + 340;
                        }
                        else
                        {
                            duty = 500;
                        }
                    break;

                    default:
                        duty = 500;
                    break;
                }
                duty = duty / 10;
            break;

            case IDENTIFY_LDQ_STALLTEST: /* do Ldq */
                step = 2;
                if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFP)
                {
                    duty = ((uint32_t)ldqIdHFP->step * 10 / 8) * 5 + 50;
                }
                else if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFS)
                {
                    duty = ((uint32_t)ldqIdHFS->ldqIdParam->idStep * 10 / 9) * 5 + 50;
                }
                else
                {
                }
            break;

            default:
                step = 15;  /* finish */
                duty = 100;
            break;
        }
    break;

    case 4: /* only flux */
        switch (g_paramIdentifyCtrl.mainStep)
        {
            case IDENTIFY_FLUX_ROTATIONTEST: /* do flux */
                step = 3;
                duty = ((uint32_t)fluxId->subStep * 10 / 9) * 10;
            break;

            default:
                step = 15;  /* finish */
                duty = 100;
            break;
        }
    break;

    case 5: /* Rs and flux */
        switch (parIdCtrl->mainStep)
        {
            case IDENTIFY_RS_DCTEST: /* do Rs */
                step = 1;
                switch (rsId->phaseOrder)
                {
                    case 0:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 170 / 6;
                        }
                        else
                        {
                            duty = 170;
                        }
                    break;

                    case 1:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 170 / 6 + 170;
                        }
                        else
                        {
                            duty = 340;
                        }
                    break;

                    case 2:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 170 / 6 + 340;
                        }
                        else
                        {
                            duty = 500;
                        }
                    break;

                    default:
                        duty = 500;
                    break;
                }
                duty = duty / 10;
            break;

            case IDENTIFY_FLUX_ROTATIONTEST: /* do flux */
                step = 3;
                duty = (uint32_t)fluxId->subStep * 5 + 50;
            break;

            default:
                step = 15;  /* finish */
                duty = 100;
            break;
        }
    break;

    case 6: /* Ldq and flux */
        if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFP)
        {
            /* should do Rs to get Ldq */
            switch (parIdCtrl->mainStep)
            {
                case IDENTIFY_RS_DCTEST: /* do Rs */
                    step = 1;
                    switch (rsId->phaseOrder)
                    {
                        case 0:
                            if (rsId->subStep < 7)
                            {
                                duty = (uint32_t)rsId->subStep * 110 / 6;
                            }
                            else
                            {
                                duty = 110;
                            }
                        break;

                        case 1:
                            if (rsId->subStep < 7)
                            {
                                duty = (uint32_t)rsId->subStep * 110 / 6 + 110;
                            }
                            else
                            {
                                duty = 220;
                            }
                        break;

                        case 2:
                            if (rsId->subStep < 7)
                            {
                                duty = (uint32_t)rsId->subStep * 110 / 6 + 220;
                            }
                            else
                            {
                                duty = 330;
                            }
                        break;

                        default:
                            duty = 330;
                        break;
                    }
                    duty = duty / 10;
                break;

                case IDENTIFY_LDQ_STALLTEST: /* do Ldq */
                    step = 2;
    //                duty = ((uint32_t)g_paramLdqIdentify.step * 11 / 8) * 3 + 33;
                    duty = ((uint32_t)ldqIdHFP->step * 11 / 8) * 3 + 33;
                break;

                case IDENTIFY_FLUX_ROTATIONTEST: /* do flux */
                    step = 3;
                    duty = ((uint32_t)fluxId->subStep * 11 / 9) * 3 + 66;
                break;

                default:
                    step = 15;  /* finish */
                    duty = 100;
                break;
            }
        }
        else if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFP)
        {
            switch (parIdCtrl->mainStep)
            {
                case IDENTIFY_LDQ_STALLTEST: /* do Ldq */
                    step = 2;
                    duty = ((uint32_t)ldqIdHFS->ldqIdParam->idStep * 10 / 8) * 5;
                break;

                case IDENTIFY_FLUX_ROTATIONTEST: /* do flux */
                    step = 3;
                    duty = ((uint32_t)fluxId->subStep * 10 / 9) * 5 + 50;
                break;

                default:
                    step = 15;  /* finish */
                    duty = 100;
                break;
            }
        }
        else
        {
        }

    break;

    case 7: /* Rs and Ldq and flux */
        switch (parIdCtrl->mainStep)
        {
            case IDENTIFY_RS_DCTEST: /* do Rs */
                step = 1;
                switch (rsId->phaseOrder)
                {
                    case 0:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 110 / 6;
                        }
                        else
                        {
                            duty = 110;
                        }
                    break;

                    case 1:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 110 / 6 + 110;
                        }
                        else
                        {
                            duty = 220;
                        }
                    break;

                    case 2:
                        if (rsId->subStep < 7)
                        {
                            duty = (uint32_t)rsId->subStep * 110 / 6 + 220;
                        }
                        else
                        {
                            duty = 330;
                        }
                    break;

                    default:
                        duty = 330;
                    break;
                }
                duty = duty / 10;
            break;

            case IDENTIFY_LDQ_STALLTEST: /* do Ldq */
                step = 2;
//                duty = ((uint32_t)g_paramLdqIdentify.step * 11 / 8) * 3 + 33;
                if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFP)
                {
                    duty = ((uint32_t)ldqIdHFP->step * 11 / 8) * 3 + 33;
                }
                else if (idCfg->pmsmLdqTypeSel == PMSM_LDQ_METHOD_HFS)
                {
                    duty = ((uint32_t)ldqIdHFS->ldqIdParam->idStep * 11 / 9) * 3 + 33;
                }
                else
                {
                }
            break;

            case IDENTIFY_FLUX_ROTATIONTEST: /* do flux */
                step = 3;
                duty = ((uint32_t)fluxId->subStep * 11 / 9) * 3 + 66;
            break;

            default:
                step = 15;  /* finish */
                duty = 100;
            break;
        }
    break;

    default:
        step = 14;  /* error */
        duty = 0;
    break;
    }
    retVal = (duty << 8) + step;
    return retVal;
}

/*!
* @brief Get parameter identification results to save.
*
* @param[in] paramIdentify: pointer to PARAM_IDENTIFY_STRUCT structure
* @param[in] paramBuffer: pointer to PARAM_IDENTIFY_BUFFER structure
* @return none
*/
void ParameterIdentifySaveProcess(PARAM_IDENTIFY_STRUCT *paramIdentify, PARAM_IDENTIFY_BUFFER *paramBuffer)
{
    paramBuffer->rsId = paramIdentify->pParamIdentifyCtrl->rsId;
    paramBuffer->rsIdPhaseA = paramIdentify->pParamRsIdentify->rsPhaseAId;
    paramBuffer->rsIdPhaseB = paramIdentify->pParamRsIdentify->rsPhaseBId;
    paramBuffer->rsIdPhaseC = paramIdentify->pParamRsIdentify->rsPhaseCId;
    paramBuffer->ld = paramIdentify->pParamIdentifyCtrl->ld;
    paramBuffer->lq = paramIdentify->pParamIdentifyCtrl->lq;
    paramBuffer->inertia = paramIdentify->pParamIdentifyCtrl->inertia;
    paramBuffer->bEmf = paramIdentify->pParamIdentifyCtrl->bEmf;
}

/*!
* @brief Get hall self-learning duty.
*
* @param[in] hallSensor: pointer to HALL_SENSOR structure
* @return Schedule duty of hall self-learning
*/
uint16_t HallSelfLearnDutyCheck(HALL_SENSOR *hallSensor)
{
    uint16_t retVal = 0;  /* Synthetic value of step and progress, used to transfer to the motor-tool */
    uint16_t duty = 0;    /* Progress of each step */
    uint16_t step = 0;    /* Step of hall self-learning */
    if (hallSensor->pHallCfg->hallSelfLearnMode == 1)
    {
        if (g_hallSelfLearnFinishFlag == 0)
        {
            if (hallSensor->pHallSelfLearn->selfLearnStage == SELF_LEARN_START)
            {
                step = 1;
                duty = 0;
            }
            else if (hallSensor->pHallSelfLearn->selfLearnStage == SELF_LEARN_10DEGREE_FINISHED)
            {
                step = 2;
                duty = 50;
            }
            else
            {
            }
        }
        else
        {
            step = 15;
            duty = 100;
        }
    }
    else if (hallSensor->pHallCfg->hallSelfLearnMode == 2)
    {
        if (g_hallSelfLearnFinishFlag == 0)
        {
            if (hallSensor->pHallSelfLearn->selfLearnStage == SELF_LEARN_START)
            {
                step = 1;
                duty = 0;
            }
            else if (hallSensor->pHallSelfLearn->selfLearnStage == SELF_LEARN_10DEGREE_FINISHED)
            {
                step = 2;
                duty = 33;
            }
            else if (hallSensor->pHallSelfLearn->selfLearnStage == SELF_LEARN_2DEGREE_FINISHED)
            {
                step = 3;
                duty = 66;
            }
            else
            {

            }
        }
        else
        {
            step = 15;
            duty = 100;
        }
    }
    else
    {
        step = 14;
        duty = 0;
    }
    retVal = (duty << 8) + step;
    return retVal;
}

/*!
* @brief Get hall self learn results to save.
*
* @param[in] hallBuffer: pointer to HALL_SELFLEARN_BUFFER structure
* @param[in] hallSensor: pointer to HALL_SENSOR structure
* @return none
*/
void HallSelfLearnSaveProcess(HALL_SELFLEARN_BUFFER *hallBuffer, HALL_SENSOR *hallSensor)
{
    hallBuffer->selfLearnOrder = hallSensor->pHallSelfLearn->selfLearnOrder;
    hallBuffer->selfLearnTheta = hallSensor->pHallSelfLearn->selfLearnTheta;
    hallBuffer->sectorWidthMax = hallSensor->pHallSelfLearn->sectorWidthMax;
    hallBuffer->sectorWidthMin = hallSensor->pHallSelfLearn->sectorWidthMin;
}

/*!
* @brief Hall sensor self learning results buffer initilization.
*
* @param[in] hallBuffer: pointer to HALL_SELFLEARN_BUFFER structure
* @return none
*/
void HallSelfLearnResultInit(HALL_SELFLEARN_BUFFER *hallBuffer)
{
    hallBuffer->selfLearnOrder = 0;
    hallBuffer->selfLearnTheta = 0;
    hallBuffer->sectorWidthMax = 0;
    hallBuffer->sectorWidthMin = 0;
}

/* =============================================  EOF  ============================================== */
