/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file parameters_config.c
*
* @brief This file provides global structure parameters initialization.
*
*/

/* ===========================================  Includes  =========================================== */
#include "parameters_config.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief Motor parameters.
*/
MOTOR_PARAM                 defMotorParam =
{
    .motorType              = MOTOR_TYPE,                       /*!< Select motor type, 1, acim; 2, bldc; 3, spmsm; 4,ipmsm */
    .statorRs               = STATOR_RS,                        /*!< Motor stator resistance value */
    .statorLs               = LS,                               /*!< Motor stator inductance value */
    .ld                     = LD,                               /*!< Motor D-axis inductance value */
    .lq                     = LQ,                               /*!< Motor Q-axis inductance value */
    .rotorRs                = ROTOR_RS,                         /*!< Motor rotor resistance value */
    .lr                     = LR,                               /*!< Motor rotor inductance value */
    .lm                     = LM,                               /*!< Motor mutual inductance value */
    .lleak                  = (LR - LM),                        /*!< Motor leakage inductance value */
    .noLoadExcite           = NO_LOAD_EXCITATION_CURRENT,       /*!< Motor no load exciting current value */

    .jm                     = (uint32_t)(INERTIA + 0.5),        /*!< Motor inertia value */
    .polePairs              = POLE_PAIR_NUM,                    /*!< Motor pole pairs value */
    .encoderLines           = PULSE_PER_REVOLUTION,             /*!< encoder pulse lines per motor mechanical cycle */
    .bEmf                   = VOLTAGE_CONSTANT,                 /*!< Motor back EMF constant  */
    .maxCur                 = MAX_CURRENT,                      /*!< Max stator current value */
    .ratedVoltage           = NORMAL_BUS_VOLTAGE,               /*!< DC bus voltage of inverter system */
    .maxSpeed               = MOTOR_MAX_SPEED_RPM,              /*!< Motor running speed max value */
    .maxFreq                = BASE_FREQ,                        /*!< Motor current frequency max value */

    .maxTorque              = MAX_TORQUE,                       /*!< Motor electromagnetic torque max value */
    .maxPower               = RATE_POWER,                       /*!< Motor rated power */
};

/*!
* @brief System hardware parameters.
*/
HW_BOARD                    defHwBoard =
{
    .pwmPolarity            = PHASE_UVW_POLARITY,               /*!< Three phase polarity of inverter */
    .hwDeadTime             = HW_DEAD_TIME_NS,                  /*!< Hardware dead time of power device in inverter */
    .swDeadTime             = SW_DEAD_TIME_NS,                  /*!< Configurable software dead time of power device in inverter */
    .maxVoltage             = MAX_BUS_VOLTAGE,                  /*!< Max DC Bus voltage */
    .normalVoltage          = NORMAL_BUS_VOLTAGE,               /*!< Normal DC Bus voltage */
    .minVoltage             = MIN_BUS_VOLTAGE,                  /*!< Normal DC Bus voltage */
    .vbusAttenuateVal       = VBUS_ATTENUATE_FACTOR,            /*!< Voltage sampling circuit attenuation factor */

    .maxCurrent             = HW_MAX_CURRENT,                   /*!< Inverter hardware max current */
    .rShunt                 = RSHUNT,                           /*!< Sampling resistance value */
    .opGainVal              = OP_AMPLIFICATION_GAIN,            /*!< Operation amplifier gain in current sampling circuit */
    .icsGainVal             = ICS_GAIN,                         /*!< Current sensor gain */
    .adcRange               = ADC_RANGE,                        /*!< ADC range */
    .adcRefV                = VSVREF,                           /*!< ADC reference voltage */
    .overCurrentPolarity    = (uint16_t)OVERCUR_FEEDBACK_POLARITY,  /*!< Hardware over current fault polarity */
};

/*!
* @brief FOC vector control configuration parameters.
*/
FOC_VARS_CFG                defFocVars =
{
    .motorDir               = MOTOR_DIR,                        /*!< Motor rotation direction */
    .controlMode            = MOTOR_CONTROL_MODE,               /*!< Motor control mode */
    #if (defined VOLTAGE_RESTRUCT)
    .voltObtainWay          = 1,                                /*!< 1: use voltage restruct to obtain sensorless input voltage */
    #else
    .voltObtainWay          = 0,                                /*!< 0: use svpwm input voltage to obtain sensorless input voltage */
    #endif
    #if (defined PLL_ANGLE_ESTIMATION)
    .angleObtainWay         = 1,                                /*!< 1: pll angle obtain method */
    #else
    .angleObtainWay         = 0,                                /*!< 0: atan angle obtain method */
    #endif
    #if (defined SMC_OBSERVE)
        #if (defined FULL_ORDER_SMC)
        .smoWay = 1,                                            /*!< 1: full order smo control */
        #else
        .smoWay = 0,                                            /*!< 0: two order smo control */
        #endif
    #endif
    .fullOrderSmoFilt       = (int16_t)FULL_SMC_FILT,           /*!< Full order smc filter enable switch, 1: means enable filt, 0: means disable filt */

    .speedMaxLimit          = MAX_SPEED_VALUE,                  /*!< Motor speed max limit */
    #if (defined HIGH_FREQUENCE_INJECTION)
    .speedMinLimit          = HFI_INIT_FREQ,                    /*!< Motor speed min limit */
    #else
    .speedMinLimit          = MIN_SPEED_VALUE,                  /*!< Motor speed min limit */
    #endif
    .stopSpeed              = STOP_SPEED_VALUE,                 /*!< Slow down speed limit */

    .motorAcceleration      = ACCELERATION_RPS,                 /*!< Acceleration of motor speed up */
    .motorDeceleration      = DECELERATION_RPS,                 /*!< Deceleration of motor speed reduction */

    .pwmFreq                = FPWM_Hz,                          /*!< PWM module counting frequency */
    .curFreq                = FTBC_Hz,                          /*!< Current calculation cycle frequency */
    .speedFreq              = FTBS_Hz,                          /*!< Speed calculation cycle frequency */
    .baseFreq               = BASE_FREQ,                        /*!< Motor frequency reference base value */

    .msUnitInt              = MS_UINT_INT,                      /*!< Millisecond unit */
    .halfPwmPeriod          = (PWM_PERIOD_VALUE >> 1),          /*!< PWM half period */
    .pwmPeriod              = PWM_PERIOD_VALUE,                 /*!< PWM period */
    .pwmDeadTime            = PWM_DEAD_TIME,                    /*!< PWM dead time */

    .sysBusClk              = MOTOR_BUS_CLK,                    /*!< MCU system bus clock */
    .pwdtPrescal            = HALL_CLK_PRESCALER,               /*!< PWDT clock prescaler */
    .curSampleMode          = CUR_SAMPLE_MODE,                  /*!< Current sampling mode: 1-shunt, 2-shunt, 3-shunt, ICS */

    .atanModeSpeedFilterH    = ATAN_SPEED_FILT_H,               /*!< Low pass filter coefficient used for ELEC_FREQUENCE_HIGH */
    .atanModeSpeedFilterL    = ATAN_SPEED_FILT_L,               /*!< Low pass filter coefficient used for ELEC_FREQUENCE_LOW */
};

/*!
* @brief Startup control configuration parameters.
*/
START_UP_CFG                defStartUp =
{
    .curStartOffset         = OPEN_LOOP_CURRENT,                /*!< FOC open loop current value */

    .curStartSub            = CUR_START_SUB,                    /*!< Decrement calue of current calculation */
    .synSpd                 = (int16_t)OPEN_LOOP_SPEED_HZ,      /*!< Target Synchronization speed of open loop */

    .synTime                = OPEN_LOOP_TIME_SYN,               /*!< Synchronization times to reach openloop target speed */

#if (defined ALIGNMENT_ONCE) || (defined ALIGNMENT_TWICE)
    .alignAngle             = ALIGNMENT_ANGLE,                  /*!< Angle of pole alignment */
    .alignTime              = T_ALIGNMENT1,                     /*!< Time of pole alignment */
    .alignTimeTwice         = T_ALIGNMENT2,                     /*!< Time of the second pole alignment */
    .alignCur               = I_ALIGNMENT1,                     /*!< Motor current with pole alignment */
    .alignCurTwice          = I_ALIGNMENT2,                     /*!< Motor current with pole alignment */
    .curRamp1Init           = I_RAMP1_INIT,                     /*!< Initial value of first alignment current ramp */
    .curRamp2Init           = I_RAMP2_INIT,                     /*!< Initial value of second alignment current ramp */
    .slopeDuration1         = T_SLOPE1,                         /*!< Duration of first alignment current ramp in ms */
    .slopeDuration2         = T_SLOPE2,                         /*!< Duration of second alignment current ramp in ms */
#endif

    .spdSmoothCur           = SPD_SMOOTH_CUR,                   /*!< Motor current with pole alignment */

    .curveFreqStart         = FREQ_REF_INITVALUE,               /*!< Frequency initial value of open loop start curve */
    .curveCurStart          = CUR_REF_INITVALUE,                /*!< Current initial value of open loop start curve */
    .curveVoltStart         = VOLT_REF_INITVALUE,               /*!< Voltage initial value of open loop start curve */
    .curveFreqSet1          = FREQ_REF_VALUE_1,                 /*!< Frequency set value of point 1 of open loop start curve */
    .curveFreqSet2          = FREQ_REF_VALUE_2,                 /*!< Frequency set value of point 2 of open loop start curve */
    .curveFreqSet3          = FREQ_REF_VALUE_3,                 /*!< Frequency set value of point 3 of open loop start curve */
    .curveFreqSet4          = FREQ_REF_VALUE_4,                 /*!< Frequency set value of point 4 of open loop start curve */
    .curveFreqSet5          = FREQ_REF_VALUE_5,                 /*!< Frequency set value of point 5 of open loop start curve */
    .curveCurSet1           = CUR_REF_VALUE_1,                  /*!< Current set value of point 1 of open loop start curve */
    .curveCurSet2           = CUR_REF_VALUE_2,                  /*!< Current set value of point 2 of open loop start curve */
    .curveCurSet3           = CUR_REF_VALUE_3,                  /*!< Current set value of point 3 of open loop start curve */
    .curveCurSet4           = CUR_REF_VALUE_4,                  /*!< Current set value of point 4 of open loop start curve */
    .curveCurSet5           = CUR_REF_VALUE_5,                  /*!< Current set value of point 5 of open loop start curve */
    .curveVoltSet1          = VOLT_REF_VALUE_1,                 /*!< Voltage set value of point 1 of open loop start curve */
    .curveVoltSet2          = VOLT_REF_VALUE_2,                 /*!< Voltage set value of point 2 of open loop start curve */
    .curveVoltSet3          = VOLT_REF_VALUE_3,                 /*!< Voltage set value of point 3 of open loop start curve */
    .curveVoltSet4          = VOLT_REF_VALUE_4,                 /*!< Voltage set value of point 4 of open loop start curve */
    .curveVoltSet5          = VOLT_REF_VALUE_5,                 /*!< Voltage set value of point 5 of open loop start curve */
    .curveSegNum            = CUSTOMER_CURVE_POINTS,            /*!< Segment number of open loop start curve */
    .curveTime1             = TIME_DURATION1,                   /*!< Time duration value of open loop start curve segment 1 */
    .curveTime2             = TIME_DURATION2,                   /*!< Time duration value of open loop start curve segment 2 */
    .curveTime3             = TIME_DURATION3,                   /*!< Time duration value of open loop start curve segment 3 */
    .curveTime4             = TIME_DURATION4,                   /*!< Time duration value of open loop start curve segment 4 */
    .curveTime5             = TIME_DURATION5,                   /*!< Time duration value of open loop start curve segment 5 */
};

/*!
* @brief Pulse injection startup configuration parameters.
*/
#if (defined PULSE_INJECTION)
PULSE_INJ_CFG               defPulseInj =
{
    .selfLearnThres         = SELF_LEARN_THRES,                 /*!< initial position self learn current threshold */
    .selfLearnDutyStep      = SELF_LEARN_DUTY_STEP,             /*!< initial position self learn pwm duty increase step */
};
#endif

/*!
* @brief Rotation high frequency voltage injection initial position check configuration parameters.
*/
#if (defined INIT_POS_DETEC)
INITPOS_DETEC_CFG           defInitPosDetec = 
{
    .pwmFreqDef             = FPWM_Hz,                          /*!< FOC PWM frequency default value */
    .pwmPrdDef              = PWM_PERIOD_VALUE,                 /*!< FOC PWM period default value */
    .injVoltFreq            = INITPOS_VOLT_FREQ,                /*!< Rotation voltage injection frequency */
    .injVoltAmp             = INITPOS_VOLT_AMP,                 /*!< Rotation voltage injection amplitude */
    .pulseTime              = INITPOS_PULSE_INJECT_TIME,        /*!< Pulse voltage injection time */
    .pulseClearTime         = INITPOS_PULSE_CLEAR_TIME,         /*!< Pulse voltage injection clear time */
    .pulseAmp               = INITPOS_PULSE_AMP,                /*!< Pulse voltage injection amplitude */
};
#endif

/*!
* @brief High frequency injection algorithm configuration parameters.
*/
//#if (defined HIGH_FREQUENCE_INJECTION)
HFI_CFG                     defHighFreqInj =
{
    .hfiDebugEnable         = HFI_DEBUG_MODE,                   /*!< High frequency injection debug mode enable switch */
    .hfiToStoEnable         = HFI_TO_STO,                       /*!< High frequency injection switch to high speed strategy enable switch */
    .hfiFreq                = HFI_FREQUENCY,                    /*!< High frequency injection frequency */
    .vPeak                  = HFI_AMPLITUDE,                    /*!< High frequency injection voltage amplitude */
    .pulseAmp               = HFI_PULSE_AMP,                    /*!< Pulse voltage amplitude */
    .pulseTime              = HFI_PULSE_TIME,                   /*!< Pulse voltage inject time */
    .clearTime              = HFI_CLEAR_TIME,                   /*!< Current clear time */
    .msUnit                 = MS_UINT_INT,                      /*!< High frequency injection ms unit */
    .hfiInitTime            = HFI_INIT_TIME,                    /*!< High frequency injection initial inject time */
    .hfiInitTheta           = HFI_INIT_THETA,                   /*!< High frequency injection initial theta */
    .hfiPllKp               = HFI_PLL_KP_DEFAULT,               /*!< High frequency injection pll kp */
    .hfiPllKi               = HFI_PLL_KI_DEFAULT,               /*!< High frequency injection pll ki */
    .hfiVdFreqHigh          = HFI_VD_FREQ_HIGH,                 /*!< High frequency injection voltage amplitude calculate frequency high */
    .hfiVdFreqLow           = HFI_VD_FREQ_LOW,                  /*!< High frequency injection voltage amplitude calculate frequency low */
    .bpfA0                  = BPF_A0,                           /*!< Band pass filter A0 */
    .bpfA2                  = BPF_A2,                           /*!< Band pass filter A2 */
    .bpfB1                  = BPF_B1,                           /*!< Band pass filter B1 */
    .bpfB2                  = BPF_B2,                           /*!< Band pass filter B2 */
    .lpfA0                  = LPF_A0,                           /*!< Low pass filter A0 */
    .lpfA1                  = LPF_A1,                           /*!< Low pass filter A1 */
    .lpfA2                  = LPF_A2,                           /*!< Low pass filter A2 */
    .lpfB1                  = LPF_B1,                           /*!< Low pass filter B1 */
    .lpfB2                  = LPF_B2,                           /*!< Low pass filter B2 */
    .thetaLpfCoeff          = THETA_LPF_COEFF,                  /*!< 1 order Low pass filter coefficient */
};
//#endif

/*!
* @brief Power estimation algorithm configuration parameters.
*/
POWEREST_CFG                defPowerEst =
{
    .uqcomp                 = UQCOMP,                           /*!< Dead zone q axis voltage configuration compensation value */
    .pwmDelayCof            = PWM_DELAY_COF,                    /*!< Power estimate pwm delay configuration coefficient */
};

/*!
* @brief Hall sensor vector control algorithm configuration parameters.
*/
HALL_SENSOR_CFG              defHallSensor =
{
    .hallSelfLearnMode      = HALL_SELF_LEARN_MODE,             /*!< Hall self learn mode, 1: learn order and theta; 2: learn order and theta and sector width */
    .hallCalcMode           = ORDER_ERROR_COMP_MODE,            /*!< Hall calculate rotor position mode */
    .hallSelfLearnEnable    = (uint16_t)HALL_SENSOR_SELF_LEARN, /*!< Hall self learn enable */
    .hallSwapTrqCoff        = HALL_SWAP_TRQ_COFF,               /*!< Hall torque boost cofficient at low speed */
    .learnIntervalTime      = HALL_SELF_LEARN_INTERVAL,         /*!< Hall self learn interval time */
    .learnCur               = HALL_SELF_LEARN_CUR,              /*!< Hall self learn current */
    .learnOrder             = HALL_SELF_LEARN_ORDER,            /*!< Hall self learn finished get hall order */
    .learnDegree            = HALL_SELF_LEARN_THETA,            /*!< Hall self learn finished get hall install error */
};

/*!
* @brief Three phase current reconstruction algorithm configuration parameters.
*/
CUR_RESTRUCT_CFG            defCurRestruct =
{
    .sampWinLength          = (uint16_t)SAMPLE_WINDOW_US,       /*!< Sample window length*/
    .adSampTim              = (uint16_t)AD_SAMPLE_TIME,         /*!< Ad sample cost time*/
    .sampTrigEdge           = SAMPLE_TRIG_EDGE,                 /*!< Sample trig edge, 0 means Up edge sample, 1 means Down edge sample*/
};

/*!
* @brief Short brake control algorithm configuration parameters.
*/
SHORT_BRAKE_CFG             defShortBrake =
{
    .shortBrakeEnable       = SHORT_BRAKE_CTRL,                 /*!< short brake function enable */
    .shortBrakeCurRatio     = SHORT_BRAKE_CUR_RATIO,            /*!< short brake current */
    .shortBrakeUdcMax       = SHORT_BRAKE_UDC_MAX,              /*!< short brake max Udc voltage */
    .shortBrakeDoneFlagTs   = SHORT_BRAKE_TIMES,                /*!< short brake finished waite time */
    .shortQuitCur           = SHORT_QUIT_CUR,                   /*!< short brake quit current */
    .shortTsCnt             = SHORT_TIME * MS_UINT_INT,         /*!< short brake time for singleR sample */
};

/*!
* @brief Enerfy saving control algorithm configuration parameters.
*/
#if (defined SAVE_ENERGY)
ENERGY_SAVE_CFG             defEnergySave =
{
    .maxCurChange           = MAX_CUR_CHANGE,                   /*!< max Current Change */
    .steadyCurChange        = STEADY_CUR_CHANGE,                /*!< steady Current Change */
    .curChangeStep          = CUR_CHANGE_STEP,                  /*!< current Change Step */
    .energySaveDec          = ENERGY_SAVE_DEC,                  /*!< energy Save Decrease */
    .energySaveInc          = ENERGY_SAVE_INC,                  /*!< energy Save Increase */
    .recheckCurChange       = RECHECK_CUR_CHANGE,               /*!< recheck Current Change */
    .maxIdTarget            = MAX_ID_TARGET,                    /*!< max id target */
    .minIdTarget            = MIN_ID_TARGET,                    /*!< min id target */
    .waiteTim               = WAIT_TIME,                        /*!< waite time */
    .enterEnergySaveLoad    = ENTER_ENERGY_SAVE_LOAD,           /*!< enter Energy Save function Load */
};
#endif

/*!
* @brief Electric brake control algorithm configuration parameters.
*/
ELECBRAKE_CFG               defElecBrake =
{
    .brakeDepth1            = ENTER_BRAKE_DEPTH,                /*!< Enter electric brake depth */
    .brakeDepth2            = EXIT_BRAKE_DEPTH,                 /*!< Exit electric brake depth */
    .elecBrakeFre1          = ENTER_ELEC_BRAKE_FREQ,            /*!< Enter electric brake frequency */
    .elecBrakeFre2          = EXIT_ELEC_BRAKE_FREQ,             /*!< Exit electric brake frequency */
    .isqAddStep             = ISQ_ADD_STEP,                     /*!< Isq add step */
    .isqDecStep             = ISQ_DEC_STEP,                     /*!< Isq decrease step */
    .elecBrakeCurRatio      = ELEC_BRAKE_CUR_RATIO,             /*!< Electric brake current ratio */
    .elecBrakeUdcMax        = ELEC_BRAKE_UDC_MAX,               /*!< Electric brake udcmax */
    .elecBrakeDepth         = ELEC_BRAKE_DEPTH,                 /*!< Electric brake depth */
    .elecBrakeMaxDepth      = ELEC_BRAKE_MAX_DEPTH,             /*!< Electric brake max depth */
};

/*!
* @brief Motor parameters identification algorithm configuration parameters.
*/
PARAM_IDENTIFY_CFG          defParamIdentify =
{
    .identifyParSel         = PARAM_IDENTIFY_SEL,               /*!< Identify parameter select */

    .statorResisOrderSel    = RS_ID_PHASE_SEL,                  /*!< Select motor stator phase order to identify stator resistance, 1: One phase; 2: Two phase; 3: Three phase */
    .pmsmLdqTypeSel         = PMSM_LDQ_METHOD_SEL,              /*!< PMSM Ldq identification method select */

    .rsFirstInjectCurAmp    = RS_ID_FIRST_INJECT,               /*!< Rs identify first inject current threshold*/
    .rsSecondInjectCurAmp   = RS_ID_SECOND_INJECT,              /*!< Rs identify second inject current threshold*/
    .rsStableDelayTime      = RS_STABLE_DELAY,                  /*!< Unit: ms, wait for stable after changing the amplitude of the injection current */
    .rsSetDelayTime         = RS_SETTING_DELAY,                 /*!< Unit: ms, wait for changing the amplitude of the injection current */
    .rsSampleDelayTime      = RS_SAMPLE_DELAY,                  /*!< Unit: ms, wait for sampling and saving current and voltage */
    .ldqInitPrd             = PMSMID_LDQ_INIT_PRD,              /*!< Ldq identify initial pwm period of PMSM */
    .ldqCurLimit            = PMSMID_LDQ_CUR_LIMIT,             /*!< Ldq identify max current limit of PMSM */
    .alignTime1             = PMSMID_ALIGN_TIME1,               /*!< First alignment time of PMSM Ldq identification in ms */
    .alignTime2             = PMSMID_ALIGN_TIME2,               /*!< Second alignment time of PMSM Ldq identification in ms */
    .alignCurVal            = PMSMID_ALIGN_CUR,                 /*!< Alignment current of PMSM Ldq identification */
    .alignWaitTime          = PMSMID_ALIGN_WAIT,                /*!< Alignment done and wait time of PMSM Ldq identification in ms */
    .fluxIdentifySetFreq    = PMSMID_FLUX_FRE,                  /*!< Flux identify set frequency of PMSM */
    .fluxIdentifySetCur     = PMSMID_FLUX_CUR,                  /*!< Flux identify set current of PMSM, must less than 16384 */
    .fluxIdentifyIfAcc      = PMSMID_FLUX_IF_ACC,               /*!< Flux identify IF run  acceleration of PMSM */

    .calcDataNum            = PARID_DATANUM,                    /*!< Motor identification data calculation number */
    .sigFilter1Coeff1       = PARID_FILTER1COEFF1,              /*!< Motor identification signal filter-1 coefficient */
    .sigFilter1Coeff2       = PARID_FILTER1COEFF2,              /*!< Motor identification signal filter-2 coefficient */
    .sigFilter2Coeff1       = PARID_FILTER2COEFF1,              /*!< Motor identification signal filter-1 coefficient */
    .sigFilter2Coeff2       = PARID_FILTER2COEFF2,              /*!< Motor identification signal filter-2 coefficient */

    .currentThreshold       = ACIMID_CURRENT_THRESHOLD,         /*!< Acim identification current threshold */
    .voltageThreshold       = ACIMID_VOLTAGE_THRESHOLD,         /*!< Acim identification voltage threshold */
    .stallFreqLimit         = ACIMID_RR_FREQ,                   /*!< Acim stall test frequency limit */
    .stallCurrentLimit      = ACIMID_RR_CURRENT,                /*!< Acim stall test current limit */
    .stallCurrentStep       = ACIMID_RR_CURSTEP,                /*!< Acim stall test current step */
    .rotationFreqLimit      = ACIMID_LM_FREQ,                   /*!< Acim rotation test frequency limit */
    .rotationFreqStep       = ACIMID_LM_FREQSTEP,               /*!< Acim rotation test frequency step */

    .curDcSet               = PMSMID_CUR_DC_SET,                /*!< PMSM Ldq identification DC current set value */
    .curAcSetD              = PMSMID_CUR_AC_SETD,               /*!< PMSM Ldq identification d-axis AC current set value */
    .curAcSetQ              = PMSMID_CUR_AC_SETQ,               /*!< PMSM Ldq identification q-axis AC current set value */
    .curFreqSetD            = PMSMID_CUR_FREQ_SETD,             /*!< PMSM Ldq identification d-axis AC current frequency set value */
    .curFreqSetQ            = PMSMID_CUR_FREQ_SETQ,             /*!< PMSM Ldq identification q-axis AC current frequency set value */
    .curDcStep              = PMSMID_CUR_DC_STEP,               /*!< PMSM Ldq identification DC current variable step value */
    .stableJudgeLimit       = STABLE_JUDGE_LIMIT,               /*!< Signal stable judgement limit */

    .standByDelay           = DELAY_500MS,                      /*!< Motor parameters identification time delay in standby step */
    .startToCalcDelay       = DELAY_100MS,                      /*!< Motor parameters identification start to calculation delay */
    .calcDelay              = DELAY_500MS,                      /*!< Motor parameters identification calculation step delay time */
    .calcToEndDelay         = DELAY_100MS,                      /*!< Motor parameters identification calculation ending delay */
    .switchDelay            = DELAY_500MS,                      /*!< Motor parameters identification steps swicth delay */

    .parIdPwmFreqSet        = PMSMID_LDQ_PWMFREQ,               /*!< PWM frequency set value */
    .parIdPwmPrdSet         = PMSMID_LDQ_PWMPRD,                /*!< PWM period set value */
    .parIdPwmFreqDef        = FPWM_Hz,                          /*!< PWM frequency default value */
    .parIdPwmPrdDef         = PWM_PERIOD_VALUE,                 /*!< PWM frequency default value */
};

/*!
* @brief SVPWM over modulation algorithm configuration parameters.
*/
OVER_MODULATION_CFG         defOverModulation =
{
    .overModEnable          = OVER_MODULATION,                  /*!< Over modulation enable */
};

/*!
* @brief SVPWM dead time compensation algorithm configuration parameters.
*/
DEADCOMP_CFG                defDeadComp =
{
    .cutoffCurVal           = CUTOFF_CUR_VAL,                   /*!< Decrease compensation cutoff curent value */
    .maxCompFreq            = MAX_COMP_FREQ,                    /*!< Max compensation frequency */
};

/*!
* @brief PID regulator configuration parameters.
*/
PID_CFG                     defPID =
{
    .torqueKp               = PID_TORQUE_KP_DEFAULT,            /*!< torque loop PID kp */
    .torqueKi               = PID_TORQUE_KI_DEFAULT,            /*!< torque loop PID ki */
    .torqueKd               = PID_TORQUE_KD_DEFAULT,            /*!< torque loop PID kd */
    .maxtorque              = PID_TORQUE_MAX_DEFAULT,           /*!< torque loop PID max */
    .mintorque              = PID_TORQUE_MIN_DEFAULT,           /*!< torque loop PID min */

    .fluxKp                 = PID_FLUX_KP_DEFAULT,              /*!< flux loop PID kp */
    .fluxKi                 = PID_FLUX_KI_DEFAULT,              /*!< flux loop PID ki */
    .fluxKd                 = PID_FLUX_KD_DEFAULT,              /*!< flux loop PID kd */
    .maxflux                = PID_FLUX_MAX_DEFAULT,             /*!< flux loop PID max */
    .minflux                = PID_FLUX_MIN_DEFAULT,             /*!< flux loop PID min */

    .speedKp                = PID_SPEED_KP_DEFAULT,             /*!< speed loop PID kp */
    .speedKi                = PID_SPEED_KI_DEFAULT,             /*!< speed loop PID ki */
    .speedKd                = PID_SPEED_KD_DEFAULT,             /*!< speed loop PID kd */
    .maxSpeed               = PID_SPEED_MAX_DEFAULT,            /*!< speed loop PID max */
    .minSpeed               = PID_SPEED_MIN_DEFAULT,            /*!< speed loop PID min */

    .positionKp             = PID_POSITION_KP_DEFAULT,          /*!< position loop PID kp */
    .positionKi             = PID_POSITION_KI_DEFAULT,          /*!< position loop PID ki */
    .positionKd             = PID_POSITION_KD_DEFAULT,          /*!< position loop PID kd */
    .maxPosition            = PID_POSITION_MAX_DEFAULT,         /*!< position loop PID max */
    .minPosition            = PID_POSITION_MIN_DEFAULT,         /*!< position loop PID min */

    .curWicPu               = CURRENT_REGULATOR_BANDWIDTH,      /*!< current loop operating frequency pu value */
    .speedWic               = SPEED_REGULATOR_BANDWIDTH,        /*!< speed loop operating frequency pu value */
    .curPidSelfLearn        = (uint16_t)CURRENT_PID_SELF_LEARN, /*!< current loop PID self learn flag */
    .speedPidSelfLearn      = (uint16_t)SPEED_PID_SELF_LEARN,   /*!< speed loop PID self learn flag */
    .damp                   = SPEED_DAMP_CONST,                 /*!< speed loop damp */
    .feedbackGain           = FEEDBACK_GAIN,                    /*!< feedback gain */
};

/*!
* @brief Flux weakening control algorithm configuration parameters.
*/
FLUXWEAK_CFG                defFluxweak =
{
    .voltageRef             = FLUXWEAKE_VOLTAGE_REF,            /*!< Voltage reference */
    .kp                     = FLUX_WEAKE_KP,                    /*!< Flux weakening pid kp */
    .ki                     = FLUX_WEAKE_KI,                    /*!< Flux weakening pid ki */
    .min                    = IDMIN,                            /*!< Flux weakening pid input min limit */
    .maxModule              = MAXMODULE,                        /*!< Circle limitation maximum allowed module */
    .demagCur               = IDDEMAG,                          /*!< Demagnetization current */
    .nominalCur             = ISMAX,                            /*!< motor nominal current */
};

/*!
* @brief Sliding mode control algorithm configuration parameters.
*/
SMC_CFG                     defSlideMode =
{
    .kslide0                = SMC_KSLIDE0,                      /*!< Open loop configuration sliding mode gain */
    .kslf0                  = SMC_KSLF0,                        /*!< Open loop configuration sliding mode filter coefficient */
    .maxErr0                = SMC_MAXERR0,                      /*!< Open loop configuration sliding mode boundary */
    .kslide1                = SMC_KSLIDE1,                      /*!< Close loop configuration sliding mode gain */
    .kslf1                  = SMC_KSLF1,                        /*!< Close loop configuration sliding mode filter coefficient */
    .maxErr1                = SMC_MAXERR1,                      /*!< Close loop configuration sliding mode boundary */
    .fullOrderGain          = FULL_SMC_GAIN,                    /*!< Full order smc gain */
};

/*!
* @brief Flux observer algorithm configuration parameters.
*/
FLUX_OBSERVER_CFG           defFluxObserver =
{
    .flitCoeff              = FLIT_COEFF,                       /*!< LPF filter coefficient */
    .compCofLimit           = COMPCOF_UPLIMIT,                  /*!< Back emf compensation uplimit */
};

/*!
* @brief Mras observer algorithm configuration parameters.
*/
MRAS_OBSERVER_CFG           defMrasObserver =
{
    .cofAlpha               = ALPHA_COEFF,                      /*!< Mras observer alpha coffcient, use this parameter calculate we*/
    .cofBeta                = BETA_COEFF,                       /*!< Mras observer beta coffcient, use this parameter calculate correct we*/
};

/*!
* @brief Speed estimation by PLL algorithm configuration parameters.
*/
PLL_CFG                     defPll =
{
    #if (defined SMC_OBSERVE)
    .coeffPll               = COEFFPLL_SMC,                     /*!< Q15, PLL PI set coefficient for smc observer */
    #elif (defined FLUX_OBSERVE)
    .coeffPll               = COEFFPLL_FLUX,                    /*!< Q15, PLL PI set coefficient for flux observer */
    #else
    .coeffPll               = 0,
    #endif
};

/*!
* @brief Calculation factor configuration parameter for conversion of real speed to per-unit value.
*/
TRANSFORM_CFG               defTransform =
{
    .speedFactor            = MOTOR_MAX_SPEED_RPM,              /*!< Coefficient: to convert speed from real value to pu value */
};

/*!
* @brief Load torque estimate configuration parameters.
*/
LOAD_ESTIMATE_CFG                defLoadTorque =
{
    .loadEstCoeff           = LOAD_ESTIMATE_COEFF,              /*!< Load torque estimate coefficient */
    .iqCompLimit            = LOAD_COMP_LIMIT,                  /*!< Q axis current compensation limit value */
};

/*!
* @brief Inertia identify configuration parameters.
*/
INERTIA_IDENTIFY_CFG        defInertiaIdentify =
{
    .inertiaGain            = INERTIA_IDENT_GAIN_MRAS,          /*!< Inertia identify gain coefficient of MRAS */
    .inertiaCofInit         = INERTIA_IDENT_COF_INIT,           /*!< Inertia identify coefficient initial value */
    .operationPeriod        = INERTIA_IDENT_OPERATION_PERIOD,   /*!< Inertia identify operation period */
    .errorThrold            = INERTIA_IDENT_ERROR_THRESHOLD,    /*!< Inertia error threshold */
    .steadyTime             = INERTIA_IDENT_STEADY_TIME,        /*!< Inertia error steady judge time */
    .inertiaOldTime         = INERTIA_IDENT_OLD_TIME,           /*!< Last period inertia refresh time */

    .forgetFactor           = INERTIA_IDENT_FACTOR_FFLS,        /*!< Inertia identify forget factor of FFLS */
    .pkInit                 = INERTIA_IDENT_PK_INIT_FFLS,       /*!< Inertia identify pk parameter initial value of FFLS, 10^4~10^6 */
};

/*!
* @brief Speed S-Curve configuration parameters.
*/
SCURVE_CFG                  defSCurve =
{
    .accStartRatio          = ACC_START_SEG_RATIO,              /*!< Proportion coefficient of acceleration starting segment m value, Math_IQ(0.0)~(1-n), max value is Math_IQ(0.99) */
    .accEndRatio            = ACC_END_SEG_RATIO,                /*!< Proportion coefficient of acceleration end segment n value, Math_IQ(0.0)~(1-m), max value is Math_IQ(0.99) */
    .decStartRatio          = DEC_START_SEG_RATIO,              /*!< Proportion coefficient of deceleration starting segment m value, Math_IQ(0.0)~(1-n), max value is Math_IQ(0.99) */
    .decEndRatio            = DEC_END_SEG_RATIO,                /*!< Proportion coefficient of deceleration end segment n value, Math_IQ(0.0)~(1-m), max value is Math_IQ(0.99) */
};

/*!
* @brief Eflash access configuration parameters.
*/
DATA_ACCESS_FLASH_CFG       defEflashAccessCfg =
{
#if (defined DATA_ACCESS_FLASH_ENABLE)
    .accessEnable          = ENABLE,                            /*!< Enable Eflash access */
#else
    .accessEnable          = DISABLE,                           /*!< Disable Eflash access */
#endif
};

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

/* =============================================  EOF  ============================================== */
