/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_MCU_CONFIG_H__
#define __BLDC_MCU_CONFIG_H__
/*!
* @file bldc_mcu_config.h
*
* @brief This file provides MCU peripheral initialization functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_output.h"
#include "pwm_input.h"
#include "adc_drv.h"
#include "bldc_adc.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */
/*!
* NOTICE!
* THE PARAMETERS IN THIS FILE CAN ONLY BE CHANGED ACCORDING TO THE SUPPORT OF TECHNICAL PERSONNEL OR REFERENCE MANUAL!
* DO NOT CHANGE AT WILL, OTHERWISE THE MCU PERIPHERAL MAY NOT WORK PROPERLY!
*/

/* define PWM Modular parameters */
#define SYSCLK_FREQ             (120000000UL)
#define APB_BUS_FREQ            (SYSCLK_FREQ / 2U)

#define MOTOR_BUS_CLK           SYSCLK_FREQ
#define PWM_FREQ                PWM_FREQ_SETTING                                                /*!< in Hz, 16k/8kHz */

#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
#define PWM_PERIOD_VALUE        (((MOTOR_BUS_CLK / (PWM_FREQ * (PWM_PRSC + 1))) >> 1) - 1)      /*!< calculate pwm period value */
#else
#define PWM_PERIOD_VALUE        ((MOTOR_BUS_CLK / (PWM_FREQ * (PWM_PRSC + 1))) - 1)
#endif

/* Set PWM dead time */
/*!<
* PWM_DEAD_TIME count value limitted from 0 to 63(required by register)
* SW_DEAD_TIME_NS = 1000  //1000=1us 1500=1.5us
* PWM_DEAD_TIME = ((SW_DEAD_TIME_NS / 1000) * (MOTOR_BUS_CLK / PWM_DEADTIME_DIVID_4 / 1000000UL))
* PWM_DEADTIME_DIVID_1 = 1
* PWM_DEADTIME_DIVID_4 = 4
* PWM_DEADTIME_DIVID_16 = 16
*/
//#if (defined SYS_CLK_72M)
//#define AD_SAMP_CLK              ADC_SPT_CLK_5
//#define PWM_DEAD_TIME            (9 * SW_DEAD_TIME_NS / 1000)                                   /*!< default dead time set to 1us */
//#elif  (defined SYS_CLK_48M)
#define AD_SAMP_CLK              ADC_SPT_CLK_5//ADC_SPT_CLK_7
#define PWM_DEAD_TIME            (6 * SW_DEAD_TIME_NS / 1000)                                   /*!< default dead time set to 1us */
//#endif

#define TIMER_1MS                (APB_BUS_FREQ / 1000 - 1)                                      /*!< Timer count value of 1 ms */
#define TIMER_2MS                (APB_BUS_FREQ / 500 - 1)                                       /*!< Timer count value of 2 ms */
#define TIMER_XUS                (24)                                                           /*!< User defined time unit us */
#define TIMER_24US               (APB_BUS_FREQ / (1000000 / TIMER_XUS) - 1)                     /*!< Timer count value of user defined time unit us */
#define COMMUTATION_TIMER_PERIOD TIMER_24US                                                     /*!< Period of Timer Use to detect commutate position and moment */

/* define ADC Modular parameters */
/* define ADC Calbiration Conversion times */
#define NUM_OF_CONVERSIONS      32U                                                             /*!< ADC Calbiration Conversion times */

/* signle shunt sample current restruct */
#define DEADTIME                ((SW_DEAD_TIME_NS / 1000.0) * (MOTOR_BUS_CLK /1000000))         /*!< Dead time for 1 shunt sample */
#define RISETIME                ((SAMPLE_RISE_TIME / 1000.0) * (MOTOR_BUS_CLK /1000000))        /*!< Rising time for 1 shunt sample */
#define AD_SAMPLE_TIME          ((SAMPLE_HOLD_TIME / 1000.0) * (MOTOR_BUS_CLK /1000000))        /*!< Sampling time for 1 shunt sample */
#define SAMPLE_WINDOW_US        (DEADTIME + RISETIME + AD_SAMPLE_TIME)                          /*!< Sampling window time for 1 shunt sample, 2.5us */

/* CTU trigger source select */
#if (defined GATE_DRIVER_HIGH_HIGH)
    #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
#elif (defined GATE_DRIVER_HIGH_LOW)
    #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
#elif (defined GATE_DRIVER_LOW_HIGH)
    #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
#elif (defined GATE_DRIVER_LOW_LOW)
    #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
#endif

#define GPIO_INPUT              0U
#define GPIO_OUTPUT             1U

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief ADC Regular sampling Buffer.
*/
extern uint32_t g_ADCRegularBuffer[4];
extern uint32_t g_ADCRegularBuffer1[4];

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Calbiration ADC to get adc channel offset value
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel);

/*!
* @brief Peripheral Initialize in bldc App.
*
* @param[in] none
* @return none
*/
void BLDC_Peripheral_Initialize(void);

/*!
* @brief PWM Initialize, config PWM mode, polarity, and FAULT interrupt.
*
* @param[in] none
* @return none
*/
void BLDC_PWM_Initialize(void);

/*!
* @brief CTU Initialize, config ADC sampling trigger mode and trigger time in bldc control.
*
* @param[in] none
* @return none
*/
void BLDC_CTU_Init(void);

/*!
* @brief ADC Initialize, config ADC mode, sampling channel, trigger source, and DMA transfer.
*
* @param[in] none
* @return none
*/
void BLDC_ADC_Init(void);

/*!
* @brief Get ADC current sampling channel offset value.
*
* @param[in] none
* @return none
*/
void BLDC_ADC_Calbiration(void);

/*!
* @brief Init Delay count factor, 1s = TICKS * (1/f)  s;
*
* @param[in]  none
* @return none
*/
void InitDelay(void);

/*!
* @brief delay us
*
* @param[in] param : us, us for delay
* @return none
*/
void udelay(uint32_t us);

/*!
* @brief delay ms
*
* @param[in] param : ms, us for delay
* @return none
*/
void mdelay(uint32_t ms);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_MCU_CONFIG_H__ */

/* =============================================  EOF  ============================================== */
