/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_data_access_eflash.c
*
* @brief This file provides data save and recovery operation with eflash functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_common.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
int32_t g_VarAccessArray[DATA_NUM] = {0};           /*!< Data access flash transmit array */

uint16_t g_dataAccessErr = 0;                       /*!< Flash operation error, 1 : recovery user date at power on status error; 2: recovery data error */
flash_config_t flashConfig;
const flash_user_config_t FLASH_InitConfig =
{
    .pFlashBase    = PFLASH_BASE_ADDRESS,
    .pFlashSize    = PFLASH_BLOCK_SIZE,
    .dFlashBase    = DFLASH_BASE_ADDRESS,
    .flexRAMBase   = FLEXRAM_BASE_ADDRESS,
    .callback      = NULL,
};

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Array value check by using ChcekSum.
*
* @param[in] array: pointer to data array
* @param[in] length: length of array to do check sum
* @return tempCheckSum: value of check sum
*/
int32_t DataCheckSum(int32_t *array, uint16_t length)
{
    int32_t tempCheckSum = 0;

    while (length--)
    {
        tempCheckSum += *array++;
    }
    return tempCheckSum;
}

/*!
* @brief Get data value to save to flash.
*
* @param[in] array: operation array
* @return none
*/
void DataInputArray(int32_t array[DATA_NUM])
{
    array[0] += 1;

    /* 1~4: motor parameters */
    array[1] = (int32_t)defBldcMotorParam.polePairs;;
    array[2] = (int32_t)defBldcMotorParam.maxCur;
    array[3] = (int32_t)defBldcMotorParam.ratedVoltage;
    array[4] = (int32_t)defBldcMotorParam.maxSpeed;

    array[5] = (int32_t)defBldcVars.motorDir;
    array[6] = (int32_t)defBldcVars.initDuty;
    array[7] = (int32_t)defBldcVars.maxDuty;
    array[8] = (int32_t)defBldcVars.minDuty;
    array[9] = defBldcVars.motorAcceleration;
    array[10] = defBldcVars.motorDeceleration;

    array[11] = 0;     /* reserve */
    array[12] = 0;     /* reserve */
    array[13] = 0;     /* reserve */

    /* 14~23: pid parameters */
    array[14] = (int32_t)defBldcPid.currentKp;
    array[15] = (int32_t)defBldcPid.currentKi;
    array[16] = (int32_t)defBldcPid.currentKd;
    array[17] = (int32_t)defBldcPid.maxCurrent;
    array[18] = (int32_t)defBldcPid.minCurrent;

    array[19] = (int32_t)defBldcPid.speedKp;
    array[20] = (int32_t)defBldcPid.speedKi;
    array[21] = (int32_t)defBldcPid.speedKd;
    array[22] = (int32_t)defBldcPid.maxSpeed;
    array[23] = (int32_t)defBldcPid.minSpeed;

    /* 24~34: sensorless preposition parameters */
    array[24] = (int32_t)defBemfVars.prepositionDutyMin;
    array[25] = (int32_t)defBemfVars.prepositionDutyMax;
    array[26] = (int32_t)defBemfVars.prepositionDutyStep;
    array[27] = (int32_t)defBemfVars.prepositionTimeMin;
    array[28] = (int32_t)defBemfVars.prepositionTimeMax;
    array[29] = (int32_t)defBemfVars.prepositionTimeStep;
    array[30] = (int32_t)defBemfVars.kickDutyMax;
    array[31] = (int32_t)defBemfVars.kickDutyStep;
    array[32] = (int32_t)defBemfVars.kickTimeMin;
    array[33] = (int32_t)defBemfVars.kickTimeMax;
    array[34] = (int32_t)defBemfVars.kickTimeStep;

    array[35] = 0;     /* reserve */
    array[36] = 0;     /* reserve */
    array[37] = 0;     /* reserve */
    array[38] = 0;     /* reserve */
    array[39] = 0;     /* reserve */

    /* last one: sum check */

    array[DATA_NUM - 1] = DataCheckSum(array, (DATA_NUM - 1));
}

/*!
* @brief Get flash data to recovery to variables.
*
* @param[in] array: operation array
* @return none
*/
void DataOutputArray(int32_t array[DATA_NUM])
{
    int32_t tempChechSum = 0;

    /* check sum */
    tempChechSum = DataCheckSum(array, (DATA_NUM - 1));
    if (tempChechSum != array[DATA_NUM - 1])
    {
        g_dataAccessErr = 2;
        return;
    }

    defBldcMotorParam.polePairs = array[1];
    defBldcMotorParam.maxCur = array[2];
    defBldcMotorParam.ratedVoltage = array[3];
    defBldcMotorParam.maxSpeed = array[4];

    defBldcVars.motorDir = array[5];
    defBldcVars.initDuty = array[6];
    defBldcVars.maxDuty = array[7];
    defBldcVars.minDuty = array[8];
    defBldcVars.motorAcceleration = array[9];
    defBldcVars.motorDeceleration = array[10];

    defBldcPid.currentKp = array[14];
    defBldcPid.currentKi = array[15];
    defBldcPid.currentKd = array[16];
    defBldcPid.maxCurrent = array[17];
    defBldcPid.minCurrent = array[18];
    defBldcPid.speedKp = array[19];
    defBldcPid.speedKi = array[20];
    defBldcPid.speedKd = array[21];
    defBldcPid.maxSpeed = array[22];
    defBldcPid.minSpeed = array[23];

    defBemfVars.prepositionDutyMin = array[24];
    defBemfVars.prepositionDutyMax = array[25];
    defBemfVars.prepositionDutyStep = array[26];
    defBemfVars.prepositionTimeMin = array[27];
    defBemfVars.prepositionTimeMax = array[28];
    defBemfVars.prepositionTimeStep = array[29];
    defBemfVars.kickDutyMax = array[30];
    defBemfVars.kickDutyStep = array[31];
    defBemfVars.kickTimeMin = array[32];
    defBemfVars.kickTimeMax = array[33];
    defBemfVars.kickTimeStep = array[34];

}

/*!
* @brief Flash page erase.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] pageAddress: flash operation page address
* @return ret: flash operation status
*/
status_t Eflash_DataAccessPageErase(DATA_ACCESS_FLASH *pDataAccess, uint32_t pageAddress)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->eraseEnable == 1)
    {
        /* AC7840x FLASH */
        ret = FLASH_DRV_UnlockCtrl();
        ret = FLASH_DRV_EraseSector(&flashConfig, pageAddress, PFLASH_PAGE_SIZE);
        if (ret != STATUS_SUCCESS)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        FLASH_DRV_LockCtrl();
        pDataAccess->eraseEnable = 0;
    }
    else
    {
    }
    return ret;
}

/*!
* @brief Flash saving operation.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] pageAddress: flash operation page address
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_SaveProcess(DATA_ACCESS_FLASH *pDataAccess, uint32_t pageAddress, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->saveEnable == 1)
    {
        ret = FLASH_DRV_UnlockCtrl();
        if (ret != STATUS_SUCCESS)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        ret = FLASH_DRV_EraseSector(&flashConfig, pageAddress, PFLASH_PAGE_SIZE);
        if (ret != STATUS_SUCCESS)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        pDataAccess->saveFinishFlag = 0;
        if (pDataAccess->saveMode == SAVE_DEFAULT_DATA)
        {
            DataInputArray(array);
            ret = FLASH_DRV_Program(&flashConfig, pageAddress, DATA_SIZE, (uint8_t *)array);
        }
        else if (pDataAccess->saveMode == SAVE_DEBUG_DATA)
        {
            DataInputArray(array);
            ret = FLASH_DRV_Program(&flashConfig, pageAddress, DATA_SIZE, (uint8_t *)array);
        }
        else
        {
        }

        if (ret != STATUS_SUCCESS)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        FLASH_DRV_LockCtrl();
        pDataAccess->saveEnable = 0;
        pDataAccess->saveFinishFlag = 1;
        ret = STATUS_SUCCESS;
    }
    else
    {
    }
    return ret;
}

/*!
* @brief Flash recovery operation.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] pageAddress: flash operation page address
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_RecoveryProcess(DATA_ACCESS_FLASH *pDataAccess, uint32_t pageAddress, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->recoveryEnable == 1)
    {
        pDataAccess->recoveryFinishFlag = 0;

        if (pDataAccess->recoveryMode == RECOVERY_DEFAULT_DATA)
        {
            ret = FLASH_DRV_Read(&flashConfig, pageAddress , (uint8_t *)array, DATA_SIZE);
            if ((array[DATA_NUM - 1] != 0xFFFFFFFF) && (array[DATA_NUM - 1] != 0))
            {
                pDataAccess->recoveryEnable = 0;
                pDataAccess->recoveryFinishFlag = 1;
                DataOutputArray(array);
                ret = STATUS_SUCCESS;
            }
            else
            {
                return STATUS_BUSY;
            }
        }
        else if (pDataAccess->recoveryMode == RECOVERY_DEBUG_DATA)
        {
            ret = FLASH_DRV_Read(&flashConfig, pageAddress , (uint8_t *)array, DATA_SIZE);
            if ((array[DATA_NUM - 1] != 0xFFFFFFFF) && (array[DATA_NUM - 1] != 0))
            {
                pDataAccess->recoveryEnable = 0;
                pDataAccess->recoveryFinishFlag = 1;
                DataOutputArray(array);
                ret = STATUS_SUCCESS;
            }
            else
            {
                return STATUS_BUSY;
            }
        }
        else
        {
        }

        pDataAccess->recoveryFinishFlag = 0;
    }
    return ret;
}

/*!
* @brief Flash operation process, do flash tasks with tool command.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccessEflashProcess(DATA_ACCESS_FLASH *pDataAccess, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;
    if (pDataAccess->eraseEnable == 1)
    {
        ret = Eflash_DataAccessPageErase(pDataAccess, ADDRESS_START_DEBUG_PARAM);
        pDataAccess->eraseEnable = 0;
    }

    if (pDataAccess->saveEnable == 1)
    {
        if (pDataAccess->saveMode == SAVE_DEFAULT_DATA)
        {
            ret = Eflash_DataAccess_SaveProcess(pDataAccess, ADDRESS_START_DEFAULT_PARAM, array);
            if (pDataAccess->saveFinishFlag == 1)
            {
                pDataAccess->saveEnable = 0;
            }
        }
        else if (pDataAccess->saveMode == SAVE_DEBUG_DATA)
        {
            ret = Eflash_DataAccess_SaveProcess(pDataAccess, ADDRESS_START_DEBUG_PARAM, array);
            if (pDataAccess->saveFinishFlag == 1)
            {
                pDataAccess->saveEnable = 0;
            }
        }
        else
        {
            pDataAccess->saveEnable = 0;
        }
    }

    if (pDataAccess->recoveryEnable == 1)
    {
        if (pDataAccess->recoveryMode == RECOVERY_DEFAULT_DATA)
        {
            ret = Eflash_DataAccess_RecoveryProcess(pDataAccess, ADDRESS_START_DEFAULT_PARAM, array);
            pDataAccess->recoveryEnable = 0;
        }
        else if (pDataAccess->recoveryMode == RECOVERY_DEBUG_DATA)
        {
            ret = Eflash_DataAccess_RecoveryProcess(pDataAccess, ADDRESS_START_DEBUG_PARAM, array);
            pDataAccess->recoveryEnable = 0;
        }
        else
        {
            pDataAccess->recoveryEnable = 0;
        }
    }
    return ret;
}

/*!
* @brief Saving macro define value to flash when mcu is power on.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_DefaultSave(DATA_ACCESS_FLASH *pDataAccess, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    pDataAccess->saveMode = SAVE_DEFAULT_DATA;
    pDataAccess->saveEnable = 1;
    ret = Eflash_DataAccess_SaveProcess(pDataAccess, ADDRESS_START_DEFAULT_PARAM, array);
    if (pDataAccess->saveFinishFlag == 1)
    {
        pDataAccess->saveEnable = 0;
        pDataAccess->saveMode = 0;
        ret = STATUS_SUCCESS;
    }

    return ret;
}

/*!
* @brief Recovery user debug value from flash to variables when mcu is power on.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return none
*/
void Eflash_DataAccess_PowerOnRecovery(DATA_ACCESS_FLASH *pDataAccess, int32_t *array)
{
    int32_t tempWrite;
    int32_t tempCheckSum = 0;

    pDataAccess->recoveryMode = RECOVERY_DEBUG_DATA;

    tempWrite = *(uint8_t*)(ADDRESS_START_DEBUG_PARAM);

    if ((tempWrite != 0) && (tempWrite != 0xFFFFFFFF))
    {
        FLASH_DRV_Read(&flashConfig, ADDRESS_START_DEBUG_PARAM , (uint8_t *)array, DATA_SIZE);

        tempCheckSum = DataCheckSum(array, (DATA_NUM - 1));
        if (tempCheckSum == array[DATA_NUM - 1])
        {
            DataOutputArray(array);
        }
        else
        {
            g_dataAccessErr = 1;
            return;
        }

    }
    else
    {
    }
}

/*!
* @brief Flash operation command initilization.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @return none
*/
void Eflash_DataAccessEflashInit(DATA_ACCESS_FLASH *pDataAccess)
{
    memset(pDataAccess, 0U, sizeof(*pDataAccess));
}

/* =============================================  EOF  ============================================== */
