/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __FOC_MCU_CONFIG_H__
#define __FOC_MCU_CONFIG_H__
/*!
* @file foc_mcu_config.h
*
* @brief This file provides MCU peripheral initialization functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_drv.h"
#include "pwm_output.h"
#include "pwm_input.h"
#include "cur_restruct.h"
#include "drive_parameters_define.h"
#include "adcdrv.h"

/* ============================================  Define  ============================================ */
/*!
* NOTICE!
* THE PARAMETERS IN THIS FILE CAN ONLY BE CHANGED ACCORDING TO THE SUPPORT OF TECHNICAL PERSONNEL OR REFERENCE MANUAL!
* DO NOT CHANGE AT WILL, OTHERWISE THE MCU PERIPHERAL MAY NOT WORK PROPERLY!
*/

/* define PWM Modular parameters */
#define SYSCLK_FREQ             120000000UL
#define APB_BUS_FREQ            (SYSCLK_FREQ / 2U)

#define PWM_MODULER_MOTOR1      (uint8_t)3U
#define PWM_MODULER_DEBUG       (uint8_t)2U
#define PWM_MODULER_HALL        (uint8_t)0U
#define PWM_MODULER_ENCODER     (uint8_t)1U

#define ADC0_MODULER            (uint8_t)0U
#define ADC1_MODULER            (uint8_t)1U

#define GPIO_INPUT              0U
#define GPIO_OUTPUT             1U

#define MOTOR_BUS_CLK           SYSCLK_FREQ
#define PWM_FREQ                PWM_FREQ_SETTING                                                /*!< in Hz, 16k/8kHz */

#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
#define PWM_PERIOD_VALUE        (((MOTOR_BUS_CLK / (PWM_FREQ * (PWM_PRSC + 1))) >> 1) - 1)      /*!< calculate pwm period value */
#else
#define PWM_PERIOD_VALUE        ((MOTOR_BUS_CLK / (PWM_FREQ * (PWM_PRSC + 1))) - 1)
#endif

/* Set PWM dead time */
/*!<
* PWM_DEAD_TIME count value limitted from 0 to 63(required by register)
* SW_DEAD_TIME_NS = 1000  //1000=1us 1500=1.5us
* PWM_DEAD_TIME = ((SW_DEAD_TIME_NS / 1000) * (MOTOR_BUS_CLK / PWM_DEADTIME_DIVID_4 / 1000000UL))
* PWM_DEADTIME_DIVID_1 = 1
* PWM_DEADTIME_DIVID_4 = 4
* PWM_DEADTIME_DIVID_16 = 16
*/
#define AD_SAMP_CLK              ADC_SPT_CLK_15//ADC_SPT_CLK_7
#define PWM_DEAD_TIME            (30 * SW_DEAD_TIME_NS / 1000)                                  /*!< default dead time set to 1us */

#define TIMER_1MS                (APB_BUS_FREQ / 1000 - 1)                                      /*!< Timer count value of 1 ms */
#define TIMER_2MS                (APB_BUS_FREQ / 500 - 1)                                       /*!< Timer count value of 2 ms */
#define TIMER_XUS                (12)                                                           /*!< User defined time unit us */
#define TIMER_12US               (APB_BUS_FREQ / (1000000 / TIMER_XUS) - 1)                     /*!< Timer count value of user defined time unit us */

#define TIMER_T_METHOD_US        (APB_BUS_FREQ / T_METHOD_HZ - 1)                               /*!< Timer count value of M_T speed measure time unit us */
#define M_T_PERIOD               TIMER_T_METHOD_US                                              /*!< Period of Timer Use to measure time in M_T speed measure */

/* define ADC Module parameters */
/* define ADC Calbiration Conversion times */
#define NUM_OF_CONVERSIONS      32U                                                             /*!< ADC Calbiration Conversion times */

/* signle shunt sample current restruct */
#define DEADTIME                ((SW_DEAD_TIME_NS / 1000.0) * (MOTOR_BUS_CLK /1000000))         /*!< Dead time for 1 shunt sample */
#define RISETIME                ((SAMPLE_RISE_TIME / 1000.0) * (MOTOR_BUS_CLK /1000000))        /*!< Rising time for 1 shunt sample */
#define AD_SAMPLE_TIME          ((SAMPLE_HOLD_TIME / 1000.0) * (MOTOR_BUS_CLK /1000000))        /*!< Sampling time for 1 shunt sample */
#define SAMPLE_WINDOW_US        (DEADTIME + RISETIME + AD_SAMPLE_TIME)                          /*!< Sampling window time for 1 shunt sample, 2.5us */

/* CTU trigger source select */
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if ((CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE))
    #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
    #elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
        #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_INIT_TRIG
        #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
        #endif
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if ((CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE))
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
    #elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_INIT_TRIG
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if ((CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE))
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
    #elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_INIT_TRIG
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if ((CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE))
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_MATCH_TRIG
    #elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
        #define CTU_ADC_TRIGGER_SOURCE   TRGMUX_TRIG_SOURCE_PWM3_INIT_TRIG
    #endif
#endif

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief ADC Regular sampling Buffer.
*/
extern uint16_t g_ADCRegularBuffer[2];
extern uint16_t g_ADC1RegularBuffer[6];

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Init Delay count factor, 1s = TICKS * (1/f)  s;
*
* @param[in]  none
* @return none
*/
void InitDelay(void);

/*!
* @brief delay us
*
* @param[in] param : us, us for delay
* @return none
*/
void udelay(uint32_t us);

/*!
* @brief delay ms
*
* @param[in] param : ms, us for delay
* @return none
*/
void mdelay(uint32_t ms);

/*!
* @brief FOC PWM Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWM_Initialize(void);

/*!
* @brief CTU Module Initialize
*
* @param[in] none
* @return none
*/
void FOC_CTU_Initialize(void);

/*!
* @brief Calbiration ADC to get adc channel offset value
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel);

/*!
* @brief ADC Initialize, first calibration current Offset, then config adc as external trigger mode.
*
* @param[in] none
* @return none
*/
void FOC_ADC_Initialize(void);

/*!
* @brief Peripheral Initialize in foc App.
*
* @param[in] none
* @return none
*/
void FOC_Peripheral_Initialize(void);

/*!
* @brief Get phase current sampling offset value.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void FOC_ADC_Calbiration(ADC_SAMPLE *adcSampleReal);

/*!
* @brief Get phase current sampling offset value.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void FOC_ADC1_Calbiration(ADC_SAMPLE *adcSampleReal);

/*!
* @brief FOC Control Hall Peripheral Initialize.
*
* @param[in] none
* @return none
*/
void FOC_Hall_Initialize(void);

///*!
//* @brief Peripheral clock configuration.
//*
//* @param[in] none
//* @return none
//*/
//void FOC_Peripheral_ClockConfig(void);

/* ======================================  Functions define  ======================================== */

#endif /* __FOC_MCU_CONFIG_H__ */

/* =============================================  EOF  ============================================== */
