/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_keys.c
*
* @brief This file provides keys control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "gpio_drv.h"
#include "foc_keys.h"
#include "foc_app.h"
#include "foc_ui_uart.h"
#include "param_identify_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
MC_STATUS g_mcStatus = POWER_ON;            /*!< define motor operation status */
MC_STATUS g_mcBufStatus = TEMPORARY_BUFFER; /*!< define motor operation buffer status */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Get the response of pressing each key on the control board.
*        The keys action are judged by the ADC value:
*        when ADC value is in [ 0, START_STOP_MAX ], it corresponds to the Start / Stop key;
*        when ADC value is in [ FORWARD_REVERSE_MIN, FORWARD_REVERSE_MAX ], it corresponds to the Forward / Reverse key;
*        when ADC value is in [ FAST_MIN, FAST_MAX ], it corresponds to the Fast key;
*        when ADC value is in [ SLOW_MIN, SLOW_MAX ], it corresponds to the Slow key.
*
* @param[in] none
* @return none
*/
void MC_Keys_Read(void)
{
    int32_t keyAdcValue = 0;

    keyAdcValue = g_ADCRegularBuffer[0];

    if ((keyAdcValue > SUSPEND_MIN) && (!s_keyStartPressed))
    {
        s_keyStartPressed = 1;
        s_keyCmdCount = 0;
    }
    else if ((keyAdcValue < START_STOP_MAX) && s_keyStartPressed)
    {
        s_keyCmdCount++;
        if (s_keyCmdCount > CONFIRM_COUNT)
        {
            s_keyStartPressed = 0;
            if (Get_McStatus() == IDLE)
            {
                if (g_paramIdentifyCtrl.identifyCtrl == 1)
                {
                    PushIn_McBufStatus(IDENTIFY);
                }
                else if (defHallSensor.hallSelfLearnEnable == ENABLE)
                {
                    PushIn_McBufStatus(HALL_SELFLEARN);
                }
                else
                {
                    if (SHORT_BRAKE_CTRL & 0x01)
                    {
                        #if (defined TAIL_HEAD_WIND)
                        PushIn_McBufStatus(SPEED_JUDGE);
                        #else
                        PushIn_McBufStatus(START_BRAKE);
                        #endif
                    }
                    else
                    {
                        PushIn_McBufStatus(START);
                    }

                }
            }
            else if ((Get_McStatus() == START_BRAKE) || (Get_McStatus() == STOP_BRAKE))
            {
                PushIn_McBufStatus(STOP);
            }
            else if (Get_McStatus() == RUN)
            {
                if (STOP_MODE == SLOW_DOWN_STOP)
                {
                    PushIn_McBufStatus(SLOW_DOWN);
                }
                else if (SHORT_BRAKE_CTRL > 1)
                {
                    PushIn_McBufStatus(STOP_BRAKE);
                }
                else
                {
                    PushIn_McBufStatus(STOP);
                }
            }
            else if (Get_McStatus() == HALL_SELFLEARN)
            {
                defHallSensor.hallSelfLearnEnable = DISABLE;
                PushIn_McBufStatus(STOP);
            }
            else if (Get_McStatus() == IDENTIFY)
            {
                g_paramIdentifyCtrl.initCtrl = 1;
                PushIn_McBufStatus(STOP);
            }
            else
            {
                PushIn_McBufStatus(STOP);
            }
            s_keyCmdCount = 0;
        }
    }
    else if ((keyAdcValue > SUSPEND_MIN) && (!s_keyDirPressed))
    {
        s_keyDirPressed = 1;
        s_keyCmdCount = 0;
    }
    else if (((keyAdcValue > FORWARD_REVERSE_MIN) && (keyAdcValue < FORWARD_REVERSE_MAX)) && (s_keyDirPressed))
    {
        s_keyCmdCount++;
        if (s_keyCmdCount > CONFIRM_COUNT)
        {
            s_keyDirPressed = 0;

        #if (defined SENSORLESS)
            if (Get_McStatus() == IDLE)
            {
                if (pFocVarsCfg->motorDir == 1)
                {
                    pFocVarsCfg->motorDir = -1;
                }
                else
                {
                    pFocVarsCfg->motorDir = 1;
                }
            }
        #else
            if (pFocVarsCfg->motorDir == 1)
            {
                pFocVarsCfg->motorDir = -1;
            }
            else
            {
                pFocVarsCfg->motorDir = 1;
            }
        #endif
            g_speedCommand.speedTarget = Math_Qabs(g_speedCommand.speedTarget) * pFocVarsCfg->motorDir;

            s_keyCmdCount = 0;
        }
    }
    else if ((keyAdcValue > SUSPEND_MIN) && (!s_keyUpPressed))
    {
        s_keyUpPressed = 1;
        s_keyCmdCount = 0;
    }
    else if (((keyAdcValue > FAST_MIN) && (keyAdcValue < FAST_MAX)) && (s_keyUpPressed))
    {
        s_keyCmdCount++;
        if (s_keyCmdCount > CONFIRM_COUNT)
        {
            s_keyUpPressed = 0;
            if (Math_Qabs(g_speedCommand.speedTarget) > (32767 - SPEED_CHANGE_STEP))
            {
                if (pFocVarsCfg->motorDir == 1)
                {
                    g_speedCommand.speedTarget = 32767 - SPEED_CHANGE_STEP;
                }
                else
                {
                    g_speedCommand.speedTarget = SPEED_CHANGE_STEP - 32767;
                }
            }
            g_speedCommand.speedTarget += (pFocVarsCfg->motorDir * SPEED_CHANGE_STEP);
            s_keyCmdCount = 0;
        }
    }
    else if ((keyAdcValue > SUSPEND_MIN) && (!s_keyDownPressed))
    {
        s_keyDownPressed = 1;
        s_keyCmdCount = 0;
    }
    else if (((keyAdcValue > SLOW_MIN) && (keyAdcValue < SLOW_MAX)) && (s_keyDownPressed))
    {
        s_keyCmdCount++;
        if (s_keyCmdCount > CONFIRM_COUNT)
        {
            s_keyDownPressed = 0;
            g_speedCommand.speedTarget -= (pFocVarsCfg->motorDir * SPEED_CHANGE_STEP);
            s_keyCmdCount = 0;
        }
    }
    else
    {
        /* do nothing */
    }

}

/*!
* @brief Get the main status of motor control right now.
*
* @param[in] none
* @return MC_STATUS: current motor control main status
*/
MC_STATUS Get_McStatus(void)
{
    return g_mcStatus;
}

/*!
* @brief Set the main status of motor control right now.
*
* @param[in] MC_STATUS: status to beset
* @return none
*/
void Set_McStatus(MC_STATUS status)
{
    g_mcStatus = status;
}

/*!
* @brief Push the buffer status in motor control main status buffer.
*
* @param[in] MC_STATUS: statusBuffer to store
* @return none
*/
void PushIn_McBufStatus(MC_STATUS statusBuffer)
{
    if (g_mcBufStatus == TEMPORARY_BUFFER)
    {
        g_mcBufStatus = statusBuffer;
    }
}

/*!
* @brief Pull out buffer status to refresh motor control main status.
*
* @param[in] none
* @return none
*/
void PullOut_McBufStatus(void)
{
    if (g_mcBufStatus != TEMPORARY_BUFFER)
    {
        g_mcStatus = g_mcBufStatus;
        g_mcBufStatus = TEMPORARY_BUFFER;
    }
}

/* =============================================  EOF  ============================================== */
