/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_PARAMETERS_DEFINE_H__
#define __BLDC_PARAMETERS_DEFINE_H__

/*!
* @file bldc_parameters_define.h
*
* @brief This file provides bldc drive parameter define.
*
*/

/* ===========================================  Includes  =========================================== */
#include "motor_parameters_define.h"
#include "hwboard_parameters_define.h"

/* ============================================  Define  ============================================ */
/*!
* @brief This macro define determines whether to execute the parameters access to flash.
*/
#define DATA_ACCESS_FLASH_ENABLE

/*!
* @brief There are two main control modes for three phase brushless motor(include MOTOR_BLDC,MOTOR_SPMSM...in motor_parameters_define.h), which are six step square wave mode and foc sine wave mode.
*        Six step square wave mode are configured in bldc_parameters_define.h; Foc sine wave mode are configured in drive_parameters_define.h.
*        If user choose six step square wave mode, should undefine FOC_SINE_WAVE in drive_parameters_define.h, and define SIX_STEP_SQUARE_WAVE in this file.
*        There are two six step square control sub modes, which are hall sensor control and sensorless control.
*        After define SIX_STEP_SQUARE_WAVE, user can continue choose the sub modes by define BLDC_HALL or BLDC_SENSORLESS, and please undefine the other sub mode meanwhile.
*        IF user choose foc sine wave mode, make sure to undefine SIX_STEP_SQUARE_WAVE and all the two sub modes.
*/
#define SIX_STEP_SQUARE_WAVE
//#define BLDC_HALL
#define BLDC_SENSORLESS

#if (defined BLDC_SENSORLESS)
//    #define BEMF_DETECT_HARDWARE                        /*!< detect bemf with hardware comparator, must rebuild NEUTRAL point. */
    #define BEMF_DETECT_SOFTWARE                        /*!< detect bemf with software way, do not need rebuild NEUTRAL point anymore. */
#endif

/*!
* @brief Current loop configuration. define CURRENT_LOOP_CTRL, bldc motor will run under speed and current loop control. otherwise bldc motor just run under speed loop control.
*/
#define CURRENT_LOOP_CTRL

/*!
* @brief Pwm peripheral configuration.
*/
#define PWM_FREQ_SETTING        12000                   /*!< Set PWM count frequency. current loop consume at least 35us.
*                                                            if defined CURRENT_LOOP_CTRL, default 10kHZ PWM frequency. otherwise PWM frequency can reach 20kHZ. */
#define PWM_PRSC                0                       /*!< PWM prescale */
#define PWM_COUNT_UP_MODE       0                       /*!< PWM count up mode */
#define PWM_COUNT_UP_DOWN_MODE  1                       /*!< PWM count up_down mode */
#define PWM_MODE_SELECT         PWM_COUNT_UP_DOWN_MODE  /*!<0: pwm count up mode; 1: up_down mode */

#define MS_UINT_INT             (PWM_FREQ / 1000)       /*!< lms, used as a timing unit */

/*!
* @brief Debug configuration.
*/
/* Using PWM to display debug information */
//#define DAC_FUNCTIONALITY                                       /*!< DAC debug output enable. If this macro definition is open, DAC debugging is supported */

#define DUTY_PERIOD(n)                  ((n / 1.0f) * PWM_PERIOD_VALUE)             /*!< turn PWM setting duty to the range of effective PWM comparison value(MCVR) */

#define MATH_Abs(x)                     (((x) > 0.0f) ? (x) : (-(x)))
#define MATH_Max(x, y)                  (x > y ? y : x)
#define MATH_Min(x, y)                  (x < y ? y : x)
#define MATH_Sat(A, P, N)               (MATH_Max(MATH_Min(A, N), P))

/*!
* @brief Speed speed loop PID paramter.
*/
#define BLDC_SPEED_KP_DEFAULT           (0.9164f)                             /*!< speed loop PID kp, the setting range is 0.0 ~ 1.0 */
#define BLDC_SPEED_KI_DEFAULT           (0.0092f)                             /*!< speed loop PID ki, the setting range is 0.0 ~ 1.0 */
#define BLDC_SPEED_KD_DEFAULT           (0.0f)                                /*!< speed loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define BLDC_SPEED_MAX_DEFAULT          (0.99999f)                            /*!< speed loop PID max, the setting range is -1.0 ~ 1.0 */
#if (defined CURRENT_LOOP_CTRL)
#define BLDC_SPEED_MIN_DEFAULT          (-0.99999f)                           /*!< speed loop PID min, the setting range is -1.0 ~ 1.0 */
#else
#define BLDC_SPEED_MIN_DEFAULT          (0.0f)                                /*!< speed loop PID min, the setting range is 0.0 ~ 1.0 */
#endif

/*!
* @brief Configuration of speed ramp.
*/
#define OPEN_LOOP_SPEED_HZ                  10.0f                                                   /*!< Final forced synchronize mechanical FREQ in Hz */
#define DEFAULT_TARGET_FREQ_START           (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ)        /*!< Default target frequency for sensorless mode */
#define SPEED_CHANGE_STEP                   (2.0f / BASE_FREQ)                                      /*!< Speed change step, 2.0 means increase 2Hz for each calculation cycle */
#define MAX_SPEED_VALUE                     (0.8f)                                                  /*!< Speed upper limit, 0.8 means max limit of motor running speed is 0.8 * MOTOR_MAX_SPEED_RPM */
#define MIN_SPEED_VALUE                     (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ)        /*!< Speed lower limit */

/*!
* @brief Configuration of acceleration and deceleration speed.
*/
#define ACCELERATION_RPS                    ((600.0 * 10) / (MOTOR_MAX_SPEED_RPM * 100))        /*!< Acceleration of motor speed up in unit RPM/s, motor mechanical rotation speed increase per second.
                                                                                                *    600.0: 600rpm increase per second, must be float type. Can be modified to change the acceleration;
                                                                                                *    MOTOR_MAX_SPEED_RPM: motor speed max value, can not be modified;
                                                                                                *    100: 1s = 100 * (10ms), the speed increase every 10ms, can not be modified.
                                                                                                */
#define DECELERATION_RPS                    ((600.0 * 10) / (MOTOR_MAX_SPEED_RPM * 100))        /*!< Deceleration of motor speed up in unit RPM/s, motor mechanical rotation speed decrease per second.
                                                                                                *    600.0: 600rpm increase per second, must be float type. Can be modified to change the deceleration;
                                                                                                *    MOTOR_MAX_SPEED_RPM: motor speed max value, can not be modified;
                                                                                                *    100: 1s = 100 * (10ms), the speed decrease every 10ms, can not be modified.
                                                                                                */

/*!
* Motor rotation direction configuration.
*/
#define FORWARD_ROTATE                      ((int16_t)1)
#define REVERSE_ROTATE                      (-(int16_t)1)
#define MOTOR_DIR                           FORWARD_ROTATE      /*!< Motor direction 1: foward rotate; -1: reverse rotate */

/*!
* @brief Current speed loop PID paramter.
*/
#define BLDC_CURRENT_KP_DEFAULT         (0.62f)                                     /*!< current loop PID kp, the setting range is 0.0 ~ 1.0 */
#define BLDC_CURRENT_KI_DEFAULT         (0.0031f)                                   /*!< current loop PID ki, the setting range is 0.0 ~ 1.0 */
#define BLDC_CURRENT_KD_DEFAULT         (0.0f)                                      /*!< current loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define BLDC_CURRENT_MAX_DEFAULT        (0.99999f)                                  /*!< current loop PID max, the setting range is 0.0 ~ 1.0 */
#define BLDC_CURRENT_MIN_DEFAULT        (0.0f)                                      /*!< current loop PID max, the setting range is 0.0 ~ 1.0 */

/*!
* @brief Configuration of ADC sample.
*/
#define CURRENT_PU_COF                  (VSVREF / (RSHUNT * OP_AMPLIFICATION_GAIN * ADC_RANGE * MAX_CURRENT))   /*!< ratio between Pu and AD value */
#define CURRENT_AD_COF                  (RSHUNT * OP_AMPLIFICATION_GAIN * ADC_RANGE / VSVREF)                   /*!< ratio between AD value and real current */
#define CURRENT_AD_VALUE(XA)            (XA * CURRENT_AD_COF)                                                   /*!< turn real current(in Ampere format) to AD value */

/*!< Bemf neutral point threshold coefficient in BLDC_SENSORLESS BEMF_DETECT_SOFTWARE way, 2 means half of the max phase voltage */
#define BEMF_NEUTRAL_THRESHOLD_COEFF    ((NORMAL_BUS_VOLTAGE * ADC_RANGE) / (2.0f * PHASE_VOLTAGE_DIVIDE_COEFF * VSVREF))

/*!
* @brief pwdt config paramter.
*        PWDT filter pulse width = PWDT_FILT_VAL * (2^PWDT_FILT_PSC), units: PWDT clks.
*/
#if (defined BLDC_SENSORLESS)
    #define PWDT_CLK_PSC                6                                           /*!< PWDT clk division factor, use for PWDT config in BLDC_SENSORLESS */
    #define PWDT_FILT_PSC               PWM_CHANNEL_INPUT_FILTER_PS_2               /*!< PWDT filter division factor, use for PWDT config in BLDC_SENSORLESS */
    #define PWDT_FILT_VAL               10                                          /*!< PWDT filter pulse width value, use for PWDT config in BLDC_SENSORLESS */
#else
    #define PWDT_CLK_PSC                255                                         /*!< PWDT clk division factor, use for PWDT config in BLDC_HALL */
    #define PWDT_FILT_PSC               PWM_CHANNEL_INPUT_FILTER_PS_2               /*!< PWDT filter division factor, use for PWDT config in BLDC_HALL */
    #define PWDT_FILT_VAL               10                                          /*!< PWDT filter pulse width value, use for PWDT config in BLDC_HALL */
#endif

/*!
* @brief Hall paramter.
*/
#define MIN_DUTY_WITH_HALL              (0.1f)                          /*!< the minimum output duty in BLDC_HALL */
#define MAX_DUTY_WITH_HALL              (0.995f)                        /*!< the maximum output duty in BLDC_HALL */

#define STARTUP_DUTY_STEP               (0.01f)                         /*!< the duty step value in bldc startup */

/*!
* @brief Sensorless paramter.
*/
#define KICK_STARTUP            /*!< open kick drag startup in BLDC_SENSORLESS, need more time get in close loop */
//#define NOKICK_STARTUP          /*!< direct drag startup in BLDC_SENSORLESS, get in close loop faster */

#define MIN_DUTY_SENSORLESS             (0.09f)                                 /*!< the minimum output duty in BLDC_SENSORLESS */
#define MAX_DUTY_SENSORLESS             (0.995f)                                /*!< the maximum output duty in BLDC_SENSORLESS */

#define LEAD_ANGLE                      (float)(7.5f)                           /*!< leading commutation angle in BLDC_SENSORLESS, range in 0 ~ 30 degree */
#define DRAG_DUTY                       (0.05f)                                 /*!< drag duty after get init position while startup */

#define FORCE_COMMUTATE_TIME            (uint16_t)(20000 / TIMER_XUS)           /*!< time threshold in force change phase check, Units: us */
#define FIRST_STEP_DRAG_TIME            (uint16_t)(10000 / TIMER_XUS)           /*!< don't detect over zero point time while drag, Units: us */
#define FIRST_COMMUTATE_TIME            (uint16_t)(2.0 * FIRST_STEP_DRAG_TIME)  /*!< first change phase time since drag */

/* After flow paramter */
//#define BLDC_AFTER_FLOW_CONTROL             /*!< suppress Bemf after flow , only active in BLDC_SENSORLESS */
#define RISE_AFTER_FLOW                     /*!< suppress rise edge Bemf after flow */
#define FALL_AFTER_FLOW                     /*!< suppress fall edge Bemf after flow */
#define AFTER_FLOW_ANGLE_PSRC           2   /*!< suppress Bemf after flow time resolution */
#define AFTER_FLOW_ANGLE_NUMERATOR      1   /*!< range:[0 - 2^AFTER_FLOW_ANGLE_PSRC] */

/* Preposition paramter */
#define MIN_DUTY_PREPOSITION            (0.02f)                 /*!< the minimum output duty in preposition */
#define MAX_DUTY_PREPOSITION            (0.05f)                 /*!< the maximum output duty in preposition */
#define STEP_DUTY_PREPOSITION           (0.01f)                 /*!< the duty step value in preposition */
#define INIT_PREPOSITION_DELAY          100                     /*!< the maximum delay time in preposition, Units: ms */
#define INIT_PREPOSITION_DELAY_STEP     20                      /*!< the delay time step value in preposition, Units: ms */
#define MIN_PREPOSITION_DELAY           40                      /*!< the minimum delay time in preposition, Units: ms */

/* Openkick paramter */
#define BLDC_SENSORLESS_INIT_DELAY_TIME 22                      /*!< the maximum delay time in Open Kick, Units: ms */
#define BLDC_SENSORLESS_MIN_DELAY_TIME  2                       /*!< the minimum delay time in Open Kick, Units: ms */
#define BLDC_SENSORLESS_STEP_DELAY_TIME 2                       /*!< the delay time step value in Open Kick, Units: ms */
#define MAX_DUTY_SENSORLESS_START       (0.15f)                 /*!< the maximum output duty in Open Kick startup */

/* comparator results */
#define UNDRIVE_PHASE_BEMF              ((ACMP0->DR)&ACMP_DR_O_Msk)     /*!< get suspend phase over zero detect results */

/* protect paramter */
#define DETECT_OK                       0                       /*!< The operation status is normal */
#define DETECT_FAULT                    1                       /*!< THe operation status is fault */
#define DETECT_UNKNOWN                  2                       /*!< THe operation status is unknow */

/*!
* @brief Definition of bus current protection parameters.
*/
#define BLDC_OVER_CURRENT_CHECK                                                             /*!< Bus current overcurrent protection enable */
#define BLDC_OVER_CURRENT_THRESHOLD     (20.0f)                                             /*!< Bus current overcurrent protection threshold, unitS: A*/
#define LIMIT_PEAK_CUR_AD               CURRENT_AD_VALUE(BLDC_OVER_CURRENT_THRESHOLD)       /*!< bus peak current limit value */
#define BLDC_OVER_CURRENT_DBC           (10 * MS_UINT_INT)                                  /*!< Bus current overcurrent protection operation period value in ms unit, 10 = 10ms */

/*!
* @brief Definition of lose phase protection parameters.
*/
//#define LOSE_PHASE_PROTECTION_ENABLE                                        /*!< Lose phase protection enable */
#define LOSE_PHASE_THRESHOLD                    (0.025f)                    /*!< Lose phase protection operation current threshold value, refer to current base value */
#define LOSE_PHASE_DBC                          20                          /*!< Lose phase protection operation period value in ms unit, 3 = 3ms */
#define LOSE_PHASE_ADD_CNT                      32                          /*!< Lose phase protection operation add count */
#define LOSE_PHASE_JUDGE_THRESHOLD              (5.0f)                      /*!< Lose phase protection operation Imax/Imin ratio judge threshold value */

/*!
* @brief Definition of motor stall protection parameters.
*/
#define MOTOR_STALL_PROTECTION_ENABLE                                       /*!< Motor stall protection enable */
#define MOTOR_STALL_JUDGE_DBC                   100                         /*!< Motor stall protection operation judge period value in ms unit, 100 = 100ms */
#define MOTOR_STALL_RECOVER_DBC                 2000                        /*!< Motor hall stall protection operation recover period value in ms unit, 2000 = 2s */
#define MOTOR_STALL_DRAG_DUTY                   (0.10f)                     /*!< Motor stall recover drag duty in sensorless control */
#define MOTOR_STALL_SPEED_THRESHOLD             (0.05f)                     /*!< Motor stall protection operation judge speed threshold value */
#define MOTOR_STALL_DUTY_THRESHOLD              (0.20f)                     /*!< Motor stall protection operation judge duty threshold value */
#define MOTOR_STALL_CURRENT_THRESHOLD           (0.4f / MAX_CURRENT)        /*!< Motor stall protection operation recover current limit threshold value,in Ampere unit,
*                                                                                0.4 means 0.4A, set limit value must less than MOTOR_STALL_DUTY_THRESHOLD */

/*!
* @brief Definition of limit bus current protection parameters.
*/
//#define LIMIT_BUS_CURRENT_PROTECTION_ENABLE                                  /*!< Limit bus current protection enable */
#define LIMIT_BUS_CURRENT_LTHRESHOLD             (0.3f / MAX_CURRENT)        /*!< Motor bus current limit protection low point threshold value */
#define LIMIT_BUS_CURRENT_HTHRESHOLD             (0.7f / MAX_CURRENT)        /*!< Motor bus current limit protection high point threshold value,in Ampere unit,
*                                                                                0.7 means 0.7A, set limit value must less than MAX_CURRENT */

/*!
* @brief Definition of bus overvoltage and undervoltage fault protection parameters.
*/
#define BUS_VOLTAGE_OVER                            DETECT_FAULT                /*!< Report status fault */
#define BUS_VOLTAGE_NORMAL                          DETECT_OK                   /*!< Report status ok */
#define BUS_VOLTAGE_UNDER                           DETECT_FAULT                /*!< Report status fault */
#define BUS_VOLTAGE_RECOVER                         DETECT_UNKNOWN              /*!< Report status unknown */

#define BLDC_UNDER_VOLTAGE_CHECK                                 /*!< Bus voltage undervoltage fault protection enable */
#define BLDC_OVER_VOLTAGE_CHECK                                  /*!< Bus voltage overvoltage fault protection enable */

#define UNDER_VBUS_RECOVERY_VOLTAGE           (MIN_BUS_VOLTAGE + 2.0f)      /*!< Bus voltage undervoltage fault recovery voltage, 2 means to recover
                                                                            *    from the fault when the bus voltage is 2V higher than the undervoltage point.
                                                                            */
#define OVER_VBUS_RECOVERY_VOLTAGE            (MAX_BUS_VOLTAGE - 2.0f)      /*!< Bus voltage overvoltage fault recovery voltage, 2 means to recover
                                                                            *    from the fault when the bus voltage is 2V lower than the overvoltage point.
                                                                            */

#define UNDER_VBUS_THRESHOLD             (MIN_BUS_VOLTAGE * 10.0f)                 /*!< 18V * 10, real vbus value enlarges 10 times, action threshold of undervoltage protection */
#define UNDER_VBUS_RECOVERY_THRESHOLD    (UNDER_VBUS_RECOVERY_VOLTAGE * 10.0f)     /*!< 20V * 10, real value enlarges 10 times, action threshold of undervoltage recovery */
#define OVER_VBUS_THRESHOLD              (MAX_BUS_VOLTAGE * 10.0f)                 /*!< 30V * 10, real vbus value enlarges 10 times, action threshold of overvoltage protection */
#define OVER_VBUS_RECOVERY_THRESHOLD     (OVER_VBUS_RECOVERY_VOLTAGE * 10.0f)      /*!< 28V * 10, real value enlarges 10 times, action threshold of overvoltage recovery */

/* ===========================================  Typedef  ============================================ */

#endif /* __BLDC_PARAMETERS_DEFINE_H__ */

/* =============================================  EOF  ============================================== */
