/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __MOTOR_PARAMETERS_DEFINE_H__
#define __MOTOR_PARAMETERS_DEFINE_H__

/*!
* @file motor_parameters_define.h
*
* @brief This file provides motor parameter define.
*
*/

/* ===========================================  Includes  =========================================== */

/* ============================================  Define  ============================================ */
/*!
* Four types of motors are defined,which are asynchronous motor, BLDC, surface PMSM and interior PMSM.
* The below paramters are defined according to motor BLDC_57BL55S06.
*/
#define MOTOR_ACIM              1               /*!< AC induction motor */
#define MOTOR_BLDC              2               /*!< Brushless DC motor */
#define MOTOR_SPMSM             3               /*!< Surface mounted permanent magnet synchronous motor */
#define MOTOR_IPMSM             4               /*!< Interior mounted permanent magnet synchronous motor */

#define MOTOR_TYPE              MOTOR_SPMSM     /*!< Select the motor type: 1, ACIM; 2, BLDC; 3, SPMSM; 4, IPMSM */

/* Define the number of motors controlled by microcontroller at the same time: one or two*/
#define MOTOR_RUN_ONE           0
#define MOTOR_RUN_TWO           1

#define MOTOR_RUN_NUM           MOTOR_RUN_ONE   /*!< Number of motors controlled by microcontroller */

/*!
* This machine parameters are based on 3-phase motor.
* Define the  motor parameter,
* The below paramters are defined according to motor BLDC_57BL55S06.
*/
#define STATOR_RS               575     /*!< Stator resistance (unit:1mohm) */
#define ROTOR_RS                40      /*!< Rotor resistance (unit:1mohm) */
#define LS                      1050    /*!< Stator inductance (unit:1uH) */
#define LD                      1050    /*!< D inductance (unit:1uH) */
#define LQ                      1050    /*!< Q inductance (unit:1uH) */
#define LR                      400     /*!< Rotor inductance (unit:1H) */
#define LM                      380     /*!< Magnetizing inductance (unit:1H) */
#define POLE_PAIR_NUM           2       /*!< Number of pole pairs(constant value) */
#define INERTIA                 240     /*!< Rotational intertia(unit:10^7kg*m2) */
#define VOLTAGE_CONSTANT        2.14    /*!< Phase back emf coefficient rms (unit:1V/Krpm) */
#define MAX_CURRENT             4       /*!< Max current(unit:1A) */
#define MAX_TORQUE              5000    /*!< Max torque(unit:1mNm) */
#define MOTOR_MAX_SPEED_RPM     6000    /*!< Max spped run,at least twice the rate speed(unit:1rpm) */
#define RATE_POWER              1000    /*!< Rate power(unit:1W) */

/* The below paramters are MOTOR_ACIM motor paramters */
#define LEAKAGE_INDUCTANCE              (LR - LM)       /*!< Leakage inductance (1uH) */
#define NO_LOAD_EXCITATION_CURRENT      (0.4)   /*!< Magnetizing current empty load, unit: PU */
#define POWER_FACTOR                    85              /*!< Power factor (constant value in 0~1) */

#define AB_QUAD_ENCODER
//#define ABZ_QUAD_ENCODER

#define ENCODER_POLARITY                0               /*!< Encoder polarity, 0:pulse count increase while turn FORWARD_ROTATE(clockwise), 1:pulse count decrease while turn REVERSE_ROTATE(anti-clockwise) */
#define ENCODER_LINES                   36              /*!< Encoder line number */
#define PULSE_PER_REVOLUTION            (ENCODER_LINES << 2)        /*!< Pulse per cycle */

/* Define the base frequence by max speed */
#define BASE_FREQ        (float)(MOTOR_MAX_SPEED_RPM * POLE_PAIR_NUM / 60)       /*!< Base electrical frequency (Hz) */
#define ONE_BASE_FREQ    (1.00f / BASE_FREQ)                                         /*!< One over Base electrical frequency (Hz) */

/* ===========================================  Typedef  ============================================ */

#endif /* __MOTOR_PARAMETERS_DEFINE_H__ */

/* =============================================  EOF  ============================================== */
