/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_ADC_H__
#define __BLDC_ADC_H__
/*!
* @file bldc_adc.h
*
* @brief This file provides ADC sampling functions interfaces in bldc control.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_parameters_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Clear bus current sample result.
*
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void Clear_CurrentSample(BLDC_ADC_TYPE *adc);

/*!
* @brief Bldc control bus voltage sample, get bus voltage feedback.
*
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void Get_VoltageValue(BLDC_ADC_TYPE *adc);

/*!
* @brief Get undrive phase vlotage AD value.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return undrive phase vlotage
*/
float BLDC_Get_UndrivePhase_Voltage(BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc);

/*!
* @brief Bldc bus current sample and current loop PI control, executed in the RUN state
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void BusCurrent_Control(BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_ADC_H__ */

/* =============================================  EOF  ============================================== */
