/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_ISR_H__
#define __BLDC_ISR_H__
/*!
* @file bldc_isr.h
*
* @brief This file provides BLDC control interruptcallback functions interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include "adc_drv.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Callback routine of TIMER0, 1ms. The Task scheduler update in Timer0 interrupt,use for bldc control time base.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Base_TimerCallback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Callback routine of TIMER1, use for back EMF zero crossing detection in bldc sensorless control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer1_Callback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Callback routine of ADC0, use for ADC sample data and current loop in bldc control.
*
* @param[in] info: device info
* @param[in] parameter: callback parameter
* @return none
*/
void BLDC_ADC_Callback(adc_interrupt_info_t *info, void *parameter);

/*!
* @brief PWMx Fault CallBack Function. When hardware over-current fault occurs, PWM fault interrupt is triggered and this function is called.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void BLDC_PWM_FaultCallback(uint8_t instance, uint32_t status, void *userData);

/*!
* @brief Callback routine of PWDT, occur while hall signal update, use for back EMF zero crossing detection hall signal captrue in bldc sensorless control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Bemf_PWDTCallback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Call back function of PWM_Hall modular used by hall sensor.
*        When PWM_Hall modular input signal level change or Hall timer overflow will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: callback status
* @param[in] userData: callback userData
* @return none
*/
void BLDC_Hall_ChannelCallback(uint8_t instance, uint32_t status, void *userData);

/*!
* @brief Call back function of PWM_Hall modular used by hall sensor.
*        When PWM_Hall modular input signal captrue count overflow will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: callback status
* @param[in] userData: callback userData
* @return none
*/
void BLDC_Hall_OFCallback(uint8_t instance, uint32_t status, void *userData);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_ISR_H__ */

/* =============================================  EOF  ============================================== */
