/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_control_api.c
*
* @brief This file provides user driven bldc motor control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "string.h"
#include "bldc_control_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Set initial startup duty of BLDC control.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void Set_BLDC_InitDuty(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    bldcCtrl->setDuty = bldcCfg->initDuty;
}

/*!
* @brief BLDC speed PID regulator parameter initialize.
*
* @param[in] PID: pointer to PID_TYPE structure
* @param[in] pidConfig: pointer to BLDC_PID_CFG structure
* @return none
*/
void BLDC_Asr_PIInit(PID_TYPE *PID, BLDC_PID_CFG *pidConfig)
{
    /* PID input structure */
    memset(PID->pPidInput, 0U, sizeof(*(PID->pPidInput)));
    PID->pPidInput->maxPu = pidConfig->maxSpeed;
    PID->pPidInput->minPu = pidConfig->minSpeed;
    /* PID coefficient structure */
    PID->pPidCoef->kpPu = pidConfig->speedKp;
    PID->pPidCoef->kiPu = pidConfig->speedKi;
    PID->pPidCoef->kdPu = pidConfig->speedKd;
    /* PID output structure */
    memset(PID->pPidOutput, 0U, sizeof(*(PID->pPidOutput)));
}

/*!
* @brief BLDC current PID regulator parameter initialize.
*
* @param[in] PID: pointer to PID_TYPE structure
* @param[in] pidConfig: pointer to BLDC_PID_CFG structure
* @return none
*/
void BLDC_Acr_PIInit(PID_TYPE *PID, BLDC_PID_CFG *pidConfig)
{
    /* PID input structure */
    memset(PID->pPidInput, 0U, sizeof(*(PID->pPidInput)));
    PID->pPidInput->maxPu = pidConfig->maxCurrent;
    PID->pPidInput->minPu = pidConfig->minCurrent;
    /* PID coefficient structure */
    PID->pPidCoef->kpPu = pidConfig->currentKp;
    PID->pPidCoef->kiPu = pidConfig->currentKi;
    PID->pPidCoef->kdPu = pidConfig->currentKd;
    /* PID output structure */
    memset(PID->pPidOutput, 0U, sizeof(*(PID->pPidOutput)));
}

/*!
* @brief Get startup status of BLDC control.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return motor startup status
*/
int16_t BLDC_Get_Startup_Status(BLDC_VARS_CTRL *bldcCtrl)
{
    return bldcCtrl->startupStatus;
}

/*!
* @brief Division of two numbers (original/divisor), return division result.
*
* @param[in] numerator: numerator in division operation
* @param[in] denominator: denominator in division operation
* @return division operation result
*/
uint16_t FastDiv_Optimize(uint32_t numerator, uint16_t denominator)
{
    uint16_t result = 0;

    while(numerator >= denominator)
    {
        numerator -= denominator;
        ++result;
    }

    return result;
}

/* =============================================  EOF  ============================================== */
