/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_parameters_init.c
*
* @brief This file provides common structure parameters initialization.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_parameters_init.h"
#include "bldc_parameters_define.h"
#include "bldc_mcu_config.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief Motor parameters.
*/
BLDC_MOTOR                 defBldcMotorParam =
{
    .statorRs               = STATOR_RS,                        /*!< Motor stator resistance value */
    .statorLs               = LS,                               /*!< Motor stator inductance value */
    .ld                     = LD,                               /*!< Motor D-axis inductance value */
    .lq                     = LQ,                               /*!< Motor Q-axis inductance value */
    .rotorRs                = ROTOR_RS,                         /*!< Motor rotor resistance value */
    .lm                     = LM,                               /*!< Motor mutual inductance value */
    .lleak                  = 0.0f,                             /*!< Motor leakage inductance value */

    .jm                     = (uint32_t)(INERTIA + 0.5f),       /*!< Motor inertia value */
    .polePairs              = POLE_PAIR_NUM,                    /*!< Motor pole pairs value */
    .encoderLines           = PULSE_PER_REVOLUTION,             /*!< encoder pulse lines per motor mechanical cycle */
    .bEmf                   = VOLTAGE_CONSTANT,                 /*!< Motor back EMF constant  */
    .maxCur                 = MAX_CURRENT,                      /*!< Max stator current value */
    .ratedVoltage           = NORMAL_BUS_VOLTAGE,               /*!< DC bus voltage of inverter system */
    .maxSpeed               = MOTOR_MAX_SPEED_RPM,              /*!< Motor running speed max value */
    .maxFreq                = BASE_FREQ,                        /*!< Motor current frequency max value */

    .maxTorque              = MAX_TORQUE,                       /*!< Motor electromagnetic torque max value */
    .maxPower               = RATE_POWER,                       /*!< Motor rated power */
};

/*!
* @brief BLDC PID regulator configuration parameters.
*/
BLDC_PID_CFG                defBldcPid =
{
    .currentKp              = BLDC_CURRENT_KP_DEFAULT,           /*!< current loop PID kp */
    .currentKi              = BLDC_CURRENT_KI_DEFAULT,           /*!< current loop PID ki */
    .currentKd              = BLDC_CURRENT_KD_DEFAULT,           /*!< current loop PID kd */
    .maxCurrent             = BLDC_CURRENT_MAX_DEFAULT,          /*!< current loop PID max */
    .minCurrent             = BLDC_CURRENT_MIN_DEFAULT,          /*!< current loop PID min */

    .speedKp                = BLDC_SPEED_KP_DEFAULT,             /*!< speed loop PID kp */
    .speedKi                = BLDC_SPEED_KI_DEFAULT,             /*!< speed loop PID ki */
    .speedKd                = BLDC_SPEED_KD_DEFAULT,             /*!< speed loop PID kd */
    .maxSpeed               = BLDC_SPEED_MAX_DEFAULT,            /*!< speed loop PID max */
    .minSpeed               = BLDC_SPEED_MIN_DEFAULT,            /*!< speed loop PID min */

};

/*!
* @brief BLDC control configuration parameters.
*/
BLDC_VARS_CFG                defBldcVars =
{
    .motorDir               = MOTOR_DIR,                        /*!< Motor rotation direction */
    #if (defined BLDC_HALL)
    .initDuty               = MIN_DUTY_WITH_HALL,               /*!< initial hall startup duty */
    #elif (defined BLDC_SENSORLESS)
    .initDuty               = MIN_DUTY_PREPOSITION,             /*!< initial sensorless startup duty */
    #else
    #endif

    .maxDuty                = 0.0f,                             /*!< max duty */
    .minDuty                = 0.0f,                             /*!< min duty */

    .speedMaxLimit          = MAX_SPEED_VALUE,                  /*!< Max limit of speed command */
    .speedMinLimit          = MIN_SPEED_VALUE,                  /*!< Min limit of speed command */
    .motorAcceleration      = ACCELERATION_RPS,                 /*!< Acceleration of motor speed up */
    .motorDeceleration      = DECELERATION_RPS,                 /*!< Deceleration of motor speed reduction */

    .baseFreq               = BASE_FREQ,                        /*!< Motor frequency reference base value */

    .msUnitInt              = MS_UINT_INT,                      /*!< Millisecond unit */
    .halfPwmPeriod          = (PWM_PERIOD_VALUE >> 1),          /*!< PWM half period */
    .pwmPeriod              = PWM_PERIOD_VALUE,                 /*!< PWM period */
    .pwmDeadTime            = PWM_DEAD_TIME,                    /*!< PWM dead time */

    .bemfNeutralCoef        = BEMF_NEUTRAL_THRESHOLD_COEFF,     /*!< Bemf neutral point threshold coefficient */

    .sysBusClk              = MOTOR_BUS_CLK,                    /*!< MCU system bus clock */
    .pwdtPrescal            = PWDT_CLK_PSC,                     /*!< PWDT clock prescaler */
};

/*!
* @brief BLDC sensorless startup Bemf configuration parameters.
*/
BLDC_BEMF_CFG               defBemfVars =
{
    .prepositionDutyMin     = 0.0f,                              /*!< Min duty while preposition in BLDC sensorless startup */
    .prepositionDutyMax     = 0.0f,                              /*!< Max duty while preposition in BLDC sensorless startup */
    .prepositionDutyStep    = 0.0f,                              /*!< Step duty once while preposition in BLDC sensorless startup */
    .prepositionTimeMin     = 0.0f,                              /*!< Min time while preposition in BLDC sensorless startup(unit:ms) */
    .prepositionTimeMax     = 0.0f,                              /*!< Max time while preposition in BLDC sensorless startup(unit:ms) */
    .prepositionTimeStep    = 0.0f,                              /*!< Step time once while preposition in BLDC sensorless startup(unit:ms) */
    .kickDutyMax            = 0.0f,                              /*!< Max duty while open kick in BLDC sensorless startup */
    .kickDutyStep           = 0.0f,                              /*!< Step duty once while open kick in BLDC sensorless startup */
    .kickTimeMin            = 0.0f,                              /*!< Min time while open kick in BLDC sensorless startup(unit:ms) */
    .kickTimeMax            = 0.0f,                              /*!< Max time while open kick in BLDC sensorless startup(unit:ms) */
    .kickTimeStep           = 0.0f,                              /*!< Step time once while open kick in BLDC sensorless startup(unit:ms) */
};

/*!
* @brief System hardware parameters.
*/
BLDC_HW                    defBldcHwBoard =
{
    .pwmPolarity            = PHASE_UVW_POLARITY,               /*!< Three phase polarity of inverter */
    .hwDeadTime             = HW_DEAD_TIME_NS,                  /*!< Hardware dead time of power device in inverter */
    .swDeadTime             = SW_DEAD_TIME_NS,                  /*!< Configurable software dead time of power device in inverter */
    .maxVoltage             = MAX_BUS_VOLTAGE,                  /*!< Max DC Bus voltage */
    .normalVoltage          = NORMAL_BUS_VOLTAGE,               /*!< Normal DC Bus voltage */
    .minVoltage             = MIN_BUS_VOLTAGE,                  /*!< Min DC Bus voltage */
    .vbusAttenuateVal       = VBUS_ATTENUATE_FACTOR,            /*!< Voltage sampling circuit attenuation factor */

    .maxCurrent             = HW_MAX_CURRENT,                   /*!< Inverter hardware max current */
    .rShunt                 = RSHUNT,                           /*!< Sampling resistance value */
    .opGainVal              = OP_AMPLIFICATION_GAIN,            /*!< Operation amplifier gain in current sampling circuit */
    .icsGainVal             = ICS_GAIN,                         /*!< Current sensor gain */
    .adcRange               = ADC_RANGE,                        /*!< ADC range */
    .adcRefV                = VSVREF,                           /*!< ADC reference voltage */
    .overCurrentPolarity    = (uint16_t)OVERCUR_FEEDBACK_POLARITY,  /*!< Hardware over current fault polarity */
};

/*!
* @brief Eflash access configuration parameters.
*/
DATA_ACCESS_FLASH_CFG       defEflashAccessCfg =
{
#if (defined DATA_ACCESS_FLASH_ENABLE)
    .accessEnable          = ENABLE,                            /*!< Enable Eflash access */
#else
    .accessEnable          = DISABLE,                           /*!< Disable Eflash access */
#endif
};

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

/* =============================================  EOF  ============================================== */
