/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PARAMETERS_CONFIG_H__
#define __PARAMETERS_CONFIG_H__
/*!
* @file parameters_config.h
*
* @brief This file provides the global structure declaration.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_input.h"
#include "foc_mcu_config.h"
#include "startup.h"
#include "pulse_inject.h"
#include "highfreqinj.h"
#include "power_dccur_estimate.h"
#include "cur_restruct.h"
#include "short_brake.h"
#include "energy_save.h"
#include "param_identify.h"
#include "flux_weakening.h"
#include "smopos.h"
#include "flux_observer.h"
#include "motorctrl.h"
#include "encoder.h"
#include "speed_angle.h"
#include "mras.h"
#include "foc_data_access_eflash.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
extern MOTOR_PARAM              defMotorParam;          /*!< Motor parameters */
extern FOC_VARS_CFG             defFocVars;             /*!< FOC vector control configuration parameters */
extern HW_BOARD                 defHwBoard;             /*!< System hardware parameters */
extern START_UP_CFG             defStartUp;             /*!< Startup control configuration parameters */

#if (defined PULSE_INJECTION)
extern PULSE_INJ_CFG            defPulseInj;            /*!< Pulse injection startup configuration parameters */
#endif
#if (defined INIT_POS_DETEC)
extern INITPOS_DETEC_CFG        defInitPosDetec;        /*!< Rotation high frequency voltage injection initial position check configuration parameters */
#endif

#if (defined SAVE_ENERGY)
extern ENERGY_SAVE_CFG          defEnergySave;          /*!< Enerfy saving control algorithm configuration parameters */
#endif

//#if (defined HIGH_FREQUENCE_INJECTION)
extern HFI_CFG                  defHighFreqInj;         /*!< High frequency injection algorithm configuration parameters */
//#endif

extern POWEREST_CFG             defPowerEst;            /*!< Power estimation algorithm configuration parameters */
extern HALL_SENSOR_CFG          defHallSensor;          /*!< Hall sensor vector control algorithm configuration parameters */
extern CUR_RESTRUCT_CFG         defCurRestruct;         /*!< Three phase current reconstruction algorithm configuration parameters */
extern SHORT_BRAKE_CFG          defShortBrake;          /*!< Short brake control algorithm configuration parameters */
extern ELECBRAKE_CFG            defElecBrake;           /*!< Electric brake control algorithm configuration parameters */
extern PARAM_IDENTIFY_CFG       defParamIdentify;       /*!< Motor parameters identification algorithm configuration parameters */
extern OVER_MODULATION_CFG      defOverModulation;      /*!< SVPWM over modulation algorithm configuration parameters */
extern DEADCOMP_CFG             defDeadComp;            /*!< SVPWM dead time compensation algorithm configuration parameters */
extern PID_CFG                  defPID;                 /*!< PID regulator configuration parameters */
extern FLUXWEAK_CFG             defFluxweak;            /*!< Flux weakening control algorithm configuration parameters */
extern SMC_CFG                  defSlideMode;           /*!< Sliding mode control algorithm configuration parameters */
extern FLUX_OBSERVER_CFG        defFluxObserver;        /*!< Flux observer algorithm configuration parameters */
extern MRAS_OBSERVER_CFG        defMrasObserver;        /*!< Mras observer algorithm configuration parameters */
extern TRANSFORM_CFG            defTransform;           /*!< Calculation factor configuration parameter for conversion of real speed to per-unit value */
extern PLL_CFG                  defPll;                 /*!< Speed estimation by PLL algorithm configuration parameters */
extern LOAD_ESTIMATE_CFG        defLoadTorque;          /*!< Load torque estimate configuration parameters */
extern INERTIA_IDENTIFY_CFG     defInertiaIdentify;     /*!< Inertia identify configuration parameters */

extern SCURVE_CFG               defSCurve;              /*!< Speed S-Curve configuration parameters */

extern DATA_ACCESS_FLASH_CFG    defEflashAccessCfg;     /*!< Eflash access configuration parameters */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

#endif /* __PARAMETERS_CONFIG_H__ */

/* =============================================  EOF  ============================================== */
