/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PWMDRV_H__
#define __PWMDRV_H__
/*!
* @file pwmdrv.h
*
* @brief This file provides PWM module functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "cur_restruct.h"

/* ============================================  Define  ============================================ */
#define PWM_OUTPUT_DISABLE      0       /*!< Mask the PWM channel output */
#define PWM_OUTPUT_ENABLE       1       /*!< Make the PWM channel output */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Load the duty cycle of three-phase inverter calculated by SVPWM algorithm into PWM module.
*
* @param[in] Ta: duty of phase A upper bridge arm
* @param[in] Tb: duty of phase B upper bridge arm
* @param[in] Tc: duty of phase C upper bridge arm
* @return none
*/
void PWM_Update(uint16_t Ta, uint16_t Tb, uint16_t Tc);

/*!
* @brief Load the duty cycle of three-phase inverter calculated by SVPWM
*        algorithm into PWM module when single shunt sampling is used.
*
* @param[in] curRestruct: pointer to CUR_RESTRUCT_DATA structure
* @return none
*/
void PWM_SingleShuntUpdate(CUR_RESTRUCT_DATA *curRestruct);

/*!
* @brief Disable PWM module output pulse.
*
* @param[in] none
* @return none
*/
void PWM_Disable(void);

/*!
* @brief Enable PWM module output pulse.
*
* @param[in] none
* @return none
*/
void PWM_Enable(void);

/*!
* @brief Enable or disable PWM channel output according to the input mark.
*
* @param[in] enable: Enable mark, 'enable' = DISABLE = 0, will mask the PWM channel output; 'enable' = ENABLE = 1, will make the PWM channel output.
* @return none
*/
void PWM_Output(uint8_t enable);

/*!
* @brief Turn on three phase lower MOSFET or IGBT to make motor winding in short circuit control.
*
* @param[in] none
* @return none
*/
void PWM_ShortCircuitOn(void);

/*!
* @brief Turn off three phase lower MOSFET or IGBT.
*
* @param[in] none
* @return none
*/
void PWM_ShortCircuitOff(void);

/* ======================================  Functions define  ======================================== */

#endif /* __PWMDRV_H__ */

/* =============================================  EOF  ============================================== */
