/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_app.c
*
* @brief This file provides FOC control parameters initialization, state machine and millisecond time base functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "gpio_drv.h"
#include "foc_app.h"
#include "foc_isr.h"
#include "foc_keys.h"
#include "globalvar.h"
#include "pulse_inject_api.h"
#include "motorctrl_api.h"
#include "foc_ui_uart.h"
#include "vf_control.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
status_t g_flashOperationStatus = STATUS_UNSUPPORTED;       /*!< Return value of eflash operation status */
uint16_t g_flashTaskFinish = 0;                             /*!< Finish tag of flash task, used to uart communication */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Motor vector control global variable parameters initialization.
*
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void STC_ParamInit(STC_PARAM *STC)
{
    pMotorParam = &defMotorParam;
    pFocVarsCfg = &defFocVars;
    pHWBoardParam = &defHwBoard;
    pTransformCfg = &defTransform;

    pStartUpCfg = &defStartUp;

#if (defined PULSE_INJECTION)
    pPulseInjCfg = &defPulseInj;
#endif
    pShortBrakeCfg = &defShortBrake;
#if (defined SAVE_ENERGY)
    pEnergySaveCfg = &defEnergySave;
#endif
    pCurRestructCfg = &defCurRestruct;
    pOverModulationCfg = &defOverModulation;
    pDeadCompCfg = &defDeadComp;
    pPidCfg = &defPID;

    pHighFreqInjCfg = &defHighFreqInj;
    g_hfiCtrl.bpf = &g_bpfCoeff;
    g_hfiCtrl.lpf = &g_lpfCoeff;
    g_hfiCtrl.initPos = &g_initPosDet;
    g_hfiCtrl.hfi = &g_hfinj;

    pPowerEstCfg = &defPowerEst;
    pFluxObserveCfg = &defFluxObserver;
    pMrasObserveCfg = &defMrasObserver;
    pPllCfg = &defPll;
    pSmoObserveCfg = &defSlideMode;
    pElecBrakeCfg = &defElecBrake;
    pFluxWeakCfg = &defFluxweak;
    pSCurveCfg = &defSCurve;
    pEflashCfg = &defEflashAccessCfg;

    pLoadTorqueCfg = &defLoadTorque;

    pInertiaIdentCfg = &defInertiaIdentify;

    STC_motorParamPuCalc(pMotorParam, pFocVarsCfg, &g_motorParamPu);
    ADC_SampleCofCalc(&g_motorParamPu, pHWBoardParam, &g_adcSampleCof);

    g_focVars.pCfg = pFocVarsCfg;
    g_focVars.pCtrl = &g_focVarsCtrl;

    STC->pFOCVars = &g_focVars;
    STC->pMotor = pMotorParam;
    STC->pMotorPu = &g_motorParamPu;

    g_startUp.pStCfg = pStartUpCfg;
    g_startUp.pStCtrl = &g_startUpCtrl;

    /* PID structure */
    g_pidAcrD.pPidInput = &g_acrCurIdPIIn;
    g_pidAcrD.pPidCoef = &g_acrCurIdCof;
    g_pidAcrD.pPidOutput = &g_acrCurIdPIOut;

    g_pidAcrQ.pPidInput = &g_acrCurIqPIIn;
    g_pidAcrQ.pPidCoef = &g_acrCurIqCof;
    g_pidAcrQ.pPidOutput = &g_acrCurIqPIOut;

    g_pidAsr.pPidInput = &g_asrSpeedPIIn;
    g_pidAsr.pPidCoef = &g_asrSpeedCof;
    g_pidAsr.pPidOutput = &g_asrSpeedPIOut;

#if (defined POSITION_LOOP_CTRL)
    g_pidApr.pPidInput = &g_aprPositionPIIn;
    g_pidApr.pPidCoef = &g_aprPositionCof;
    g_pidApr.pPidOutput = &g_aprPositionPIOut;
#endif

    g_pidPll.pPidInput = &g_asrSpeedEstPIIn;
    g_pidPll.pPidCoef = &g_asrSpeedEstCof;
    g_pidPll.pPidOutput = &g_asrSpeedEstPIOut;

    g_pidFw.pPidInput = &g_fluxWeakPIIn;
    g_pidFw.pPidCoef = &g_fluxWeakPICof;
    g_pidFw.pPidOutput = &g_fluxWeakPIOut;

    g_pidHfi.pPidInput = &g_asrHfiEstPIIn;
    g_pidHfi.pPidCoef = &g_asrHfiEstCof;
    g_pidHfi.pPidOutput = &g_asrHfiEstPIOut;

    g_fluxObserver.pCfg = pFluxObserveCfg;
    g_fluxObserver.pCtrl = &g_fluxObserverCtrl;

    /* ACR for Rs identification */
    g_pidAcrAlfaRsId.pPidCoef = &g_acrAlfaRsIdPICof;
    g_pidAcrAlfaRsId.pPidInput = &g_acrAlfaRsIdPIIn;
    g_pidAcrAlfaRsId.pPidOutput = &g_acrAlfaRsIdPIOut;
    g_pidAcrBetaRsId.pPidCoef = &g_acrBetaRsIdPICof;
    g_pidAcrBetaRsId.pPidInput = &g_acrBetaRsIdPIIn;
    g_pidAcrBetaRsId.pPidOutput = &g_acrBetaRsIdPIOut;

    g_paramIdentify.pParamIdentifyCfg = &defParamIdentify;
    g_paramIdentify.pParamIdentifyCtrl = &g_paramIdentifyCtrl;
    g_paramIdentify.pParamRsIdentify = &g_paramRsIdentify;
    g_paramIdentify.pParamLdqIdentify = &g_paramLdqIdentify;
    g_paramIdentify.pParamFluxIdentify = &g_paramFluxIdentify;
    g_paramIdentify.pParamInertiaIdentify = &g_paramInertiaIdentify;
    g_paramIdentify.pmsmLdqIdentify = &g_pmsmLdqId;
    g_paramIdentify.pmsmLdqIdentify->ldqIdParam = &g_pmsmLdqSwParam;
    g_paramIdentify.pmsmLdqIdentify->fftParam = &g_fftParam;

    g_hall.pHallCfg = &defHallSensor;
    g_hall.pHallSensor = &g_hallSensor;
    g_hall.pHallSelfLearn = &g_hallSelfLearn;

    g_speedCommand.speedCalc = &g_rotationalSpeed;

    g_startUpCurveIn.curveSeg1 = &g_startUpCurveSeg1;
    g_startUpCurveIn.curveSeg2 = &g_startUpCurveSeg2;
    g_startUpCurveIn.curveSeg3 = &g_startUpCurveSeg3;
    g_startUpCurveIn.curveSeg4 = &g_startUpCurveSeg4;
    g_startUpCurveIn.curveSeg5 = &g_startUpCurveSeg5;

#if (defined INIT_POS_DETEC)
    pInitPosDetecCfg = &defInitPosDetec;
    g_initPosDetecStruct.fftParam = &g_initPosDetecFft;
    g_initPosDetecStruct.initPosDetec = &g_initPosDetecParam;
#endif

#if (defined FOC_ACIM)
    g_acimFocCtrl.cur = &g_acimCur;
    g_acimFocCtrl.flux = &g_acimFlux;
    g_acimFocCtrl.mot = &g_aciMotor;
    g_acimFocCtrl.speed = &g_acimSpeed;
    g_acimFocCtrl.volt = &g_acimVolt;
    g_paramIdentify.pAcimParamIdentify = &g_acimParamId;
#endif
}

/*!
* @brief Foc control variables initialization, clear all the variables of FOC_VARS_CTRL to zero.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @return none
*/
void FOC_CtrlVarsInit(FOC_VARS_CTRL *focVarsCtrl)
{
    memset(focVarsCtrl, 0, sizeof(*focVarsCtrl));
}

/*!
* @brief ACR parameters initialization of stator resistance identification.
*
* @param[in] pAcrAlfaRsId: pointer to PID_REGULATOR structure
* @param[in] pAcrBetaRsId: pointer to PID_REGULATOR structure
* @return none
*/
void StatorResistanceACRInit(PID_REGULATOR *pAcrAlfaRsId, PID_REGULATOR *pAcrBetaRsId)
{
    memset(pAcrAlfaRsId->pPidOutput, 0, sizeof(*(pAcrAlfaRsId->pPidOutput)));
    memset(pAcrBetaRsId->pPidOutput, 0, sizeof(*(pAcrBetaRsId->pPidOutput)));
    pAcrAlfaRsId->pPidCoef->kpPu = (0.5f);
    pAcrAlfaRsId->pPidCoef->kiPu = (0.005f);
    pAcrAlfaRsId->pPidInput->maxPu = 0.9999f;
    pAcrAlfaRsId->pPidInput->minPu = -1.0f;

    pAcrBetaRsId->pPidCoef->kpPu = (0.5f);
    pAcrBetaRsId->pPidCoef->kiPu = (0.005f);
    pAcrBetaRsId->pPidInput->maxPu = 0.9999f;
    pAcrBetaRsId->pPidInput->minPu = -1.0f;
}

/*!
* @brief ACR parameters initialization of stator resistance identification.
*
* @param[in] pwmModuler: PWM moduler to get Hall sector
* @return hallInitStatus
*/
uint8_t GetHallInitStatus(const uint8_t pwmModuler)
{
    uint16_t tempStatus;
    uint8_t hallInitStatus;
    uint8_t tempA, tempB, tempC;

    tempStatus = PWM_DRV_GetAllChannelInputStatus(pwmModuler) >> 16;
    tempA = tempStatus & 0x1U;
    tempB = (tempStatus & 0x2U) >> 1;
    tempC = (tempStatus & 0x4U) >> 2;
    hallInitStatus = (tempA << 2) + (tempB << 1) + tempC;

    return (hallInitStatus);
}

/*!
* @brief Foc control parameters initialization, executed in the Stop state and the first Power on.
*
* @param[in] none
* @return none
*/
void FOC_Parameters_Initialize(void)
{
    FOC_CtrlVarsInit(&g_focVarsCtrl);
    Clear_CurrentSample(&g_adcTransform);
    MC_SpeedTrqInit();
    ASR_SpeedCommandInit(pFocVarsCfg, &g_speedCommand);
#if (defined POSITION_LOOP_CTRL)
    MC_PositionInit();
    APR_PositionCommandInit(pFocVarsCfg, &g_positionCommand);
#endif
#if (defined MT_SPEED_MEASURE)
    Encoder_MT_Init(&g_encoderSpeed);
#endif
    Asr_SpeedPLLInit(&g_pidPll, pFocVarsCfg, pPllCfg);

#if (defined SCURVE_SPEED_RAMP)
    SCurve_InitProcess(&g_sCurveData);
#endif

    /* Sensorless FOC */
#if ((defined SENSORLESS) ||  (defined ENCODER_SENSORS))
    StartUp_Ctrl_Init(&g_startUp);
    /* Slide Mode observer */
    #if (defined SMC_OBSERVE)
    Smc_ParamInit(&g_motorParamPu, pFocVarsCfg, pSmoObserveCfg, &g_smcEstimator);
    /* Flux observer */
    #elif (defined FLUX_OBSERVE)
    FluxObserver_Init(&g_fluxObserver, &g_motorParamPu, pFocVarsCfg);
    #elif (defined MRAS_OBSERVE)
    Mras_Init(&g_mrasObserver, &g_motorParamPu, pMrasObserveCfg);
    #else
    #endif

    /* High frequency injection */
    #if (defined HIGH_FREQUENCE_INJECTION)
    Hfi_Init(&g_hfiCtrl, &g_pidHfi, pHighFreqInjCfg, pFocVarsCfg);
    #endif

    /* Rotor initial position identification by Pulse injection */
    #if (defined PULSE_INJECTION)
    PulseInject_Init(&g_pulseInject);
    #endif
#endif

#if (defined FOC_ACIM)
    Acim_FluxObserverInit(&g_acimFocCtrl, &g_STC);
    StartUp_Ctrl_Init(&g_startUp);
#endif

    /* Motor protection */
    Init_Protection();

    /* Is over load fall frequency */
#if (defined OVER_LOAD_PROTECT_ENABLE)
    Is_OverLoad_Init(&g_overLoadPro);
#endif

    /* Power decrease limit speed target command */
#if (defined POWER_DECREASE_PROTECT_ENABLE)
    PowerDecrease_init(&g_powerDecrease);
#endif

    /* Speed calculation parameters initialization */
    ASR_RotationSpeedInit(&g_speedCommand);

    /* Short circuit brake */
#if (SHORT_BRAKE_CTRL > 0)
    ShortBrake_Init(&g_shortBrake);
#endif
    /* Electric brake */
#if (defined ELEC_BRAKE_ENABLE)
    ElecBrake_Init(&g_elecBrake, pElecBrakeCfg);
#endif

    /* Power and DC current estimation */
#if (defined POWER_DCCUR_EST_ENABLE)
    PowerEst_Init(&g_powerEst);
    DcCurEst_Init(&g_dcCurEst);
#endif

    /* DC bus current limit */
#if (defined DCCUR_LIMIT_ENABLE)
    DcCurLimit_Init(&g_dcCurLimit);
#endif

    /* Power limit */
#if (defined POWER_LIMIT_ENABLE)
    PowerLimit_Init(&g_powerLimit);
#endif

    /* Flux weakening */
#if (defined FLUX_WEAKENING)
    Fluxweak_Init(&g_pidFw, &g_fluxWeak, pFluxWeakCfg);
#endif

    /* MTPA, maximum torque per ampere */
#if (defined MTPA_ENABLE)
    Mtpa_ParamInit(&g_thetaMtpaComp, &g_motorParamPu);
#endif
    /* Hall FOC */
#if (defined HALL_SENSORS)
    HallSensor_Init(GetHallInitStatus(PWM_MODULER_HALL), &g_hall, &g_focVars);
#endif
    /* Hall + Encoder FOC */
#if (defined ENCODER_HALL_SENSORS)
    HallSensor_Init(GetHallInitStatus(PWM_MODULER_HALL), &g_hall, &g_focVars);
    EncoderHall_Init(&g_encoderHallSensor, &g_hallSensor, pMotorParam);
#endif

    /* Motor parameters identification */
    if (g_paramIdentifyCtrl.initCtrl == 1)
    {
        ParamIdentify_Init(&g_paramIdentify);
#if (defined FOC_PMSM)
//        PmsmId_LdqParamInit(&g_paramIdentify, &g_sigPhaseShift);
#elif (defined FOC_ACIM)
        AcimId_ParamInit(&g_paramIdentify, &g_sigPhaseShift);
#endif
    }
    StatorResistanceACRInit(&g_pidAcrAlfaRsId, &g_pidAcrBetaRsId);
#if (defined FOC_PMSM)
//    PmsmId_LdqParamInitValueSet(&g_paramIdentify, &g_STC, &g_sigPhaseShift);
#elif (defined FOC_ACIM)
    AcimId_ParamInitValueSet(&g_paramIdentify, &g_sigPhaseShift, &g_STC);
#endif

#if (defined TAIL_HEAD_WIND)
    InitSpeed_Judge_Init(&g_initSpeedJudge);
#endif

#ifdef OVER_VOL_SUPPRESS
    OverVoltageSuppressInit(&g_overVoltSupp, &g_pidUdcLimit);
#endif

#if (defined LOAD_TORQUE_ESTIMATE)
    TrqLoadEst_Init(&g_loadTorqueEst, pLoadTorqueCfg, &g_motorParamPu);
#endif

#if (defined INERTIA_ONLINE_IDENT)
    InertiaIdentify_Init(&g_inertiaIdentify, pInertiaIdentCfg, pMotorParam);
#endif

#if (defined INIT_POS_DETEC)
    InitPosDetec_Init(&g_initPosDetecStruct, &g_initPosPolar);
#endif

    g_hallSelfLearnFinishFlag = 0;

    g_flashTaskFinish = 0;
    g_flashOperationStatus = STATUS_UNSUPPORTED;
}

/*!
* @brief Foc control initialization, include control parameters and peripheral initialization.
*
* @param[in] none
* @return none
*/
void MC_Initialize(void)
{
#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
    FOC_Hall_Initialize();
#endif

    #if (defined FOC_SINE_WAVE)
    FOC_Parameters_Initialize();
    #elif (defined VF_CONTROL)
    Vf_ParamInit();
    #endif
    FOC_Peripheral_Initialize();
}

/*!
* @brief LED1 light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED1Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOD, 15U, ledStatus);
}

/*!
* @brief LED2 light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED2Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOE, 3U, ledStatus);
}

/*!
* @brief Torque control with speed limit function.
*
* @param[in] torque: pointer to TORQUE_COMMAND structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] asr: pointer to speed PID_REGULATOR structure
* @param[in] acr: pointer to q axis current PID_REGULATOR structure
* @return none
*/
void TorqueCtrlMode(TORQUE_COMMAND *torque, SPEED_RAMP_HANDLE *speed, PID_REGULATOR *asr, PID_REGULATOR *acr)
{
    if (defFocVars.controlMode == SPEED_CONTROL_MODE)
    {
        torque->speedFlag = 1;
        if (torque->torqueFlag == 1)
        {
            torque->torqueFlag = 0;
            TroqueCmdSet(defPID.maxSpeed, asr->pPidInput);
            speed->speedTarget = asr->pPidInput->fbkPu;
            speed->speedTargetRamp = asr->pPidInput->fbkPu;
            asr->pPidOutput->uiPu = acr->pPidInput->refPu;
        }
        else
        {
        }
    }
    else if (defFocVars.controlMode == TORQUE_CONTROL_WITH_SPDLIMIT)
    {
        torque->torqueFlag = 1;
        if (torque->speedFlag == 0)
        {
            speed->speedTarget = torque->speedLimit * pFocVarsCfg->motorDir;
            torque->iqTarget = MathFloat_Sat(torque->iqTarget, torque->iqMax, torque->iqMin);
            GIVE_SLOPE(torque->iqTarget, torque->iqTargetRamp, torque->iqChangeStep);
            torque->iqTargetRamp = MathFloat_Sat(torque->iqTargetRamp, torque->iqMax, torque->iqMin);
            TroqueCmdSet(torque->iqTargetRamp, asr->pPidInput);
        }
        else
        {
            torque->speedFlag = 0;
            torque->iqTarget = asr->pPidOutput->outPu;
            torque->iqTargetRamp = asr->pPidOutput->outPu;
            acr->pPidInput->refPu = asr->pPidOutput->outPu;
        }
    }
    else
    {
    }
}

/*!
* @brief Inertia identify calculation function of PMSM.
*
* @param[in] none
* @return none
*/
void InertiaIdentify_Calc(void)
{
#if (defined SENSORLESS)
    if (g_startUpCtrl.startUpFlag >= 6)
    {
        #if (defined INERTIA_IDENT_MRAS)
        InertiaIdentify_MRAS(&g_inertiaIdentify, &g_motorParamPu, g_asrSpeedEstPIOut.outPu, g_focVarsCtrl.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify, &g_motorParamPu, g_asrSpeedEstPIOut.outPu, g_focVarsCtrl.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS */
    }
#elif (defined HALL_SENSORS)
    if (Get_McStatus() == RUN)
    {
        #if (defined INERTIA_IDENT_MRAS)
        InertiaIdentify_MRAS(&g_inertiaIdentify, &g_motorParamPu, g_hallSensor.estFre, g_focVarsCtrl.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify, &g_motorParamPu, g_hallSensor.estFre, g_focVarsCtrl.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS */
    }
#elif ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
    if (Get_McStatus() == RUN)
    {
        #if (defined INERTIA_IDENT_MRAS)
        InertiaIdentify_MRAS(&g_inertiaIdentify, &g_motorParamPu, g_rotationalSpeed.estSpeed, g_focVarsCtrl.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify, &g_motorParamPu, g_rotationalSpeed.estSpeed, g_focVarsCtrl.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS */
    }
#else
#endif  /* end of SENSORLESS */

}

/*!
* @brief The Main state machine, used to set the running state of the control system.
*
* @param[in] none
* @return none
*/
void MainStateMachine(void)
{
    PullOut_McBufStatus();
    switch (Get_McStatus())
    {
    /* Power on, jump to STOP after completion */
    case POWER_ON:
        Stop_Motor();
        FOC_Parameters_Initialize();
    //    ASR_SpeedCommandInit(pFocVarsCfg, &g_speedCommand);
        ACR_TorqueCommandInit(&g_torqueCommand);
        ASR_SpeedCommandSet(pFocVarsCfg, &g_speedCommand);
#if (defined DATA_ACCESS_FLASH_ENABLE)
        DisableInterrupts;
        g_flashOperationStatus = Eflash_DataAccess_DefaultSave(&g_dataAccessFlash, g_VarAccessArray);
        if (g_flashOperationStatus == STATUS_SUCCESS)
        {
            Eflash_DataAccess_PowerOnRecovery(&g_dataAccessFlash, g_VarAccessArray);
            Eflash_DataAccessEflashInit(&g_dataAccessFlash);
            Set_McStatus(IDLE);
            EnableInterrupts;
        }
#else
        Set_McStatus(IDLE);
#endif

        /* must behind adc init, check current sampling offset*/
        Current_Sensor_Check(&g_adcSampleReal);
    break;

    /* Idle, system standby */
    case IDLE:
        FOC_Parameters_Initialize();
        /* check current sampling offset*/
        Current_Sensor_Check(&g_adcSampleReal);

#if (defined DATA_ACCESS_FLASH_ENABLE)
        if (g_dataAccessFlash.operateEnable == 1)
        {
            Set_McStatus(FLASH_TASK);
        }
#endif

#if (defined STARTUP_CHECK_RESTART_ENABLE)
        Foc_RestartInit(&g_restart);
#else
#endif

    #if (defined SENSORLESS)
        #if (defined HIGH_FREQUENCE_INJECTION)
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (HFI_INIT_FREQ * pFocVarsCfg->motorDir));
        #else
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ) * pFocVarsCfg->motorDir);
        #endif
    #endif
    break;

    /* Start brake, Short circuit brake */
    case START_BRAKE:
        ShortBrake_RunCaseBrake(&g_shortBrake, pShortBrakeCfg, pFocVarsCfg);
        if (g_shortBrake.subStatus == (uint8_t)CONFIG_RESET)
        {
            PWM_Disable();
            ShortBrake_Init(&g_shortBrake);
            Set_McStatus(START);
        }
    break;

    /* Start, according to the configuration, different startup schemes are implemented */
    case START:
#if (defined FOC_PMSM)
    #if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
//    ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, g_hallSensor.estFre);
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (0.05f * pFocVarsCfg->motorDir));
        Set_McStatus(RUN);
        StartUp_Motor();
    #endif

    #if ((defined SENSORLESS) || (defined ENCODER_SENSORS))
        if (g_startUp.pStCtrl->alignState)
        {
            Set_McStatus(RUN);
            StartUp_Motor();
        }
        else
        {
        #if (defined SENSORLESS)
            #if (defined PULSE_INJECTION)
            Set_McStatus(ANGLE_JUDGE);
            #elif (defined INIT_POS_DETEC)
            Set_McStatus(INIT_POS);
            #else
            StartUp_Motor();
            Set_McStatus(ALIGN);
            #endif
        #endif
        #if (defined ENCODER_SENSORS)
            StartUp_Motor();
            Set_McStatus(ALIGN);
        #endif
        }
    #endif
#elif (defined FOC_ACIM)
        StartUp_Motor();
        Set_McStatus(ALIGN);
#endif
    break;

    #if (defined PULSE_INJECTION)
    case ANGLE_JUDGE:
        if (EXE_SUCCESS == PulseInject_Process(&g_pulseInject, &g_startUpCtrl))
        {
            StartUp_Motor();
            Set_McStatus(RUN);
        }
        else if (EXE_FAIL == PulseInject_Process(&g_pulseInject, &g_startUpCtrl))
        {
            StartUp_Motor();
            Set_McStatus(ALIGN);
        }
        else
        {
        }
    break;
    #endif

    #if (defined TAIL_HEAD_WIND)
    case SPEED_JUDGE:
        if (Get_InitSpeed_Judge_State(&g_initSpeedJudge))
        {
            Set_McStatus(FLY_RUN);
            Clear_InitSpeed_Judge_State(&g_initSpeedJudge);
        }
        break;

        case FLY_RUN:
        Tail_Head_Wind_Process(&g_speedCommand, &g_initSpeedJudge);
    break;
    #endif

    /* Align, obtain the initial position of rotor */
    case ALIGN:
#if (defined FOC_PMSM)
#if (defined ENCODER_SENSORS)
        Angle_Prepositioning_Process(&g_focVarsCtrl, &g_startUp);
        if (g_startUp.pStCtrl->alignState)
        {
            Encoder_GetAlignOffset(&g_encoderSensor);
            Set_McStatus(RUN);
        }
#else

    #if (defined HIGH_FREQUENCE_INJECTION)
        Set_McStatus(RUN);
    #else
        #if (defined ALIGNMENT_TWICE)
        Angle_Alignment_Process(&g_focVarsCtrl, &g_startUp);
        #endif
        #if (defined ALIGNMENT_ONCE)
        Angle_Prepositioning_Process(&g_focVarsCtrl, &g_startUp);
        #endif
        if (g_startUp.pStCtrl->alignState)
        {
            Set_McStatus(RUN);
        }
    #endif
#endif
#elif (defined FOC_ACIM)
        if (EXE_SUCCESS == Acim_FocPreExcitStartup(&g_acimFocCtrl, &g_focVars, 1000))
        {
            Set_McStatus(RUN);
        }
#endif
    break;

#if (defined INIT_POS_DETEC)
    case INIT_POS:
        if (InitPosDetec_1msTask(&g_initPosDetecStruct, pInitPosDetecCfg, &g_STC))
        {
            /* Open loop startup: id = 0, set iqref, startup angle is rotor angle - pi/2 */
            if (g_focVars.pCfg->motorDir == 1)
            {
                g_startUpCtrl.startUpElecTheta = g_initPosDetecStruct.initPosDetec->estTheta - 0.25f;
            }
            else
            {
                g_startUpCtrl.startUpElecTheta = g_initPosDetecStruct.initPosDetec->estTheta + 0.25f;
            }
            StartUp_Motor();
            Set_McStatus(RUN);
        }
    break;
#endif

    /* Run, according to the configuration, different control schemes are implemented */
    case RUN:
#if (defined FOC_PMSM)
#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
        #if (!defined POSITION_LOOP_CTRL)
            ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
        #endif
        }
#elif (defined SENSORLESS)
    #if (defined IF_STARTUP) || (defined DIRECT_STARTUP)|| (defined VF_STARTUP)
        StartUp_Ctrl_1ms(&g_focVars, &g_startUp);
    #endif
//    Acr_GetAvgIs(&g_focVarsCtrl);
#else
    #if (!defined POSITION_LOOP_CTRL)
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
    #endif
#endif

#if (defined ELEC_BRAKE_ENABLE)
        ElecBrake_Calc(g_speedCommand.speedFbk, &g_elecBrake, pElecBrakeCfg, &g_STC, &g_acrCurIdPIOut, &g_acrCurIqPIOut);
#endif

#if (defined FLUX_WEAKENING)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            g_fluxWeak.vdq = g_focVarsCtrl.vdqFilter;
            Fluxweak_CurRefCalc(&g_fluxWeak, &g_focVarsCtrl, &g_pidFw, &g_asrSpeedPIOut);
        }
//    Fluxweak_CurRef_Lookup(g_asrSpeedPIIn.refPu, g_asrSpeedPIOut.outPu, &g_focVarsCtrl, &g_fluxWeak);
#endif

#if (defined MTPA_ENABLE)
        Mtpa_Compensator(g_focVarsCtrl.ialphabeta, &g_thetaMtpaComp);
#endif

#if (defined SAVE_ENERGY)
    #if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        g_focVarsCtrl.idRef = EnergySave(&g_energySave, pEnergySaveCfg);
    #elif (defined SENSORLESS)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            g_focVarsCtrl.idRef = EnergySave(&g_energySave, pEnergySaveCfg);
        }
    #endif
#endif

#if (defined STARTUP_CHECK_RESTART_ENABLE)
        Foc_RestartJudgeProcess(&g_restart, &g_speedCommand);
#endif

#if (defined OVER_LOAD_PROTECT_ENABLE)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            Is_OverLoad_FallFreq(&g_overLoadPro, &g_focVarsCtrl, &g_speedCommand);
        }
#endif
        TorqueCtrlMode(&g_torqueCommand, &g_speedCommand, &g_pidAsr, &g_pidAcrQ);
#elif (defined FOC_ACIM)
    #if (defined SENSOR_ACIM)
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
    #elif (defined SENSORLESS_ACIM)
        Acim_Startup_1ms(&g_focVars, &g_startUp);
    #else
    #endif
    #if (defined ELEC_BRAKE_ENABLE)
        ElecBrake_Calc(g_speedCommand.speedFbk, &g_elecBrake, pElecBrakeCfg, &g_STC, &g_acrCurIdPIOut, &g_acrCurIqPIOut);
    #endif
        TorqueCtrlMode(&g_torqueCommand, &g_speedCommand, &g_pidAsr, &g_pidAcrQ);
#endif

#if (defined SPEED_ADJUST_KNOB)
        ASR_GetSpeedRampWithKnob(&g_speedCommand);
#endif
    break;

    /* Slow_Down, decelerate the motor to a certain speed and then stop the motor */
    case SLOW_DOWN:
        g_speedCommand.speedMin = pFocVarsCfg->stopSpeed;
        ASR_SetSpeedTarget_Pu(&g_speedCommand, (pFocVarsCfg->stopSpeed * pFocVarsCfg->motorDir));

#if (defined FOC_PMSM)
    #if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
    #elif (defined SENSORLESS)
        StartUp_Ctrl_1ms(&g_focVars, &g_startUp);
    #else
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
    #endif

    #ifdef OVER_VOL_SUPPRESS
        OverVoltageSuppressProcess(g_focVarsCtrl.vdcPu, &g_overVoltSupp, &g_pidUdcLimit, &g_asrSpeedPIIn);
    #endif

#elif (defined FOC_ACIM)
    #if (defined SENSOR_ACIM)
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
    #elif (defined SENSORLESS_ACIM)
        Acim_Startup_1ms(&g_focVars, &g_startUp);
    #else
    #endif
#endif

#if (defined OPEN_LOOP)
        if (MathFloat_Abs(g_speedCommand.speedTargetRamp) <= pFocVarsCfg->stopSpeed)
        {
            Set_McStatus(STOP);
            PWM_Disable();
        }
#elif (defined CLOSE_LOOP)
        if (MathFloat_Abs(g_speedCommand.speedFbk) <= pFocVarsCfg->stopSpeed)
        {
            Set_McStatus(STOP);
            PWM_Disable();
        }
#else
#endif
    break;

    /* Stop brake, Short circuit brake */
    case STOP_BRAKE:
        ShortBrake_RunCaseBrake(&g_shortBrake, pShortBrakeCfg, pFocVarsCfg);
        if (g_shortBrake.subStatus == (uint8_t)CONFIG_RESET)
        {
            PWM_Disable();
            ShortBrake_Init(&g_shortBrake);
            Set_McStatus(STOP);
        }
    break;

    /* Stop, stop the motor and clear the control parameters, then go to Idle state */
    case STOP:
        Stop_Motor();
        FOC_Parameters_Initialize();
//    ASR_SpeedCommandInit(pFocVarsCfg, &g_speedCommand);
        ACR_TorqueCommandInit(&g_torqueCommand);
        ASR_SpeedCommandSet(pFocVarsCfg, &g_speedCommand);
        PWM_Update(PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1);
        Set_McStatus(IDLE);
    break;

    /* Identify, perform motor parameter identification function */
    case IDENTIFY:
        ParamIdentify(&g_paramIdentify, &g_STC, pHWBoardParam);
        if (g_paramIdentifyCtrl.initCtrl == 1)
        {
            ParameterIdentifySaveProcess(&g_paramIdentify, &g_paramIdentifyBuffer);
            Set_McStatus(IDLE);
        }
    break;

    /* Fault, stop the motor. If it is expected to run again, it is necessary to clear the fault and enter the stop state */
    case FAULT:
        Stop_Motor();
        if (g_protector.ErrFlag.all == 0)
        {
//        Set_McStatus(STOP);
        }
    break;

    /* Restart, re-do FOC start. when motor start process is abnormal, restart the motor */
#if (defined STARTUP_CHECK_RESTART_ENABLE)
    case RESTART:
        Foc_RestartProcess(&g_restart);
        Set_McStatus(START);
    break;
#endif

    /* flash task, do flash write operation to save user debug value of variables, and flash read operation to recovery flash data to variables */
#if (defined DATA_ACCESS_FLASH_ENABLE)
    case FLASH_TASK:
        DisableInterrupts;
        g_flashOperationStatus = Eflash_DataAccessEflashProcess(&g_dataAccessFlash, g_VarAccessArray);
        if (g_flashOperationStatus == STATUS_SUCCESS)
        {
            Set_McStatus(IDLE);

            Eflash_DataAccessEflashInit(&g_dataAccessFlash);
            g_flashTaskFinish = 1;
            EnableInterrupts;
        }
    break;
#endif

    /* Hall sensor self learn, do self learning to get Hall sensor installation information */
    case HALL_SELFLEARN:
        StartUp_Motor();
        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
            Stop_Motor();
            Set_McStatus(STOP);
            HallSelfLearnResultInit(&g_hallSelfLearnBuffer);
        }
        if (g_hallSelfLearnFinishFlag == 1)
        {
            HallSelfLearnSaveProcess(&g_hallSelfLearnBuffer, &g_hall);
            defHallSensor.hallSelfLearnEnable = DISABLE;
            Stop_Motor();
            Set_McStatus(STOP);
        }
    break;

    default:
    break;
    }
}

/*!
* @brief Foc task in 1 ms time base, do speed loop, speed ramp, etc. FOC 1ms task (sensorless, hall, encoder all in this), callback in task_scheduler.
*
* @param[in] none
* @return none
*/
void Motor_1ms_Task(void)
{
#if (defined VF_CONTROL)
    VF_1ms_Task();
#elif (defined FOC_SINE_WAVE)
    #if (defined POWER_DCCUR_EST_ENABLE)
    /* g_powerEst.powerInTrue means input power true value, increase 100 times */
    PowerEst_Calc(&g_STC, pPowerEstCfg, g_asrSpeedPIIn.fbkPu, g_dcCurEst.dcCurTrue, &g_powerEst);
    /* g_dcCurEst.dcCurTrue means bus current true value, increase 100 times */
    DcCurEst_Calc(&g_motorParamPu, &g_focVarsCtrl, &g_pwmValue, 100, &g_dcCurEst);
    #endif

    MainStateMachine();

    Get_VoltageSample(&g_adcSampleCof, &g_motorParamPu, &g_adcTransform);
//    /* Start adc convert */
//    ADC_DRV_SoftwareStartRegularConvert(ADC0_MODULER);

    Motor_Protection();

    ASR_SpeedTargetLimitProcess(&g_speedCommand);

    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
    {
    #if (defined FOC_ACIM)
        Acim_AsrPiCoefUpdate();
    #elif (defined FOC_PMSM)
        #if (defined ASR_VARPI_ENABLE)
        ASR_SpeedVarPICalc(g_speedCommand.speedFbk, &g_asrSpeedVarPI, &g_asrSpeedCof);
        #endif

        #if (defined SENSORLESS)
            #if ((defined ATAN_ANGLE_ESTIMATION) && (defined ELEC_FREQUENCE_LOW))
        Asr_EstSpeedCalc(g_startUp.pStCtrl->elecThetaEst, &g_rotationalSpeed);
            #endif
        #elif ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
            #if ((!defined MT_SPEED_MEASURE) && (defined ELEC_FREQUENCE_LOW))
        Asr_EstSpeedCalc(g_focVarsCtrl.elecAngle, &g_rotationalSpeed);
            #endif
        #else
        #endif
    #endif
        ASR_SpeedFbk_Update(&g_speedCommand);

    #if (defined SCURVE_SPEED_RAMP)
    /* DO NOTHING IN 1MS TASK WHILE SELECT S_CURVE RAMP FUNCTION*/
    #else
        #if (defined SENSORLESS)
        if (g_startUp.pStCtrl->startUpFlag == 6)
        {
            ASR_SpeedTargetRampCalc(pFocVarsCfg, &g_speedCommand);
        }
        #else
        if (g_focVarsCtrl.pwmOutMask == SVPWM_OUTPUT_ENABLE)
        {
            ASR_SpeedTargetRampCalc(pFocVarsCfg, &g_speedCommand);
        }
        #endif
    #endif

        ASR_AvgSpeedCalc(&g_speedCommand);
    }

#else
#endif
}

/*!
* @brief Foc task in 2 ms time base. If TOOL_UART_FUNCTION macro is defined, the UART function with host PC tool is executed in 2ms time base task.
*
* @param[in] none
* @return none
*/
void Motor_2ms_Task(void)
{
#if (defined VF_CONTROL)
    VF_2ms_Task();
#elif (defined FOC_SINE_WAVE)
    #if (defined TOOL_UART_FUNCTION)
    /* Motor Debug Tool command Process */
    UI_UART_ToolCmd_Proc(&g_uiRxBuf);
    UI_UART_TimeOut_Cnt();
    UI_MasterCmd_ProcCnt();
    #endif

    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
    {
    #if (defined SCURVE_SPEED_RAMP)
        #if (defined SENSORLESS)
        if (g_startUp.pStCtrl->startUpFlag == 6)
        {
            SCurve_MainProcess(&g_sCurveData, &g_speedCommand);
        }
        #else
        if (g_focVarsCtrl.pwmOutMask == SVPWM_OUTPUT_ENABLE)
        {
            SCurve_MainProcess(&g_sCurveData, &g_speedCommand);
        }
        #endif
    #endif
    }
#else
#endif
}

/*!
* @brief Foc task in 4 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_4ms_Task(void)
{
    if (Get_McStatus() == IDENTIFY)
    {
        ParamIdentify_4msTask(&g_paramIdentify, &g_STC);
    }
}

/*!
* @brief Foc task in 6 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_6ms_Task(void)
{

}

/*!
* @brief Foc task in 10 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_10ms_Task(void)
{
#if (defined POSITION_LOOP_CTRL)
    PositionCommandUpdate(&g_positionCommand);
    APR_PositionTargetRampCalc(&g_positionCommand);

    APR_PositionLoop_Process(&g_positionCommand, &g_pidApr, &g_focVarsCtrl);
#endif

#if (defined INERTIA_ONLINE_IDENT)
    InertiaIdentify_Calc();
#endif

#if (defined INIT_POS_DETEC)
    if (Get_McStatus() == INIT_POS)
    {
        InitPosDetec_LongPeriodTask(&g_initPosDetecStruct, &g_STC);
    }
#endif

#if (defined MT_SPEED_MEASURE)
    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
    {
    Encoder_MT_SpeedCalc(&g_encoderSpeed);
    }
#endif
}

/*!
* @brief Foc task in 2 s time base.
*
* @param[in] none
* @return none
*/
void FOC_2000ms_Task(void)
{
    if (g_protector.ErrFlag.all)
    {
//        printf(" g_protector.ErrFlag.all = 0x%x \r\n", g_protector.ErrFlag.all);
    }
}

/* =============================================  EOF  ============================================== */
