/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file pwmdrv.c
*
* @brief This file provides PWM module functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_output.h"
#include "pwmdrv.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Disable PWM module output pulse.
*
* @param[in] none
* @return none
*/
void PWM_Disable(void)
{
    PWM_Output(PWM_OUTPUT_DISABLE);
}

/*!
* @brief Enable PWM module output pulse.
*
* @param[in] none
* @return none
*/
void PWM_Enable(void)
{
    PWM_Output(PWM_OUTPUT_ENABLE);
}

/*!
* @brief Enable or disable PWM channel output according to the input mark.
*
* @param[in] enable: Enable mark, 'enable' = DISABLE = 0, will mask the PWM channel output; 'enable' = ENABLE = 1, will make the PWM channel output.
* @return none
*/
void PWM_Output(uint8_t enable)
{
    if (enable)
    {
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_0, DISABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_1, DISABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_2, DISABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_3, DISABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_4, DISABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_5, DISABLE);
    }
    else
    {
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_0, ENABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_1, ENABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_2, ENABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_3, ENABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_4, ENABLE);
        PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_5, ENABLE);
    }
}

/*!
* @brief Load the duty cycle of three-phase inverter calculated by SVPWM algorithm into PWM module.
*
* @param[in] Ta: duty of phase A upper bridge arm
* @param[in] Tb: duty of phase B upper bridge arm
* @param[in] Tc: duty of phase C upper bridge arm
* @return none
*/
void PWM_Update(uint16_t Ta, uint16_t Tb, uint16_t Tc)
{
#if (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    if ((Tc > Ta) && (Tc > Tb))
    {
        PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_6, ((g_focVars.pCfg->pwmPeriod - Tc) >> 1));
    }
    else if ((Ta > Tb) && (Ta > Tc))
    {
        PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_6, ((g_focVars.pCfg->pwmPeriod - Ta) >> 1));
    }
    else
    {
        /* ((Tb > Ta) && (Tb > Tc)) */
        PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_6, ((g_focVars.pCfg->pwmPeriod - Tb) >> 1));
    }
#endif

//    if (Ta == 0)
//    {
//        Ta = g_focVars.pCfg->pwmPeriod;
//    }
//    else if (Ta == g_focVars.pCfg->pwmPeriod)
//    {
//        Ta = 0;
//    }
//    else{}

//    if (Tb == 0)
//    {
//        Tb = g_focVars.pCfg->pwmPeriod;
//    }
//    else if (Tb == g_focVars.pCfg->pwmPeriod)
//    {
//        Tb = 0;
//    }
//    else{}

//    if (Tc == 0)
//    {
//        Tc = g_focVars.pCfg->pwmPeriod;
//    }
//    else if (Tc == g_focVars.pCfg->pwmPeriod)
//    {
//        Tc = 0;
//    }
//    else
//    {
//    }

#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, (g_focVars.pCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, (g_focVars.pCfg->pwmPeriod - Tc));

    #else
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, ((g_focVars.pCfg->pwmPeriod - Ta) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, ((g_focVars.pCfg->pwmPeriod + Ta) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, ((g_focVars.pCfg->pwmPeriod - Tb) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, ((g_focVars.pCfg->pwmPeriod + Tb) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, ((g_focVars.pCfg->pwmPeriod - Tc) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, ((g_focVars.pCfg->pwmPeriod + Tc) >> 1));

    #endif

    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, Ta);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, Ta);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, Tb);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, Tb);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, Tc);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, Tc);

    #else
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, (Ta >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, (g_focVars.pCfg->pwmPeriod - (Ta >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, (Tb >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, (g_focVars.pCfg->pwmPeriod - (Tb >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, (Tc >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, (g_focVars.pCfg->pwmPeriod - (Tc >> 1));

    #endif
    #endif

#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, (g_focVars.pCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, (g_focVars.pCfg->pwmPeriod - Tc));

#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, (g_focVars.pCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, (g_focVars.pCfg->pwmPeriod - Tc));

#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, (g_focVars.pCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, (g_focVars.pCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, (g_focVars.pCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, (g_focVars.pCfg->pwmPeriod - Tc));

    #else
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, ((g_focVars.pCfg->pwmPeriod - Ta) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, ((g_focVars.pCfg->pwmPeriod + Ta) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, ((g_focVars.pCfg->pwmPeriod - Tb) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, ((g_focVars.pCfg->pwmPeriod + Tb) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, ((g_focVars.pCfg->pwmPeriod - Tc) >> 1));
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, ((g_focVars.pCfg->pwmPeriod + Tc) >> 1));

    #endif
#endif

    PWM_DRV_SetSoftwareTriggerSync(PWM_MODULER_MOTOR1, ENABLE);
}

/*!
* @brief Load the duty cycle of three-phase inverter calculated by SVPWM
*        algorithm into PWM module when single shunt sampling is used.
*
* @param[in] curRestruct: pointer to CUR_RESTRUCT_DATA structure
* @return none
*/
void PWM_SingleShuntUpdate(CUR_RESTRUCT_DATA *curRestruct)
{
    /* if compare value equle to 1, will resul to pluse abnormal at high speed */
    curRestruct->pwmTaUp = MathFloat_Sat(curRestruct->pwmTaUp, (g_focVars.pCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTbUp = MathFloat_Sat(curRestruct->pwmTbUp, (g_focVars.pCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTcUp = MathFloat_Sat(curRestruct->pwmTcUp, (g_focVars.pCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTaDown = MathFloat_Sat(curRestruct->pwmTaDown, (g_focVars.pCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTbDown = MathFloat_Sat(curRestruct->pwmTbDown, (g_focVars.pCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTcDown = MathFloat_Sat(curRestruct->pwmTcDown, (g_focVars.pCfg->pwmPeriod - 1), 2);
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_0, curRestruct->pwmTaUp);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_1, curRestruct->pwmTaDown);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_2, curRestruct->pwmTbUp);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_3, curRestruct->pwmTbDown);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_4, curRestruct->pwmTcUp);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_5, curRestruct->pwmTcDown);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_6, curRestruct->maxMidTrig);
    PWM_DRV_SetChannelCountValue(PWM_MODULER_MOTOR1, PWM_CH_7, curRestruct->midMinTrig);

    #else

    #endif
    PWM_DRV_SetSoftwareTriggerSync(PWM_MODULER_MOTOR1, ENABLE);
}

/*!
* @brief Turn on three phase lower MOSFET or IGBT to make motor winding in short circuit control.
*
* @param[in] none
* @return none
*/
void PWM_ShortCircuitOn(void)
{
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_0, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_1, DISABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_2, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_3, DISABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_4, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_5, DISABLE);
}

/*!
* @brief Turn off three phase lower MOSFET or IGBT.
*
* @param[in] none
* @return none
*/
void PWM_ShortCircuitOff(void)
{
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_0, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_1, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_2, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_3, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_4, ENABLE);
    PWM_DRV_SetChannelOutputMask(PWM_MODULER_MOTOR1, PWM_CH_5, ENABLE);
}

/* =============================================  EOF  ============================================== */
