/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __MAX_TORQUE_PER_AMPERE_H__
#define __MAX_TORQUE_PER_AMPERE_H__

/*!
* @file maxtorque_per_ampere.h
*
* @brief This file provides the current vector angle by formula calculation method functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Mtpa theta compensation parameter structure.
*/
typedef struct
{
    float fr;             /*!< Magnetic flux */
    float frSq;           /*!< Square of magnetic flux */
    float deltaL;         /*!< Delta inductance */
    float deltaLSq;       /*!< Square of Delta inductance */
    float coeffPu;        /*!< Mtpa compensation coeff for pu */
    float thetadGain;     /*!< Mtpa compensation angle gain */
    float thetad;         /*!< Mtpa compensation angle */
} MTPA_THETA_COMPENSATOR;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Maximum torque per ampere control parameter initialize, usually it is initialized only once before running.
*
* @param[in] mtpaThetaComp: pointer to MTPA_THETA_COMPENSATOR structure
* @param[in] motorParam: pointer to MOTOR_PARAM_PU structure
* @return none
*/
void Mtpa_ParamInit(MTPA_THETA_COMPENSATOR *mtpaThetaComp, MOTOR_PARAM_PU *motorParam);

/*!
* @brief Maximum torque per ampere control parameter calculate.
*
* @param[in] mtpaThetaComp: pointer to MTPA_THETA_COMPENSATOR structure
* @param[in] motorParam: pointer to MOTOR_PARAM_PU structure
* @return none
*/
void Mtpa_ParamCalc(MTPA_THETA_COMPENSATOR *mtpaThetaComp, MOTOR_PARAM_PU *motorParam);

/*!
* @brief Calculate maximum torque per ampere control current vector angle by formula calculation method,
*        according to the vector angle to get the given value of d-q axis current.
*
* @param[in] IS: vector current
* @param[in] mtpaThetaComp: pointer to MTPA_THETA_COMPENSATOR structure
* @return none
*/
void Mtpa_Compensator(float IS, MTPA_THETA_COMPENSATOR *mtpaThetaComp);

/* ======================================  Functions define  ======================================== */

#endif /* __MAX_TORQUE_PER_AMPERE_H__ */

/* =============================================  EOF  ============================================== */
