/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __MRAS_H__
#define __MRAS_H__
/*!
* @file mras.h
*
* @brief This file provides the rotar angle and speed estimated by the mras observer for sensorless vector control functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "autospd_regulator.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief MRAS sensorless mode control parameters structure.
*/
typedef struct
{
    float estTheta;      /*!< Estimated electrical theta by MRAS observer */
    float estFreq;        /*!< Estimated electrical frequency by MRAS observer */
    float estFreqBuf;     /*!< Estimated electrical frequency by MRAS observer */
    float estFreqFilter;  /*!< Estimated electrical frequency by MRAS observer */
    float fMrasPos;       /*!< Model reference adjust system fsmopos coefficient */
    float gMrasPos;       /*!< Model reference adjust system gsmopos coefficient */
    float hMrasPos;       /*!< Model reference adjust system hsmopos coefficient */
    float cofAlpha;       /*!< Model reference adjust system alpha coefficient */
    float cofBeta;        /*!< Model reference adjust system beta coefficient */
    float estId;          /*!< Estimated d axis current */
    float fdbIdFilter;    /*!< Feedback d axis current filter */
    float estIq;          /*!< Estimated q axis current */
    float fdbIqFilter;    /*!< Feedback q axis current filter */
    float vdFilter;       /*!< D axis voltage filter */
    float vqFilter;       /*!< Q axis voltage filter */
} MRAS_OBSERVER;

/*!
* @brief MRAS configuration parameter structure.
*/
typedef struct
{
    float cofAlpha;       /*!< Model reference adjust system alpha coefficient */
    float cofBeta;        /*!< Model reference adjust system beta coefficient */
} MRAS_OBSERVER_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief MRAS variables initialize, the LPF filter coeffieient and cutoff frequency need to be passed externally,
*        usually it is initialized only once before running.
*
* @param[in] mras: pointer to MRAS structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] mrasCfg: pointer to MRAS_OBSERVER_CFG structure
* @return none
*/
void Mras_Init(MRAS_OBSERVER *mras, MOTOR_PARAM_PU *paramPu, MRAS_OBSERVER_CFG *mrasCfg);

/*!
* @brief Calculate the rotar angle and speed estimated by the MRAS for sensorless vector control,
*
* @param[in] STC: pointer to STC_PARAM structure
* @param[in] mras: pointer to MRAS_OBSERVER structure
* @param[in] pidPll: pointer to PID_REGULATOR structure
* @param[in] startupStage: IF startup stage parameter
* @return motor electrical angle
*/
float Mras_PosEst(STC_PARAM *STC, MRAS_OBSERVER *mras, PID_REGULATOR *pidPll, uint16_t startupStage);

/* ======================================  Functions define  ======================================== */

#endif /* __MRAS_H__ */

/* =============================================  EOF  ============================================== */
