/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __VERSION_H__
#define __VERSION_H__
/*!
* @file version.h
*
* @brief his file provides MCU chip ID and library version functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>

/* ============================================  Define  ============================================ */
#define FOCLIB_MAIN_VER     1                   /*!< Main version number */
#define FOCLIB_SUB_VER      0                   /*!< sub-version number */
#define FOCLIB_REV_VER      5                   /*!< rev-version number */

#define FOCLIB_VER_YEAR     2023                /*!< Year of the current lib version released */
#define FOCLIB_VER_MONTH    4                   /*!< Month of the current lib version released */
#define FOCLIB_VER_DAY      12                  /*!< Date of the current lib version released */

#define MASTER                              0           /*!< Master */
#define MCU_TRUNK_AC7840X                   1           /*!< MCU foc lib code integer branch */
#define MCU_TRUNK_AC7840X_FLOAT             2           /*!< MCU foc lib code float branch */
#define MCU_TRUNK_AC7840X_DUALMOTOR         3           /*!< MCU foc lib code dual motor integer branch */
#define MCU_TRUNK_AC7840X_DUALMOTOR_FLOAT   4           /*!< MCU foc lib code dual motor float branch */

#define FOCLIB_BRANCH                       MCU_TRUNK_AC7840X_FLOAT    /*!< Code branch */

#define CHIP_ID                             0xAC78              /*!< MCU chip ID */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Version information structure of FOC library.
*/
typedef struct
{
    uint32_t main;                  /*!< Main-version */
    uint32_t sub;                   /*!< Sub-version */
    uint32_t rev;                   /*!< Rev-version */
    uint32_t year;                  /*!< Year of the current lib version released */
    uint32_t month;                 /*!< Month of the current lib version released */
    uint32_t day;                   /*!< Date of the current lib version released */
    uint32_t branch;                /*!< Code branch */
} FOCLIB_VERSION;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Get MCU chip ID.
*
* @param[in] none
* @return chipID
*/
uint32_t ChipIDGet(void);

/*!
* @brief Get focLib Version. Used to confirm the release date and
*        version number of the current librafy file.
*
* @param[in] none
* @return focLibVer: version information of the current foc lib file.
*/
FOCLIB_VERSION FocLibVersionGet(void);

/* ======================================  Functions define  ======================================== */

#endif /* __VERSION_H__ */

/* =============================================  EOF  ============================================== */
