/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __TASK_SCHEDULER_H__
#define __TASK_SCHEDULER_H__
/*!
* @file task_scheduler.h
*
* @brief This file provides task scheduling based on millisecond time base functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>

/* ============================================  Define  ============================================ */

#define SCH_TASKS_SEQUENCE_MAX          (6)     /*!< Maximum number of tasks allowed in task scheduling sequence */
#define ERROR_SCH_TOO_MANY_TASKS        (1)     /*!< Error code of task number overrun */
#define ERROR_SCH_CANNOT_DELETE_TASK    (2)     /*!< Error code of task cannot be deleted */

#define RETURN_NORMAL                   (0)     /*!< Flag of nornal task execution */
#define RETURN_ERROR                    (1)     /*!< Flag of task execution failure */

#define SCH_REPORT_ERRORS                       /*!< Allow task scheduling function to report errors */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Task scheduling function structure.
*/
typedef struct
{
    void (* pTask)(void);       /*!< Pointer to the task (must be a 'void (void)' function) */
    uint32_t delay;             /*!< delay (ticks) until the function will (next) be run. See SCH_Add_Task() for further details */
    uint32_t period;            /*!< Interval (ticks) between subsequent runs. see SCH_Add_Task() for further details */
    uint8_t runOrder;           /*!< Incremented (by scheduler) when task is due to execute */
} TASK_SCHEDULER;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Task scheduler update base on TIMER ISR.
*
* @param[in] none
* @return none
*/
void SCH_Update(void);

/*!
* @brief Task scheduler main processer, called by Main function.
*
* @param[in] none
* @return none
*/
void Task_Scheduler(void);

/* ======================================  Functions define  ======================================== */

#endif /* __TASK_SCHEDULER_H__ */

/* =============================================  EOF  ============================================== */
